#include "uucode.h"

/*****************************************************************************
E n c o d e                            
==============================================================================

This function encodes a byte stream into uu-code. An array of bytes for the
the result must be given.

Parameters
  pbByteStream     array of bytes to be converted into uu-code
  pbUUCodeStream   array for the resulting uu-code
  lNoBytes         number of bytes to be converted into uu-code

Return
  This function returns the number of the resulting uu-coded bytes.

*****************************************************************************/

long CUUCode::Encode(BYTE *pbByteStream, BYTE *pbUUCodeStream, long lNoBytes)
{
  long lIndex;
  long lNoCycles = lNoBytes / 3L;

  // calculate number of needed bytes for uu-code
  long lNoUUCode = (lNoBytes / 3L) * 4L;
  if (lNoBytes % 3L > 0) {
    lNoUUCode += lNoBytes % 3L + 1L;
  } // if

  

  mpbByteStream = pbByteStream;
  mpbUUCodeStream = pbUUCodeStream;

  // convert bytes into uu-code (3 bytes -> 4 uu-coded bytes)
  for (lIndex = 0; lIndex < lNoCycles; lIndex++) {
    Byte2UUCode(3);
	mpbByteStream += 3;
	mpbUUCodeStream += 4;
  } // for

  // if there is a rest of 1 or 2 bytes, convert them
  if (lNoBytes % 3L > 0) {
	Byte2UUCode((int)(lNoBytes % 3L));
  } // if

  return lNoUUCode;

} // Encode()



/*****************************************************************************
D e c o d e                            
==============================================================================

This function decodes a uu-code stream into a byte stream. An array of bytes
for the the result must be given.

Parameters
  pbUUCodeStream   array of uu-code to be convertedinto bytes
  pbByteStream     array for the resulting bytes
  lNoUUCode        number of uu-coded bytes to be converted into bytes

Return
  This function returns the number of the resulting bytes.

*****************************************************************************/

long CUUCode::Decode(BYTE *pbUUCodeStream, BYTE *pbByteStream, long lNoUUCode)
{
  long lIndex;
  long lNoCycles = lNoUUCode / 4L;

  // calculate number of needed bytes
  long lNoBytes = (lNoUUCode / 4L) * 3L;
  if (lNoUUCode % 4L > 0) {
    lNoBytes += lNoUUCode % 4L - 1L;
  } // if

  mpbByteStream = pbByteStream;
  mpbUUCodeStream = pbUUCodeStream;

  // convert uu-code into bytes (4 uu-coded bytes -> 3 bytes)
  for (lIndex = 0; lIndex < lNoCycles; lIndex++) {
    UUCode2Byte(4);
	mpbByteStream += 3;
	mpbUUCodeStream += 4;
  } // for

  // if there is a rest of 2 or 3 uu-coded bytes, convert them
  if (lNoUUCode % 4L > 0) {
	UUCode2Byte((int)(lNoUUCode % 4L));
  } // if

  return lNoBytes;

} // Decode()


  
/*****************************************************************************
B y t e 2 U U C o d e                  
==============================================================================

This function encodes 3 bytes to 4 uu-coded bytes. Both are stored in the
private members mabBytes and mabUUCode.

Parameters
  nNumber   Number of bytes to be converted (max. 3)

*****************************************************************************/

void CUUCode::Byte2UUCode(int nNumber)
{

	switch (nNumber) {
	  
	case 1: mpbUUCodeStream[0] = (mpbByteStream[0] & 0x3F) + OFFSET;
		    mpbUUCodeStream[1] = ((mpbByteStream[0] & 0xC0) >> 6) + OFFSET;
			break;

	case 2: mpbUUCodeStream[0] = (mpbByteStream[0] & 0x3F) + OFFSET;
		    mpbUUCodeStream[1] = ((mpbByteStream[0] & 0xC0) >> 6)
				               + ((mpbByteStream[1] & 0x0F) << 2) + OFFSET;
		    mpbUUCodeStream[2] = ((mpbByteStream[1] & 0xF0) >> 4) + OFFSET;
			break;

	case 3: mpbUUCodeStream[0] = (mpbByteStream[0] & 0x3F) + OFFSET;
		    mpbUUCodeStream[1] = ((mpbByteStream[0] & 0xC0) >> 6)
				               + ((mpbByteStream[1] & 0x0F) << 2) + OFFSET;
		    mpbUUCodeStream[2] = ((mpbByteStream[1] & 0xF0) >> 4)
			                   + ((mpbByteStream[2] & 0x03) << 4) + OFFSET;
		    mpbUUCodeStream[3] = ((mpbByteStream[2] & 0xFC) >> 2) + OFFSET;
			break;

	} // switch


} // Byte2UUCode()



/*****************************************************************************
U U C o d e 2 B y t e                  
==============================================================================

This function decodes 4 uu-coded bytes to 3 bytes. Both are stored in the
private members mabBytes and mabUUCode.

Parameters
  nNumber   Number of uu-coded bytes to be converted (max. 4)

*****************************************************************************/

void CUUCode::UUCode2Byte(int nNumber)
{

	switch (nNumber) {
	  
	case 2: mpbByteStream[0] = ((mpbUUCodeStream[0] - OFFSET) & 0x3F)
				             + (((mpbUUCodeStream[1] - OFFSET) & 0x03) << 6);
			break;

	case 3: mpbByteStream[0] = ((mpbUUCodeStream[0] - OFFSET) & 0x3F)
				             + (((mpbUUCodeStream[1] - OFFSET) & 0x03) << 6);
		    mpbByteStream[1] = (((mpbUUCodeStream[1] - OFFSET) & 0x3C) >> 2)
				             + (((mpbUUCodeStream[2] - OFFSET) & 0x0F) << 4);
			break;

	case 4: mpbByteStream[0] = ((mpbUUCodeStream[0] - OFFSET) & 0x3F)
				             + (((mpbUUCodeStream[1] - OFFSET) & 0x03) << 6);
		    mpbByteStream[1] = (((mpbUUCodeStream[1] - OFFSET) & 0x3C) >> 2)
				             + (((mpbUUCodeStream[2] - OFFSET) & 0x0F) << 4);
		    mpbByteStream[2] = (((mpbUUCodeStream[2] - OFFSET) & 0x30) >> 4)
				             + (((mpbUUCodeStream[3] - OFFSET) & 0x3F) << 2);
			break;

	} // switch


} // UUCode2Byte()

// =================
// end of uuCode.cpp
// =================
