package com.wibu;

/**
 * Copyright........... (c) 2001-2008
 *
 *                      Stomedia GmbH - Engineering
 *                      Lindenhofstrasse 9
 *                      CH-9630 Wattwil
 *                      Switzerland
 *
 * Internet............ http://www.stomedia.ch
 * Email............... engineering@stomedia.ch
 *
 * Description......... Java(tm)-Adaption for WIBU(tm)-KEY
 * Current SDK......... Sun Java, Standard Developer Kit 1.3.1
 *
 * Version............. 1.0
 * Date................ Februar 2001
 * Author.............. Ph.Mayer, Dipl.Ing.HTL
 *
 * Release............. 1.1
 * Date................ August 2001
 * Author.............. Reto Mettler, Dipl.Ing.FH
 *                      Freddy Kuehne, Dipl.Ing.FH
 *                      Juergen Ockert (WIBU-SYSTEMS)
 *                      Heiko Abraham (WIBU-SYSTEMS)
 * Changes
 * -------
 * 2002Apr22 3.21 jo: added WKB_VERSION_STD and WKB_ACC_AUTOCANCEL flags
 * 2002Sep12 3.30 jo: added WKB_ACC_CREATE flag
 * 2002Sep19 3.30 jo: added WKB_SET_BOXVERSION flag
 * 2002Sep27 3.31 jo: initialize WKBACCESS.flCtrl to 0 (ZERO)
 * 2002Oct14 3.31 jo: new method WibuKJniGetVersionString
 * 2002Oct15 3.31 jo: added WKB_SC_STATIONSHARE
 * 2002Oct21 3.31 jo: added WKB_SEL_MASK
 * 2002Nov04 3.31 jo: added Wkm methods (Network / Monitor API)
 * 2002Dec05 3.31 jo: added WKB_QI_SERVER, WKB_QI_DRIVER, WKB_CRYPT_CTRL
 * 2003Jan29 3.31 jo: disabled (comment out) Wkm methods
 * 2003Feb27 3.32 jo: added WkbQueryEntry2(WKBBOXDATA)
 * 2003Apr04 3.32 jo: added missing WKB_OPEN_??? and WKB_QE_?? values
 * 2003May19 3.32 jo: added missing WKB_AREA_SELECT value
 * 2003Aug08 3.32 jo: added missing flag WKB_BE_INDEXMASK
 * 2003Aug21 4.00 jo: added WKB_VERSION4/5 & Support for Extended Memory
 * 2003Sep19 4.00 jo: added WKB_VERSION_MAX, WKB_VERSION_RED, WKB_VERSION_MASK
 * 2004Jan13 4.01 jo: initialized WKBOPENENTRY::wkbsr member
 * 2005Feb03 5.00 jo: added WKB_QI_JNI
 * 2008Oct06 4.30 ha: addapt for Armv4/Linux
 */

import java.lang.*;

/**
 *
 * @version 5.10
 * 
 * @see <h4>Base WIBU-BOX Open and Encryption</h4> <p>
 * <A HREF = "WkJavaApi.html#WkbOpen2(long, long, java.lang.String, long, long, java.lang.Object)">WkbOpen2</A>
 * @see <A HREF = "WkJavaApi.html#WkbCrypt2(long, long, byte[], com.wibu.WkJavaApi.WKBAREA, com.wibu.WkJavaApi.pInt)">WkbCrypt2</A>
 * @see <A HREF = "WkJavaApi.html#WkbClose2(long)">WkbClose2</A>
 * @see <BR><A HREF = "WkJavaApi.html#WkbQueryEntry2(long, long, com.wibu.WkJavaApi.WKBBOXENTRY)">WkbQueryEntry2</A>
 * 
 * @see <p> <h4>Subsystem (Network) Management</h4> <p>
 * <A HREF = "WkJavaApi.html#WkbAccess2(long, com.wibu.WkJavaApi.WKBACCESS)">WkbAccess2</A>
 * @see <A HREF = "WkJavaApi.html#WkbRelease2(long)">WkbRelease2</A>
 * @see <BR><A HREF = "WkJavaApi.html#WkbQuerySystem2(long, long, com.wibu.WkJavaApi.WKBSYSTEM, int)">WkbQuerySystem2</A>
 * @see <A HREF = "WkJavaApi.html#WkbEnumServers2(long, java.lang.StringBuffer, int)">WkbEnumServers2</A>
 *
 * @see <p> <h4>List Port and WIBU-BOX Contents </h4> <p>
 * <A HREF = "WkJavaApi.html#WkbEnumPorts2(long, long, java.lang.StringBuffer, int)">WkbEnumPorts2</A>
 * @see <A HREF = "WkJavaApi.html#WkbListPort2(long, long, java.lang.String, java.lang.Object, int)">WkbListPort2</A>
 * @see <A HREF = "WkJavaApi.html#WkbListBox2(long, long, java.lang.String, com.wibu.WkJavaApi.WKBSERIAL, com.wibu.WkJavaApi.WKBBOXDATA)">WkbListBox2</A>
 *
 * @see <p> <h4>General Error and Execution Management  </h4> <p>
 * <A HREF = "WkJavaApi.html#WkbGetLastError()">WkbGetLastError</A>
 * @see <A HREF = "WkJavaApi.html#WkbSetLastError(int)">WkbSetLastError</A>
 *
 * @see <p> <h4>Driver Status and Control</h4> <p>
 * <A HREF = "WkJavaApi.html#WkbGetVersion3(long)">WkbGetVersion3</A>
 * @see <A HREF = "WkJavaApi.html#WibuKJniGetVersionString(java.lang.StringBuffer)">WibuKJniGetVersionString</A><!-- 
 * <!-- @see <A HREF = "WkJavaApi.html#WkbControlSystem2(long, long, java.lang.Object, long)">WkbControlSystem2</A>-->
 * @see <A HREF = "WkJavaApi.html#WkbQueryInfo2(long, long, java.lang.Object, int)">WkbQueryInfo2</A>
 * @see <A HREF = "WkJavaApi.html#WkbQueryStatus2(long, long, com.wibu.WkJavaApi.WKBSTATUS, int)">WkbQueryStatus2</A>
 *
 * <!--@see <p> <h4>Port Status and Control </h4> <p>
 * <A HREF = "WkJavaApi.html#WkbQueryPort2(long, long, java.lang.String, java.lang.Object, int)">WkbQueryPort2</A>
 * @see <A HREF = "WkJavaApi.html#WkbSetPort2(long, long, java.lang.String, byte[], int)">WkbSetPort2</A>
 * -->
 *
 * @see <p> <h4>Programming WIBU-BOXes </h4> <p>
 * <A HREF = "WkJavaApi.html#WkbProgram2(long, long, java.lang.String, com.wibu.WkJavaApi.WKBSERIAL, com.wibu.WkJavaApi.WKBPROGRAM)">WkbProgram2</A>
 *
 * <!--@see <p> <h4>Driver Local Memory Management </h4> <p>
 * <A HREF = "WkJavaApi.html#WkbAllocMem(long, long)">WkbAllocMem</A>
 * @see <A HREF = "WkJavaApi.html#WkbFreeMem(long)">WkbFreeMem</A>
 * @see <A HREF = "WkJavaApi.html#WkbWriteMem(long, byte[], long)">WkbWriteMem</A>
 * @see <A HREF = "WkJavaApi.html#WkbReadMem(long, byte[], long)">WkbReadMem</A>
 * -->
 *
 */ 
 
public class WkJavaApi
{
  private static int  mnMajor   = 5,
                      mnMinor   = 20,
                      mnBuild   = 2,
                      mnCount   = 500;
  public static String getVersion()
  {
    String  strVersion = new String();
    strVersion = mnMajor + "."
      + mnMinor + "."
      + mnBuild + "."
      + mnCount;
    return strVersion;
  } // getVersion()

   // ******************************************************************************************************** //
   // ERROR CODES, WIBU DESCRIPTION
   // ******************************************************************************************************** //

   /**
    * The operation was successfully. Normally this code is used to check an error code against validity.
    */
   public static final int WKB_NO_ERROR                         = 0;
  
   /**
    * The specified handle is invalid. Normally the object which is addressed by this handled is closed or
    * destroyed previously.
    */
   public static final int WKB_ERROR_INVALID_HANDLE             = 1;
  
   /**
    * A method like <A HREF = "WkJavaApi.html#WkbOpen2(long, long, java.lang.String, long, long,
    * java.lang.Object)">WkbOpen2</A> or WkbAccess2 fails because the memory is small to create a new object and
    * returns a handle of this object. Under Windows 3.1 this error can be also a result of a lot of
    * application breakdowns, which remained non-addressable data structures in the driver. In this case
    * the complete Windows system should be started again. The DOS driver supports only one opened WIBU-BOX
    * entry at the same time.
    */
   public static final int WKB_ERROR_NO_MORE_HANDLES            = 2;
  
   /**
    * A method like <A HREF = "WkJavaApi.html#WkbOpen2(long, long, java.lang.String, long, long,
    * java.lang.Object)">WkbOpen2</A> cannot interpret the specified port number or port name. The WkbEnumPort2
    * method returns the names of all available WibuKey ports and only such names may be used for port
    * specifications except <A HREF = "WkJavaApi.html#WkbOpen2(long, long, java.lang.String, long, long,
    * java.lang.Object)">WkbOpen2</A> permits wildcards where port index names like 1 in LPT1 may be replaced
    * by a star (*). This error code is also returned by the WkbEnumPorts2 method with sub command
    * WKB_EPT_NEXT, if the end of the list of all available ports is reached.
    */
   public static final int WKB_ERROR_INVALID_PORT               = 3;
  
   /**
    * A method has detected an invalid parameter. This may be an invalid address, a wrong option, a not
    * properly terminated string or a bad length. Invalid handles produces normally the WKB_ERROR_INVALID_HANDLE
    * error code, if a specific option is not implemented in the used driver variant, normally the error
    * WKB_ERROR_OPTION_NOT_SUPPORTED is returned. Invalid port numbers produces the WKB_ERROR_INVALID_PORT error code.
    */
   public static final int WKB_ERROR_INVALID_PARAMETER          = 4;
  
   /**
    * The size of the specified destination buffer is too small. Normally each method of the WibuKey API 2
    * returns the needed size as return value if this error occurred. When the method is called again with
    * this high size value and an increased buffer, the operation should be executed successfully.
    */
   public static final int WKB_ERROR_BUFFER_OVERFLOW            = 5;
  
   /**
    * A critical part of the WibuKey driver cannot be entered because it is locked by another thread or process.
    * This error is only possible under preemptive multitasking systems like OS/2 or Windows NT/2000. This error
    * is normally the result of a time-out error of the operation system inside the WibuKey driver.
    */
   public static final int WKB_ERROR_LOCK_VIOLATION             = 6;
  
   /**
    * An addressed WIBU-BOX has not answered in the expected time interval. Possibly the WIBU-BOX is removed by
    * the user or is defect or a hardware or driver problem exists.
    */
   public static final int WKB_ERROR_NOT_READY                  = 7;
  
   /**
    * A method like WkbListPort2 has no WIBU-BOX detected at the specified WibuKey port.
    */
   public static final int WKB_ERROR_NOT_CONNECTED              = 8;
  
   /**
    * The called method is not implemented in the used driver variant or version. Normally this error can be
    * a result in a mixed driver concept like on Windows 95/98/NT/2000 if the (newer) client driver supports the
    * called method but not the (older) server driver.
    */
   public static final int WKB_ERROR_INVALID_FUNCTION           = 9;
  
   /**
    * The WkbGetFC or WkbGetUC method has detected that the program is not encrypted with CRYPT or WKCRYPT
    * and has no valid Firm and User Code.
    */
   public static final int WKB_ERROR_NOT_CRYPTIC                = 10;
  
   /**
    * A method cannot interpret a specified flag or option. This is a result of a specific limitation of the
    * used WibuKey driver and no general parameter specification problem. It may be that that the same method
    * call occurs correctly with another WibuKey driver of the same version. For example, Unicode and the use of
    * the WKB_UNICODE option is supported by WKWIN32.DLL but not by any other driver.
    */
   public static final int WKB_ERROR_OPTION_NOT_SUPPORTED       = 11;
  
   /**
    * The memory of the WibuKey driver ran out. It may be that too many WIBU-BOX entries are opened or
    * too many buffers are allocated. On Windows 3.1 this error can be also a result of a lot of application
    * breakdowns, which remained non-addressable data structures in the driver. In this case the complete
    * Windows system should be started again.
    */
   public static final int WKB_ERROR_MEMORY_TOO_SMALL           = 12;
  
   /**
    * The version of the connected WIBU-BOX doesn't support the specified operation.
    * Following situations of calling <A HREF = "WkJavaApi.html#WkbOpen2(long, long, java.lang.String, long, long,
    * java.lang.Object)">WkbOpen2</A> are typical to receive this error:
    * - A WIBU-BOX version 1 is used with algorithm version WKB_VERSION2 or WKB_VERSION2.
    * - A WIBU-BOX version 3 or 4 is used with algorithm version WKB_VERSION3.
    * - A WIBU-BOX version 5 is used with algorithm version WKB_VERSION1 or WKB_VERSION2.
    * <A HREF = "WkJavaApi.html#WkbAccess2(long, com.wibu.WkJavaApi.WKBACCESS)">WkbAccess2</A> returns this error code if one
    * of the following condition is true:
    * - A WIBU-BOX version 5 on a WkLAN server accessed with WKB_ACC_NONETLIMIT.
    */
   public static final int WKB_ERROR_INVALID_VERSION            = 13;

   /**
    * The method cannot interpret the calling level, which is specified in the control flag parameter with
    * a WKB_LEVELx-constant. Except other specified, all methods of the driver supports level 0 or 1, which
    * is specified with the constant WKB_LEVEL0 and WKB_LEVEL1. The static DOS driver library (WKDOS.LIB) and
    * the OS/2 drivers (WKOS2.DLL and WKOS232.DLL) support only level 0.
    */
   public static final int WKB_ERROR_INVALID_LEVEL              = 14;
  
   /**
    * A method like <A HREF = "WkJavaApi.html#WkbCrypt2(long, long, byte[], com.wibu.WkJavaApi.WKBAREA, WkJavaApi.pInt)">WkbCrypt2</A>
    * detects an CRC verification error in the encrypted or decrypted data sequence
    * which was transferred directly through the WibuKey-Hardware. This error is only generated if all specified
    * bytes were transferred and the CRC checking is activated.
    */
   public static final int WKB_ERROR_CRC_VERIFY                 = 15;
  
   /**
    * A method like <A HREF = "WkJavaApi.html#WkbCrypt2(long, long, byte[], com.wibu.WkJavaApi.WKBAREA, WkJavaApi.pInt)">WkbCrypt2</A>
    * cannot encrypt or decrypt data via the encrypted because the WIBU-BOX is not
    * selected. Normally this error occurs if no previous call of a method like <A HREF = 
    * "WkJavaApi.html#WkbSelect2(long, long, long, java.lang.Object)">WkbSelect2</A> was done, or in the interval between
    * <A HREF = "WkJavaApi.html#WkbSelect2(long, long, long, java.lang.Object)">WkbSelect2</A> and
    * <A HREF = "WkJavaApi.html#WkbCrypt2(long, long, byte[], com.wibu.WkJavaApi.WKBAREA, WkJavaApi.pInt)">WkbCrypt2</A>
    * the WIBU-BOX was removed by the user.
    */
   public static final int WKB_ERROR_NOT_SELECTED               = 16;
  
   /**
    * A method like <A HREF = "WkJavaApi.html#WkbOpen2(long, long, java.lang.String, long, long,
    * java.lang.Object)">WkbOpen2</A> or <A HREF = "WkJavaApi.html#WkbAccess2(long, com.wibu.WkJavaApi.WKBACCESS)">WkbAccess2</A>
    * (via the local subsystem) could not found a WIBU-BOX entry with
    * the specified Firm and User Code. Typically this is a sum error when the WIBU-BOX entry is searched at
    * several ports or at all ports of a subsystem.
    */
   public static final int WKB_ERROR_ENTRY_NOT_FOUND            = 17;
  
   /**
    * A method like WkbListBox2 could not address the WIBU-BOX with the specified Serial Number. Possibly
    * this number or the port specification is wrong. Another reason could be that the user has removed the
    * WIBU-BOX at the port. To read all valid Serial Numbers at a WibuKey port use the WkbListPort2 method.
    */
   public static final int WKB_ERROR_SERIAL_NOT_FOUND           = 18;
  
   /**
    * A method like WkbProgram2 cannot program the specified WIBU-BOX entry into a WIBU-BOX because the
    * programming signature is wrong. The signature depends on the Firm and User Code, on the Serial Number
    * and of values of the WIBU-BOX internal programming counters which are increased with every programming
    * operation. Thus the signature should be calculated again with the current state of the WIBU-BOX.
    */
   public static final int WKB_ERROR_ENTRY_NOT_CHANGED          = 19;
  
   /**
    * A method like WkbProgram2 cannot address the specified WIBU-BOX entry. Normally each entry of a WIBU-BOX,
    * version 1 up to 3 may have an index in the range 0 to 9 but it may be that entries, which index 5 to 9 are
    * filled with an added entry like an Added Data Entry or a Limit Counter. To change such an entry, the index
    * of the base entry in the range 0 to 4 must be used. To program a base entry in such a position, the
    * corresponding base entry including its added entry must be cleared previously.
    */
   public static final int WKB_ERROR_INVALID_ENTRY              = 20;
  
   /**
    * A method like WkbProgram2 could not program the specified WIBU-BOX entry into a WIBU-BOX because the
    * programming signature is wrong. The signature depends on the Firm and User Code, on the Serial Number and
    * on the value of WIBU-BOX internal programming counters which are increased with every programming operation.
    * Thus the signature should be calculated again with the current state of the WIBU-BOX.
    */
   public static final int WKB_ERROR_BAD_SIGNATURE              = 21;
  
   /**
    * A method like WkbProgram2 could not program the specified WIBU-BOX entry into a WIBU-BOX because the
    * needed power to do this operation is too small. This error can also be produced by the
    * <A HREF = "WkJavaApi.html#WkbSelect2(long, long, long, java.lang.Object)">WkbSelect2</A> method
    * if a Selection Code with bit 31=1 (constant WKB_SC_BOXLIMIT) is used which results in a decrement of the Limit
    * Counter of the corresponding WIBU-BOX entry.
    */
   public static final int WKB_ERROR_POWER_TOO_SMALL            = 22;
  
   /**
    * A method like <A HREF = "WkJavaApi.html#WkbSelect2(long, long, long, java.lang.Object)">WkbSelect2</A>
    * could not select the specified WIBU-BOX entry because another task had already
    * selected this entry itself. It must be waited until this task has terminated the selection via calling the
    * <A HREF = "WkJavaApi.html#WkbUnSelect2(long)">WkbUnSelect2</A> or
    * <A HREF = "WkJavaApi.html#WkbClose2(long)">WkbClose2</A> method.
    */
   public static final int WKB_ERROR_ALREADY_SELECTED           = 23;
  
   /**
    * A method like <A HREF = "WkJavaApi.html#WkbSelect2(long, long, long, java.lang.Object)">WkbSelect2</A>
    * could not select the specified WIBU-BOX entry because the used entry has a Limit
    * Counter entry which value is 0. To use this entry again, the Limit Counter must be set to a non-zero value
    * with a programming operation.
    */
   public static final int WKB_ERROR_BOXLIMIT_IS_ZERO           = 24;
  
   /**
    * A method like WkbProgram2 has programmed the specified WIBU-BOX entry into a WIBU-BOX entry but a memory
    * check results in an error. This error can also be produced by the <A HREF = 
    * "WkJavaApi.html#WkbSelect2(long, long, long, java.lang.Object)">WkbSelect2</A> method if a Selection Code
    * with bit 31=1 (constant WKB_SC_BOXLIMIT) is used which results in decreasing the Limit Counter of the
    * corresponding WIBU-BOX entry by 1. A possible reason for this error is one more defect bits in the E2PROM
    * memory. A solution is to use another WIBU-BOX entry or another WIBU-BOX.
    */
   public static final int WKB_ERROR_PROGRAM_VERIFY             = 25;
  
   /**
    * A method like <A HREF = "WkJavaApi.html#WkbSelect2(long, long, long, java.lang.Object)">WkbSelect2</A> has failed
    * because the Algorithm Configuration memory of the WIBU-BOX has corrupt contents (the checksum verification failed).
    * The WIBU-BOX cannot be used for any other encryption. It must be sent back to WIBU-SYSTEMS where it may be repaired.
    */
   public static final int WKB_ERROR_ALGCFG_MEMORY              = 26;

   /**
    * A method like <A HREF = "WkJavaApi.html#WkbOpen2(long, long, java.lang.String, long, long,
    * java.lang.Object)">WkbOpen2</A> cannot open a found entry because the WIBU-BOX with this entry does not support
    * the desired WibuKey algorithm. The algorithm 1 (WKB_VERSION1) is supported by WIBU-BOX version 1, 3 and 4;
    * the algorithm 2 (WKB_VERSION2) by WIBU-BOX version 2, 3 and 4 and algorithm 3 (WKB_VERSION3) is supported by
    * WIBU-BOX version 5.
    */
   public static final int WKB_ERROR_ALGORITHM_NOT_SUPPORTED    = 27;
  
  /**
   * The extended memory of a WIBU-BOX with version 6 or 7 cannot be read or write due to a physical error.
   */
  public static final int WKB_ERROR_BAD_EXTMEM                 = 28;

  /**
   * The 'WKBEXTMEM::iPage' parameter in a call to WkbBoxList2 or WkbProgram2 contains an invalid value.
   */
  public static final int WKB_ERROR_INVALID_EXTMEM_PAGE        = 29;

   /**
    * The error occurs, if a WibuKey API method tries to access to a WIBU-BOX via an existing WibuKey port,
    * but this port is locked permanently and system internally by another application. Normally, the WibuKey
    * kernel driver tries to steal a port from another application, for example LPT1 from the printer driver and
    * waits for the receiving some time. But if thi time overrun and the port is still not available for the WibuKey
    * driver, this returns to the calling application with this error code.
    */
   public static final int WKB_ERROR_PORT_IS_BUSY               = 32;
  
   /**
    * <A HREF = "WkJavaApi.html#WkbSelect2(long, long, long, java.lang.Object)">WkbSelect2</A> returns this error code if
    * one of the following condition is true:
    * - The entry was opened with <A HREF = "WkJavaApi.html#WkbOpen2(long, long, java.lang.String,
    *   long, long, java.lang.Object)">WkbOpen2</A> and WKB_VERSION1 and the specified Selection Code is larger than 65535.
    * - The entry was opened with <A HREF = "WkJavaApi.html#WkbOpen2(long, long, java.lang.String,
    *   long, long, java.lang.Object)">WkbOpen2</A> and the WKB_OPEN_NONETLIMIT option and a Selection Code is specified
    *   with bit 30 (WKB_SC_NETLIMIT) set to 1.
    * - The used subsystem was accessed by <A HREF = "WkJavaApi.html#WkbAccess2(long, com.wibu.WkJavaApi.WKBACCESS)">WkbAccess2</A>
    *   and the WKB_ACC_NONETLIMIT option and a Selection Code is specified with bit 30 (WKB_SC_NETLIMIT) set to 1.
    * A cached indirect selection is set again with the WKB_USE_CACHE but the driver has discarded internally the
    * saved caching data.
    */
   public static final int WKB_ERROR_INVALID_SELECTION          = 33;
  
   /**
    * <A HREF = "WkJavaApi.html#WkbAccess2(long, com.wibu.WkJavaApi.WKBACCESS)">WkbAccess2</A> or <A HREF = "WkJavaApi.html#WkbOpen2(long, long,
java.lang.String, long, long,
    * java.lang.Object)">WkbOpen2</A> returns this error code if the specified entry is already used by another process
    * (typically a WibuKey server application) at the same computer system. Such an entry was opened by
    * <A HREF = "WkJavaApi.html#WkbOpen2(long, long, java.lang.String, long, long, java.lang.Object)">WkbOpen2</A>
    * with the WKB_OPEN_GLOBAL option or by <A HREF = "WkJavaApi.html#WkbAccess2(long, com.wibu.WkJavaApi.WKBACCESS)">WkbAccess2</A>
    * with the WKB_ACC_GLOBAL command avoids that such an entry can be used repeatedly at same time. The WibuKey driver
    * locks opened entries at the same computer except another open operation uses the WKB_OPEN_NONETLIMIT or
    * WKB_ACC_NONETLIMIT option which avoids the using of a Selection Code in which bit 30 (WKB_SC_NETLIMIT) is set.
    */
   public static final int WKB_ERROR_ENTRY_ALREADY_USED         = 34;
  
   /**
    * <A HREF = "WkJavaApi.html#WkbSelect2(long, long, long, java.lang.Object)">WkbSelect2</A> returns this error code
    * if the opened entry has an expiration date and this date is older than
    * the current date. Please note that the Expiration Date can still be used at its own day.
    */
   public static final int WKB_ERROR_OUT_OF_DATE                = 35;
  
   /**
    * A specified pointer addresses invalid memory. It may be that the memory does not exist or a block to be
    * written is write-protected. The API method checks normally not only the pointer itself but the complete
    * memory area, which is filled in dependence of specified block length values.
    */
   public static final int WKB_ERROR_BAD_ADDRESS                = 36;
  
   /**
    * A command code specification (typically in bit 3 up to 7 of the flCtrl parameter) of a method call is
    * invalid. If the used command code is defined, it may be that the specific platform does not support this
    * command or that an older driver is used.
    */
   public static final int WKB_ERROR_INVALID_COMMAND            = 37;
  
   /**
    * <A HREF = "WkJavaApi.html#WkbAccess2(long, com.wibu.WkJavaApi.WKBACCESS)">WkbAccess2</A> returns this error code if one of
    * the following condition is true:
    * - The used driver does not support any of the specified subsystems (kernel/local, WkNet or WkLAN).
    * - The used driver supports one or more of the specified subsystems, but the user has not activated
    *   them via the WKSYSTEM driver setting variable.
    * If one or more of the specified subsystems are checked for a local subsystem or a server subsystem,
    * but no suitable WIBU-BOX entry is found, the WKB_ERROR_SERVER_NOT_FOUND or WKB_ERROR_ENTRY_NOT_FOUND is
    * returned instead of WKB_ERROR_INVALID_SUBSYSTEM.
    */
   public static final int WKB_ERROR_INVALID_SUBSYSTEM          = 38;
  
   /**
    * <A HREF = "WkJavaApi.html#WkbCrypt2(long, long, byte[], com.wibu.WkJavaApi.WKBAREA, WkJavaApi.pInt)">WkbCrypt2</A> returns this error
    * code if the data to be encrypted or decrypted is too long for the selected
    * algorithm. A typical limitation is 4096 bytes for the direct encryption if the driver is split into a calling
    * and a kernel driver (for example on Windows 95/98 or Windows NT/2000).
    */
   public static final int WKB_ERROR_DATA_TOO_COMPLEX           = 39;
  
   /**
    * This error is returned by <A HREF = "WkJavaApi.html#WkbAccess2(long, com.wibu.WkJavaApi.WKBACCESS)">WkbAccess2</A> or
    * <A HREF = "WkJavaApi.html#WkbOpen2(long, long, java.lang.String, long, long, java.lang.Object)">WkbOpen2</A> in combination with the WKB_ACC_EXCLUSIVE
res. WKB_OPEN_EXCLUSIVE
    * option to force an access in the Exclusive Mode and if the specified entry is already used by another access
    * instance (for example in another application) at the same computer system and which has already set the Exclusive
    * Mode for the same Firm Code and User Code.
    */
   public static final int WKB_ERROR_EXCLUSIVE_MODE_CONFLICT    = 40;
  
   /**
    * <A HREF = "WkJavaApi.html#WkbAccess2(long, com.wibu.WkJavaApi.WKBACCESS)">WkbAccess2</A> returns this error code if a client
    * specific user identification should be read from the driver profile and this identification is not found. Reading
    * such identification is forced by setting the idNetUser member in the WKBACCESS structure to WKB_NETUSER_REG. The user identification is then selected
by the application
    * name which is specified in the pszAppName member.
    */
   public static final int WKB_ERROR_NO_USER_SET                = 41;
  
   /**
    * This is no true error code but a valid error state of the WkbIsComplete method. As long as the method with
    * the specified action code has not terminated, WkbIsComplete returns this code. The WkbGetLastError method will
    * normally never return this error code.
    */
   public static final int WKB_ERROR_IN_ACTION                  = 64;
  
   /**
    * The WkbIsComplete method could not find a method call, which corresponds with the specified action code.
    * Either the identification value is completely wrong or the corresponding method has terminated and the past
    * and an earlier call of WkbIsComplete has read the return value and invalidated the identification value. For
    * each asynchronous method call, the WkbIsComplete method returns only once the completion result. If an
    * event or a callback method is set with the WkbSetCompletionEvent or WkbSetCompletionFunction method, the
    * WkbIsComplete method may only be used until the event is masked or the callback method is called because
    * both actions invalidate the action code immediately.
    */
   public static final int WKB_ERROR_UNDEFINED_ACTION           = 65;

   /**
    * An asynchronously called method like <A HREF = "WkJavaApi.html#WkbSelect2(long, long, long, java.lang.Object)">
    * WkbSelect2</A> could not be started because no further action code is
    * available. Under the single tasking DOS driver, only one action code is available; thus only one method
    * can be started asynchronously at the same time.
    */
   public static final int WKB_ERROR_TOO_MANY_ACTIONS           = 66;
  
   /**
    * A method like <A HREF = "WkJavaApi.html#WkbSelect2(long, long, long, java.lang.Object)">WkbSelect2</A> or 
    * <A HREF = "WkJavaApi.html#WkbCrypt2(long, long, byte[], com.wibu.WkJavaApi.WKBAREA, WkJavaApi.pInt)">WkbCrypt2</A>
    * is called for a WibuKey local or network subsystem with waiting
    * (not completed or cancelled) asynchronous calls. A queuing of such calls is not permitted. The method
    * call may be repeated when all asynchronous actions for the common subsystem are completed or when they are
    * cancelled via the WkbControlSystem2 method.
    */
   public static final int WKB_ERROR_SUBSYSTEM_IN_ACTION        = 67;
  
   /**
    * A network access was not possible because no network system exists. Possibly an existing network system
    * is not compatible with WibuKey or a WkNet server or it is not loaded.
    */
   public static final int WKB_ERROR_NO_NET_SYSTEM              = 96;
  
   /**
    * A network access was not successful because data could not be sent after several retries. This is
    * possibly the result of a physical network problem.
    */
   public static final int WKB_ERROR_NET_SEND_FAULT             = 97;
  
   /**
    * A network access was not successful because receiving data are wrong after several retries.
    * This is possibly the result of a physical network problem.
    */
   public static final int WKB_ERROR_NET_RECEIVE_FAULT          = 98;
  
   /**
    * A network access was not successful because a network station has not answered. This is possibly the
    * result of a physical network problem. Under WkNet this error is typically if the WkNet server is not started.
    */
   public static final int WKB_ERROR_NET_NOT_READY              = 99;
  
   /**
    * A method like <A HREF = "WkJavaApi.html#WkbSelect2(long, long, long, java.lang.Object)">WkbSelect2</A> or
    * <A HREF = "WkJavaApi.html#WkbCrypt2(long, long, byte[], com.wibu.WkJavaApi.WKBAREA, WkJavaApi.pInt)">WkbCrypt2</A> cannot access a
    * WibuKey network subsystem because it is locked
    * permanently by other processes or by the server. This may be a temporary problem if the WibuKey network
    * is accessed from an unusual high number of clients at same time or a permanent problem if a client or the
    * server processes failed and hangs up.
    */
   public static final int WKB_ERROR_NET_IS_BUSY                = 100;
  
   /**
    * A method like <A HREF = "WkJavaApi.html#WkbAccess2(long, com.wibu.WkJavaApi.WKBACCESS)">WkbAccess2</A> cannot access the
    * specified WibuKey network system because it is corrupt.
    * Under WkNet for example it may be, that the WkNet server file is corrupt or encrypted by an incompatible
    * method.
    */
   public static final int WKB_ERROR_BAD_NET_SYSTEM             = 101;
  
   /**
    * A method like <A HREF = "WkJavaApi.html#WkbAccess2(long, com.wibu.WkJavaApi.WKBACCESS)">WkbAccess2</A> or
    * <A HREF = "WkJavaApi.html#WkbSelect2(long, long, long, java.lang.Object)">WkbSelect2</A> could not open the specified
    * WkNet system or select the specified WIBU-BOX entry on a WkLAN server because the maximum number of
    * concurrent users of the addressed network server is reached. The
    * <A HREF = "WkJavaApi.html#WkbSelect2(long, long, long, java.lang.Object)">WkbSelect2</A>
    * method produces this error code only if the bit 30 (constant WKB_SC_NETLIMIT) of the Selection Code is set.
    */
   public static final int WKB_ERROR_NET_LIMIT_IS_ZERO          = 102;
  
   /**
    * A network access was not successful because the actual user was canceled by the WibuKey network subsystem.
    * This could be caused either by exceeding the specified timeout value, every user must access the network
    * system at least one time within the timeout period, or the user was canceled manually by the WibuKey
    * application WkNetLst.exe or WkSvMon.exe.
    */
   public static final int WKB_ERROR_NET_USER_CANCELLED         = 103;
  
   /**
    * The data, which should be encrypted via a WibuKey network subsystem, is too complex. Normally, under WkNet
    * only relative small blocks (for example 32 bytes) may be encrypted directly via the WkNet server. Under
    * WkLAN this error may be a result of too less memory on the server side.
    */
   public static final int WKB_ERROR_NET_DATA_TOO_COMPLEX       = 104;
  
   /**
    * This error code is returned by the <A HREF = "WkJavaApi.html#WkbAccess2(long, com.wibu.WkJavaApi.WKBACCESS)">WkbAccess2</A>
    * method if the specified WkLAN-Server is not found in the network.
    * This error code is also returned by the WkbEnumServers2 method with the subcommand WKB_ESV_NEXT, when the list
    * of all available WkLAN servers is terminated.
    */
   public static final int WKB_ERROR_SERVER_NOT_FOUND           = 105;

   /**
    * This method specifies that the called driver could not access a needed kernel driver on lower level.
    * This is typical only for Windows 95/98 res. Windows NT/2000.
    */
   public static final int WKB_ERROR_NO_KRNL_DRIVER             = 128;
  
   /**
    * An error occurred during the communication between a caller driver and a kernel driver of WibuKey, for example
    * on Windows NT/2000 between WKWIN32.DLL and WIBUKEY.SYS. Such a communication error may be a result of a too small
    * data buffer for a data object to store. This error occurs possibly, if drivers of different versions are used at
    * same time.
    */
   public static final int WKB_ERROR_KRNL_COMMUNICATION         = 129;



 // ********************************************************************************************************** //
 // BELOW ARE ALL IMPLEMENTED NATIVE CALLS DOCUMENTED
 // ********************************************************************************************************** //

  /**
  * System handle for local WIBU-BOX.
  */
  public static final long HWKB_LOCAL   = 0x1;
  /**
  * Standard flag (use: WKB_STDCTRL|WKB_OTHER_VALUE|).
  */
  public static final long WKB_STDCTRL  = 0x0100;

  /**
   * This class contains the serial number of a single WIBU-BOX and is used by methods like <A HREF = 
   * "WkJavaApi.html#WkbListBox2(long, long, long, String, com.wibu.WkJavaApi.WKBSERIAL, com.wibu.WkJavaApi.WKBBOXDATA)"> WkbListBox2</A>
   * or <A HREF="WkJavaApi.html#WkbProgram2(long, long, java.lang.String, com.wibu.WkJavaApi.WKBSERIAL, com.wibu.WkJavaApi.WKBPROGRAM)"> WkbProgram2</A> to address a specific WIBU-BOX.
   */
  public static class WKBSERIAL
  {
    /**
     * contains the kernel of the Serial Number and is a value in the range between 1 and 4294967294.
     */
    public long ulSerial;
   
    /**
     * contains the mask value of the WibuKey-ASIC which is 1 for WIBU-BOXes of version 1, 2 for
     * WIBU-BOXes of version 2, etc.
     */
    public short usMask;
   
    /**
     * is reserved for future use and must be set to 0.
     */
    public short usReserved;
  }


  


  // ******************************************************************************************************** //
  // NATIVE CALL for WkbOpen2
  // ******************************************************************************************************** //

  /**
   * Opens an entry in a WIBU-BOX for encryption or decryption with subsequent call of 
   * <A HREF = "WkJavaApi.html#WkbCrypt2(long, long, byte[], com.wibu.WkJavaApi.WKBAREA, com.wibu.WkJavaApi.pInt)">WkbCrypt2</A>.
   * @param hwkbsys contains the handle of a valid accessed WibuKey subsystem. To open a WIBU-BOX at the standard
   *        local subsystem the predefined handle value HWKB_LOCAL can be used.
   * @param flCtrl contains flags which control the opening of the WIBU-BOX.
   * <ul> 
   * <li>       The first flag group specifies the command code of the method: 
   * <ul> 
   *   <li>WKB_OPEN_FIND (0x0000) Search a WIBU-BOX entry with the specified Firm and User Code. The WIBU-BOX search 
   *       order depends on several rules; see the Comments paragraph below. 
   *   <li>WKB_OPEN_ENTRY (0x0010) Open the WIBU-BOX 
   *       entry, which is specified by the pvCtrl parameter. 
   *   <li>WKB_OPEN_MULTIFIND (0x0020) Opens all entries in a 
   *       specific WIBU-BOX which are matched by the specified Firm and User Code. The WIBU-BOX search order is the same like 
   *       of WKB_OPEN_FIND and depends on several rules; see the Comments paragraph below. This includes standard Firm/User 
   *       Code entries and also Master entries. If this mode is specified, WkbSelect2 can touch a Limit Counter or check an 
   *       expiration date, which is not necessarily bundled with the first matched entry of a specific WIBU-BOX. 
   *       WkbQueryEntry2 may return the list of all opened entries if the WKB_QE_MULTIENTRY option is set. 
   *    </ul>
   * <li> The second group specifies the version of the WIBU-BOX, which is opened: 
   * <ul> 
   *   <li>WKB_VERSION1 (0x0000) Open the WIBU-BOX to use WibuKey algorithm version 1 in an encryption. 
   *   <li>WKB_VERSION2 (0x0001) Open the WIBU-BOX to use WibuKey algorithm version 2 in an encryption. 
   *   <li>WKB_VERSION3 (0x0002) Open the WIBU-BOX to use WibuKey algorithm version 3 in an encryption. 
   *   <li>WKB_VERSION_STD (0x0007) Open the WIBU-BOX with the WibuKey algorithm of the highest version, which is 
   *       supported by this WIBU-BOX; for example WKB_VERSION2 for a WIBU-BOX/U and WKB_VERSION3 for WIBU-BOX/RU. It is 
   *       recommended for the RED (Random Encryption Decryption) high security encryption method. 
   * </ul>
   * <li> The third group contains additional option flags, which are permitted only for the WKB_OPEN_FIND command: 
   * <ul> 
   *   <!-- <li> <font color="#FF00FF">WKB_PORTNUMBER (0x0008) Use a port number instead of port names. </font>
   *   -->
   *   <li>WKB_OPEN_NONETLIMIT (0x100000) The WIBU-BOX entry is opened without touching the network user limitation. 
   *       This avoids the using of Selection Codes with WKB_SC_NETLIMIT (bit 30) set (see WKBAREA) but allows also the using 
   *       of WIBU-BOX entries are globally locked by the WKB_OPEN_GLOBAL option (see below). This mode is automatically 
   *       inherited from its subsystem if this was created with the WKB_ACC_NONETLIMIT flag. 
   *   <li>WKB_OPEN_GLOBAL (0x200000) The WIBU-BOX entry is opened globally at the used computer. This means, that no other 
   *       open operation may be done to the same WIBU-BOX entry except that the WKB_OPEN_NONETLIMIT option (see above) is specified. 
   *   <li>WKB_OPEN_EXCLUSIVE (0x400000) The WIBU-BOX entry is opened in the Exclusive Mode for the specified Firm Code 
   *       and User Code. Each WIBU-BOX entry may be accessed only once at same time for a specific Firm Code and User Code if 
   *       this flag is set. This flag must be set if an application should be started only once within its local or WkLAN 
   *       subsystem for its specific Firm Code and User Code. This mode is automatically inherited from its subsystem if this 
   *       was created with the WKB_ACC_EXCLUSIVE flag. 
   *   <li>WKB_OPEN_STATIONSHARE (0x800000) The WIBU-BOX entry is opened 
   *       in the StationShare Mode for the specified Firm Code and User Code. This mode is automatically inherited from its 
   *       subsystem if this was created with the WKB_ACC_STATIONSHARE flag. Each encryption must have set the 
   *       WKB_SC_STATIONSHARE flag (bit 28) in the Selection Code, see <A HREF = "WkJavaApi.WKBAREA.html">WKBAREA</A> class, 
   *       otherwise the selection of the encryption fails. 
   * </ul>
   * <li> This parameter contains also standard function control flags.
   * </ul>
   * 
   * @param pszPort contains one or more port specifications at which WIBU-BOXes are 
   * searched. If this is set to null (recommended for normal use), all available WibuKey ports of the specified 
   * WibuKey subsystem are scanned in an optimized order (see Comments paragraph below). If the string is a star (*), the 
   * order begins with the port of the lowest index (LPT1) and is continued with the next index (LPT2 for example) until 
   * all ports are scanned. Otherwise, one or more port names must be set, separated by a vertical bar (|). Furthermore, 
   * complete port groups can be specified by replacing a port name index number by a star, for example LPT* or COM*. 
   * <ul>
   *   <li> This parameter is ignored and must be set to null if the WKB_OPEN_ENTRY mode is specified. 
   * </ul>
   *
   * @param lFirmCode contains the Firm Code (range 1..16777215) for the WKB_OPEN_FIND or WKB_OPEN_MULTIFIND mode. 
   * This parameter is ignored and must be set to 0 if a control class is specified via the pvCtrl parameter. If the 
   * entry is searched in a valid subsystem, created by an WkbAccess2 call, the Firm Code of this subsystem may be used 
   * by setting the WKB_ACCESS_CODE (-1) in this parameter. Otherwise a value greater 0 must be used.
   *
   * @param lUserCode contains the User Code (range 0..16777215) for the WKB_OPEN_FIND or WKB_OPEN_MULTIFIND mode. 
   * This parameter is ignored and must be set to 0 if a control class is specified via the pvCtrl parameter. If the 
   * entry is searched in a valid subsystem, created by a WkbAccess2 call, the User Code of this subsystem may be used 
   * by setting the WKB_ACCESS_CODE (-1) in this parameter.
   *
   * @param pvCtrl optional control data which are used in several commands. For the WKB_OPEN_ENTRY command, this 
   * parameter is a <A HREF = "WkJavaApi.WKBOPENENTRY.html">WKBOPENENTRY</A> class which contains all data which 
   * is needed to open a specific WIBU-BOX entry. The class must contain the desired WibuKey encryption algorithm 
   * variant (WKB_VERSION1, WKB_VERSION2, WKB_VERSION3 or WKB_VERSION_STD) in the flCtrl member variable, the port 
   * specification in the lPort member variable, the Serial Number of the accessed WIBU-BOX in the wkbsr member variable,
   * and the entry index of the desired WIBU-BOX entry in the usEntry member. For opening a Master Entry, the lUserCode 
   * member variable must also contain the desired User Code. The contents of all other class members are ignored. 
   * If these members are not properly initialized, for example by receiving their contents via the 
   * <A HREF = "WkJavaApi.html#WkbQueryEntry2(long, long, com.wibu.WkJavaApi.WKBBOXENTRY)">WkbQueryEntry2</A> method, 
   * these members should be set to 0.
   *
   * @return handle of the opened WIBU-BOX entry if the operation was successful and null if an error occurred. In 
   * this case a subsequent call of the <A HREF = "WkJavaApi.html#WkbGetLastError()">WkbGetLastError</A> method 
   * returns the corresponding error code.
   * <ul>
   *   <li> If a single port is specified in the pszPort parameter, a specific error code is returned, for example 
   *   WKB_ERROR_INVALID_PORT or WKB_ERROR_NOT_READY. 
   *   <li> If several ports are specified and scanned, the error code WKB_ERROR_ENTRY_NOT_FOUND is returned as a sum error 
   *   code. 
   *   <li> If a WIBU-BOX entry is found but the WIBU-BOX version of this entry is wrong and no other suitable entry is found, 
   *   WKB_ERROR_INVALID_VERSION is returned also if several ports are scanned. 
   *   <li> If a WIBU-BOX entry is found but the WIBU-BOX entry is locked by another process and no other suitable entry is 
   *   found, WKB_ERROR_ENTRY_ALREADY_USED is returned also if several ports are scanned. 
   * </ul>
   */
   
  public static native long WkbOpen2(long hwkbsys, long flCtrl, String pszPort, long lFirmCode,
                                                                 long lUserCode, Object pvCtrl);
   
  /**
   * (0x0000) Open the WIBU-BOX for using WibuKey algorithm version 1 in an encryption.
   */
  public static final long WKB_VERSION1         = 0x0;
   
  /**
   * (0x0001) Open the WIBU-BOX for using WibuKey algorithm version 2 in an encryption.
   */
  public static final long WKB_VERSION2         = 0x1;
 
  /**
   * (0x0002) Open the WIBU-BOX for using WibuKey algorithm version 3 in an encryption.
   */
  public static final long WKB_VERSION3         = 0x2;

  /**
   * (0x0003) Open the WIBU-BOX for using WibuKey algorithm version 4 in an encryption.
   */
  public static final long WKB_VERSION4         = 0x3;

  /**
   * (0x0004) Open the WIBU-BOX for using WibuKey algorithm version 5 in an encryption.
   */
  public static final long WKB_VERSION5         = 0x4;
 
  /**
   * (0x0007) Open the WIBU-BOX with the WibuKey algorithm of the highest version, 
   * which is supported by this WIBU-BOX; for example WKB_VERSION2 for a WIBU-BOX/U and
   * WKB_VERSION3 for WIBU-BOX/RU. It is recommended for the RED (Random Encryption 
   * Decryption) high security encryption method.
   */
  public static final long WKB_VERSION_STD      = 0x7;
  
  public static final long WKB_VERSION_MAX      = WKB_VERSION5;

  public static final long WKB_VERSION_RED      = 0x0006;

  public static final long WKB_VERSION_MASK      = 0x0007;
  /**
   * (0x0000) Search a WIBU-BOX entry with the specified Firm and User Code. The WIBU-BOX 
   * search order depends on several rules; see the Comments paragraph below.
   */
  public static final long WKB_OPEN_FIND        = 0x0000;
  
  /**
   * (0x0010) Open the WIBU-BOX entry, which is specified by the pvCtrl parameter.
   */
  public static final long WKB_OPEN_ENTRY       = 0x0010;
  
  /**
   * (0x0020) Opens all entries in a specific WIBU-BOX which are matched by the specified
   * Firm and User Code. The WIBU-BOX search order is the same like of WKB_OPEN_FIND and
   * depends on several rules; see the Comments paragraph below. This includes standard 
   * Firm/User Code entries and also Master entries. If this mode is specified, WkbSelect2 
   * can touch a Limit Counter or check an expiration date, which is not necessarily bundled
   * with the first matched entry of a specific WIBU-BOX. WkbQueryEntry2 may return the list
   * of all opened entries if the WKB_QE_MULTIENTRY option is set. 
   */
  public static final long WKB_OPEN_MULTIFIND   = 0x0020;
  
  /**
   * (0x00F0) Bitmask for WKB_OPEN_??? flags.
   */
  public static final long WKB_OPEN_MASK        = 0x00F0;
  
  /**
   * (0x100000) The WIBU-BOX entry is opened without touching the network user limitation.
   * This avoids the using of Selection Codes with WKB_SC_NETLIMIT (bit 30) set (see WKBAREA)
   * but allows also the using of WIBU-BOX entries are globally locked by the WKB_OPEN_GLOBAL
   * option (see below). This mode is automatically inherited from its subsystem if this was
   * created with the WKB_ACC_NONETLIMIT flag.
   */
  public static final long WKB_OPEN_NONETLIMIT  = 0x100000;

  /**
   * (0x200000) The WIBU-BOX entry is opened globally at the used computer. This means, that no
   * other open operation may be done to the same WIBU-BOX entry except that the WKB_OPEN_NONETLIMIT
   * option (see above) is specified.
   */
  public static final long WKB_OPEN_GLOBAL      = 0x200000;
 
  /**
   * (0x400000) The WIBU-BOX entry is opened in the Exclusive Mode for the specified Firm Code and User Code.
   * Each WIBU-BOX entry may be accessed only once at same time for a specific Firm Code and User Code if this
   * flag is set. This flag must be set if an application should be started only once within its local or WkLAN
   * subsystem for its specific Firm Code and User Code. This mode is automatically inherited from its subsystem
   * if this was created with the WKB_ACC_EXCLUSIVE flag.
   */
  public static final long WKB_OPEN_EXCLUSIVE   = 0x400000;

  /**
   * (0x800000) The WIBU-BOX entry is opened in Station Share Mode for the specified Firm Code and User Code.
   * 
   */
  public static final long WKB_OPEN_STATIONSHARE   = 0x800000;

  
  // ******************************************************************************************************** //
  // NATIVE CALL for WkbSelect2
  // ******************************************************************************************************** //
 
  /**
   * Selects an algorithm variant for encryption via the opened WIBU-BOX entry.
   * @param hwkbe contains the handle of an opened WIBU-BOX entry.
   * @param flCtrl contains flags which control the execution of the method. The command code is identical with the 
   * flSelCtrl member of the <A HREF = "WkJavaApi.WKBAREA.html">WKBAREA</A> class. 
   * <ul>
   *   <li> This parameter contains also standard function control flags. 
   * </ul>
   *
   * @param ulSelectCode contains the Selection Code. This value is identical with the ulSelectCode member of the 
   * <A HREF = "WkJavaApi.WKBAREA.html">WKBAREA</A> class.
   * 
   * @param pvCtrl optional control data for the selection. This parameter is identical with the pvCtrl member of the 
   * <A HREF = "WkJavaApi.WKBAREA.html">WKBAREA</A> class.
   *
   * @return This method returns value > 0 if the operation was successful and value = 0 if an error occurred.
   * In this case a subsequent call of the <A HREF = "WkJavaApi.html#WkbGetLastError()">WkbGetLastError</A> method 
   * returns the corresponding error code.
   * The method locks the driver internally until the WkbUnSelect2 method is called. Hence the WIBU-BOX cannot 
   * process any calls of the WkbSelect2 method of another WIBU-BOX entry handle of the current or other tasks or 
   * processes until the <A HREF = "WkJavaApi.html#WkbUnSelect2(long)">WkbUnSelect2</A> method is called which unlocks 
   * the WIBU-BOX again. Because a complete WIBU-BOX is locked by a single call of a WkbSelect2 method to one of its 
   * entries, other calls of the WkbSelect2 method with different entry handle fail. Therefore, after terminating the 
   * encryption or decryption via the 
   * <A HREF = "WkJavaApi.html#WkbCrypt2(long, long, byte[], com.wibu.WkJavaApi.WKBAREA, com.wibu.WkJavaApi.pInt)"> 
   * WkbCrypt2</A> method, the entry should be freed as quickly as possible by calling the 
   * WkbUnSelect2 method. For compatibility with future purposes, this should be also realized by programs, which have 
   * currently exclusively addressed a WIBU-BOX.  For a specific subsystem, all asynchronously executed WkbSelect2 and 
   * WkbCrypt2 method calls must be completed or cancelled before a further WkbSelect2 call may be executed. Under 
   * Windows the unlocking operation should be done before each explicit task switch (which is executed, for example, by 
   * calling the Windows GetMessage method). This avoids failures of WkbSelect2 method calls in another tasks that 
   * address the same WIBU-BOX as the task currently running. Under Win32, this limitation can be ignored. Please note also 
   * the comments, which are specified in the description of the WKBAREA class regarding the 
   * interpretation of the Selection Code.
   *
   * @deprecated This method is obsolete and replaced by 
   * <A HREF = "WkJavaApi.html#WkbCrypt2(long, long, byte[], com.wibu.WkJavaApi.WKBAREA, com.wibu.WkJavaApi.pInt)">
   * WkbCrypt2</A> method with specified <A HREF = "WkJavaApi.WKBAREA.html">WKBAREA</A> class and set 
   * WKB_AREA_SELECT option.
   */
  public static native int WkbSelect2(long hwkbe, long flCtrl, long ulSelectCode, Object pvCtrl);
  
  
  // ******************************************************************************************************** //
  // NATIVE CALL for WkbUnSelect2
  // ******************************************************************************************************** //
 
  /**
   * This method sets the specified WIBU-BOX entry to an unselected state. This entry can be selected now by other 
   * opened WIBU-BOX entries.
   * @param hwkbe contains the handle of an opened WIBU-BOX entry.
   * @return This method returns value > 0 if the operation was successful and value = 0 if an error occurred.
   * In this case a subsequent call of the <A HREF = "WkJavaApi.html#WkbGetLastError()">WkbGetLastError</A> 
   * method returns the corresponding error code.
   *
   * @deprecated This method is obsolete and replaced by 
   * <A HREF = "WkJavaApi.html#WkbCrypt2(long, long, byte[], com.wibu.WkJavaApi.WKBAREA, com.wibu.WkJavaApi.pInt)">
   * WkbCrypt2</A> method with specified <A HREF = "WkJavaApi.WKBAREA.html">WKBAREA</A> class and set 
   * WKB_AREA_SELECT option.
   */
  public static native int WkbUnSelect2(long hwkbe);
  
  
 
  // ******************************************************************************************************** //
  // NATIVE CALL for WkbQueryEntry2
  // ******************************************************************************************************** //

  /**
   * Gets the complete data of a single opened WIBU-BOX entry (WKB_OPEN_FIND/WKB_OPEN_ENTRY) or of the first WIBU-BOX 
   * entry (WKB_OPEN_MULTIENTRY) by reading the specific WIBU-BOX contents.
   *
   * @param hwkbe Handle to an entry.
   *
   * @param flCtrl contains flags which control the execution of the method. WKB_QE_BOX will be set by this method.
   * <ul>
   *   <li> This parameter contains also standard function control flags. 
   * </ul>
   *
   * @param boxEntry <A HREF = "WkJavaApi.WKBBOXENTRY.html">WKBBOXENTRY</A> class to receive entry infos including 
   * additional entry info.
   *
   * @return 0 if any error occurred.
   * In this case a subsequent call of the <A HREF = "WkJavaApi.html#WkbGetLastError()">WkbGetLastError</A> method 
   * returns the corresponding error code.
   */
  public static native long WkbQueryEntry2(long hwkbe, long flCtrl, WKBBOXENTRY boxEntry);

  /**
   * Gets the driver internal stored data about an opened WIBU-BOX entry.
   *
   * @param hwkbe Handle to an entry.
   *
   * @param flCtrl contains flags which control the execution of the method. WKB_QE_OPEN will be set by this method.
   * <ul>
   *   <li> This parameter contains also standard function control flags. 
   * </ul>
   *
   * @param openEntry <A HREF = "WkJavaApi.WKBOPENENTRY.html">WKBOPENENTRY</A> class to receive the driver internal 
   * stored data about an opened WIBU-BOX entry.
   *
   * @return 0 if any error occurred.
   * In this case a subsequent call of the <A HREF = "WkJavaApi.html#WkbGetLastError()">WkbGetLastError</A> method 
   * returns the corresponding error code.
   */
  public static native long WkbQueryEntry2(long hwkbe, long flCtrl, WKBOPENENTRY openEntry);

  /**
   * Gets all entries, which are open and is normally used in combination with WKB_OPEN_MULTIFIND.
   *
   * @param hwkbe Handle to entries.
   *
   * @param flCtrl contains flags which control the execution of the method. WKB_QE_MULTIENTRY will be set by this method.
   * <ul>
   *   <li> This parameter contains also standard function control flags. 
   * </ul>
   *
   * @param boxData <A HREF = "WkJavaApi.WKBBOXDATA.html">WKBBOXDATA</A> class to receive all entries, which are 
   * opened, including additional entry info. If the entry was opened with the WKB_OPEN_FIND or the WKB_OPEN_ENTRY 
   * command, this single open entry is returned. 
   *
   * @return 0 if any error occurred.
   * In this case a subsequent call of the <A HREF = "WkJavaApi.html#WkbGetLastError()">WkbGetLastError</A> method 
   * returns the corresponding error code.
   */
  public static native long WkbQueryEntry2(long hwkbe, long flCtrl, WKBBOXDATA boxData);

  /**
   * (0x0000) returns the internal WkbOpen2 data into a WKBOPENENTRY structure.
   */
  public static final long WKB_QE_OPEN         = 0x0000;
  
  /**
   * (0x0010) returns the complete contents of the opened WIBU-BOX entry into a WKBBOXENTRY
   * structure. If WkbOpen2 has opened the WIBU-BOX with the WKB_OPEN_MULTIENTRY option, the
   * first entry is returned.
   */
  public static final long WKB_QE_BOX          = 0x0010;
  
  /**
   * (0x0020) returns in a WKBBOXDATA structure a list of the complete contents of all 
   * WIBU-BOX entries, which are opened by WkbOpen2 with the WKB_OPEN_MULTIFIND option. If 
   * the entry was opened with the WKB_OPEN_FIND or the WKB_OPEN_ENTRY command, this single
   * open entry is returned. 
   */
  public static final long WKB_QE_MULTIENTRY   = 0x0020;

  // ******************************************************************************************************** //
  // Structure WKBBOXENTRY
  // ******************************************************************************************************** //
 
  /**
   * This class describes a single WIBU-BOX base including an optional Added Data Entry or a Limit Counter.
   * This class is read as part of a WKBBOXDATA class by the WkbListBox2 method.
   */
  public static class WKBBOXENTRY
  {
    /**
     * contains flag which specify the position of the entry, its type and further options.
     * The member is separated in several groups.
     * list of all several groups ...
     */
    public long flCtrl;
   
    /**
     * contains the Firm Code of a FC/UC or Master Entry. For defect or invalid entries,
     * this member contains the three lower bytes of a WIBU-BOX entry. Otherwise this member contains 0.
     */
    public long ulFirmCode;
   
    /**
     * contains the User Code of a FC/UC or Master Entry. For defect or invalid entries, this member contains
     * the three lower bytes of a WIBU-BOX entry. Otherwise this member contains 0.
     */
    public long ulUserCode;
   
    /**
     * contains the lower 32 bits of a User Data Entry or of an Added Data Entry, the absolute value of an
     * Expiration Date, or the 20 bits of the Limit Counter value (absolute or offset, see WKB_LIMIT_xxx option
     * in the flCtrl member). Otherwise this member contains 0.
     */
    public long ulData1;
   
    /**
     * contains the upper 8 bits of a User Data Entry, the relative number of days between the Expiration Date
     * and the current day or the 8 bit local programming counter value of an Added Data Entry. Otherwise this
     * entry contains 0.
     */
    public long ulData2;
    //Functions replaces corresponding bit-mask-constants
    //Get type of entry
   
    /**
     * Method only in the Java Version available. Returns TRUE if type maske equals WKB_BE_FCUC.
     */
    public boolean isFCUC() { return (flCtrl & WKB_BE_TYPEMASK) == WKB_BE_FCUC;}
   
    /**
     * Method only in the Java Version available. Returns TRUE if type maske equals WKB_BE_MASTER.
     */
    public boolean isMASTER() { return (flCtrl & WKB_BE_TYPEMASK) == WKB_BE_MASTER;}
  
    /**
     * Method only in the Java Version available. Returns TRUE if type maske equals WKB_BE_DATA.
     */
    public boolean isDATA() { return (flCtrl & WKB_BE_TYPEMASK) == WKB_BE_DATA;}
   
    /**
     * Method only in the Java Version available. Returns TRUE if type maske equals WKB_BE_EMPTY.
     */
    public boolean isEMPTY() { return (flCtrl & WKB_BE_TYPEMASK) == WKB_BE_EMPTY;}
   
    /**
     * Method only in the Java Version available. Returns TRUE if type maske equals WKB_BE_DEFECT.
     */
    public boolean isDEFECT() { return (flCtrl & WKB_BE_TYPEMASK) == WKB_BE_DEFECT;}
  
  
    /**
     * Method only in the Java Version available. Returns TRUE if type maske equals WKB_BE_ADDLIMIT.
     */
    public boolean hasADDLIMIT() { return (flCtrl & WKB_BE_ADDMASK) == WKB_BE_ADDLIMIT;}
  
    /**
     * Method only in the Java Version available. Returns TRUE if type maske equals WKB_BE_ADDDATA.
     */
    public boolean hasADDDATA() { return (flCtrl & WKB_BE_ADDMASK) == WKB_BE_ADDDATA;}
   
    /**
     * Method only in the Java Version available. Returns TRUE if type maske equals WKB_BE_ADDEXPDATE.
     */
    public boolean hasADDEXPDATE() { return (flCtrl & WKB_BE_ADDMASK) == WKB_BE_ADDEXPDATE;}
  }


  // ******************************************************************************************************** //
  // Commands and Flags
  // ******************************************************************************************************** //
 
  /**
   * (0x0000) The entry is empty. In the WIBU-BOX an entry is empty if all 6 bytes are all set to 0.
   */
  public static final long WKB_BE_EMPTY         = 0x0000;
 
  /**
   * (0x0100) The entry is a standard FC/UC entry with a valid Firm and User Code.
   */
  public static final long WKB_BE_FCUC          = 0x0100;
 
  /**
   * 0x0200) The entry is a Master Entry with a valid Firm Code and a Master Mask instead of a User Code.
   */
  public static final long WKB_BE_MASTER        = 0x0200;
 
  /**
   * (0x0300) The entry is a (not-bundled) User Data entry, which contains 40 bits of general data.
   */
  public static final long WKB_BE_DATA          = 0x0300;
 
  /**
   * (0x0D00) The entry not empty but allocated with any (unknown) data. This type is used in the Remote
   * Programming Context description only.
   */
  public static final long WKB_BE_FILLED        = 0x0D00;
 
  /**
   * (0x0E00) The entry is defect and cannot be cleared completely to receive a new valid entry.
   */
  public static final long WKB_BE_DEFECT        = 0x0E00;
 
  /**
   * (0x0F00) The entry is invalid because it is contains data which cannot be currently
   * interpreted for any useful thing.
   */
  public static final long WKB_BE_INVALID       = 0x0F00;
 
  /**
   * Using this flag you can separate the type flags (WKB_BE_EMPTY, WKB_BE_FCUC, 
   * WKB_BE_MASTER, ...) from the flCtrl member.
   */
  public static final long WKB_BE_TYPEMASK      = 0x0F00;
 
  /**
   * This mask selects the index of the WIBU-BOX entry, which is a value in the range 
   * from 0 and 9, which is displayed by programs like WIBUKE32.CPL or WKU32.EXE as 
   * entry 1 up to 10. 
   */
  public static final long	WKB_BE_INDEXMASK	= 0x000F;

  /**
   * (0x1000) The entry has a Limit Counter entry.
   */
  public static final long WKB_BE_ADDLIMIT      = 0x1000;
 
  /**
   * (0x2000) The entry has an Added Data entry.
   */
  public static final long WKB_BE_ADDDATA       = 0x2000;
 
  /**
   * (0x4000) The entry has an Expiration Date entry.
   */
  public static final long WKB_BE_ADDEXPDATE    = 0x4000;
 
  /**
   *
   */
  public static final long WKB_BE_ADDMASK       = 0x7000;
 
  /**
   * (0x10000) The programming signature, which is used to modify the data contents of the Added Data or
   * Expiration Date entry, depends also on the Serial Number of the addressed WIBU-BOX.
   */
  public static final long WKB_DATA_SERIALVARY  = 0x10000;
                                                          
  /**
   * (0x20000) The programming signature, which is used to modify the data contents of the Added Data or
   * Expiration Date entry, depends also on the WIBU-BOX internal programming counter and varies with each
   * programming operation.
   */
  public static final long WKB_DATA_COUNTVARY   = 0x20000;
                                                          
  /**
   * (0x100000) The Limit Counter value replaced by the sum of the current value of the Limit Counter in the
   * WIBU-BOX and the offset, which is specified in the ulData2 member. The result of this addition is limited
   * to 20 bits; a reduction of the Limit Counter value is possible by specifying the two-complement of the
   * difference to the lower value into ulData2. This option is only supported by WIBU-BOX with version 4 or more.
   */
  public static final long WKB_DATA_DATAVARY    = 0x40000;
 
  /**
   * (0x000000, default) The Limit Counter value is replaced by the absolute value,
   * which is specified in the ulData2 member. Only for programming a Limit Counter:
   */
  public static final long WKB_LIMIT_ABSOLUTE   = 0x000000;
 
  /**
   * Only for programming a Limit Counter
   */
  public static final long WKB_LIMIT_OFFSET     = 0x100000;


  // ******************************************************************************************************** //
  // Structure WKBOPENENTRY
  // ******************************************************************************************************** //

  /**
   * This class is returned by the <A HREF = "WkJavaApi.html#WkbQueryEntry2(long, long, com.wibu.WkJavaApi.WKBBOXENTRY)">WkbQueryEntry2</A>
   * method and can be used to open a specific entry via the <A HREF = "WkJavaApi.html#WkbOpen2(long, long,
   * java.lang.String, long, long, java.lang.Object)">WkbOpen2</A> method.
   */
  public static class WKBOPENENTRY
  {
    /**
     * This member contains additional information which specifies some opening variants.
     */
    public long flCtrl;
   
    /**
     * contains the handle of the subsystem which was used to open the specified entry.
     */
    public long hwkbsys;
   
    /**
     * contains the logical port number which is used to address the port by a number with the WKB_PORTNUMBER option.
     */
    public long lPort;
   
    /**
     * contains the logical port name as a 0-terminated string.
     */
    public StringBuffer pszPort = new StringBuffer("");
   
    /**
     * contains the Serial Number of the opened WIBU-BOX in a WKBSERIAL class.
     */
    public WKBSERIAL    wkbsr   = new WKBSERIAL();
   
    /**
     * contains the version of the opened WIBU-BOX. This is a value started with 1 for WIBU-BOX version 1,
     * 2 for WIBU-BOX version 2 etc.
     */
    public short usVersion;
   
    /**
     * contains the index of the opened entry in the WIBU-BOX. This is a value in the range from 0 to 9.
     */
    public short usEntry;
   
    /**
     * contains the Firm Code which was used to open the entry.
     */
    public long ulFirmCode;
   
    /**
     * contains the User Code which was used to open the entry. If a Master Entry is opened, this member
     * contains furthermore the User Code, which is used for the selection and encryption or decryption operations.
     */
    public long ulUserCode;
  }

 
  // ******************************************************************************************************** //
  //                                    List Port and WIBU-BOX Contents
  // ******************************************************************************************************** //
 
  // ******************************************************************************************************** //
  // NATIVE CALL WkbEnumPorts2
  // ******************************************************************************************************** //
 
  /**
   * This method determines all available WibuKey ports at the specified WibuKey subsystem and writes the names of 
   * all these ports into the specified destination string buffer. A vertical bar character ('|') separates these 
   * partial strings. A second calling variant returns this list name by name via subsequent method calls. 
   *
   * @param hwkbsys contains handle of an accessed WibuKey subsystem. For using the WIBU-BOXes at the local system,
   *                the predefined system handle HWKB_LOCAL can be used.

   * @param flCtrl contains flags which control the execution of the operation. Following command codes are currently supported: 
   * <ul>
   *   <li> WKB_EPT_LIST (0x0000) pszDest receives a string containing the names of all available WibuKey ports, 
   *   separated by the vertical bar.	
   *   <li> WKB_EPT_NEXT (0x0010) pszDest receives a string which corresponds with the next name of an available WibuKey
   *   port which follows in the name ordering after the name which is specified in the pszDest if the 
   *   method is called. If the latter name is an empty string or a star (*), the first available WibuKey port name is 
   *   returned. If a wildcard name is used, for example LPT*, the enumeration begins at the first port name which 
   *   corresponds with this wildcard. After all available port names were returned back, the method returns with 0 and 
   *   a subsequent call of the WkbGetLastError method returns the WKB_ERROR_INVALID_PORT error code. 
   *   <li> This parameter contains also standard function control flags. 
   * </ul>
   * @param pszDest list of all port names or with the next port name depending on flCtrl.
   *
   * @param cchDest  not used in Java. Has to be set to any integer value, e.g. 0.
   * <!--<font color="#FF00FF"><b> ??? </b>Vorschlag: Neue Funktion ohne cchDest, alte auf deprecated. </font>
   * -->
   *
   * @return 0 if any error occurred.
   * In this case a subsequent call of the <A HREF = "WkJavaApi.html#WkbGetLastError()">WkbGetLastError</A> method 
   * returns the corresponding error code.
   *
   */
   public static native int WkbEnumPorts2(long hwkbsys, long flCtrl, StringBuffer pszDest, int cchDest);
    
   /**
    * (0x0000) The buffer specified by the pszDest parameter receives a string containing the names of all
    * available WibuKey ports, separated by the vertical bar and terminated by a 0 character.
    */
   public static final long WKB_EPT_LIST        = 0x0000;
  
   /**
    * (0x0010) The buffer specified by the pszDest parameter receives a string which corresponds with the next
    * name of an available WibuKey ports which follows in the name ordering after the name which is specified
    * in the pszDest buffer if the method is called. If the latter name is an empty string or a star (*), the
    * first available WibuKey port name is returned. If a wildcard name is used, for example LPT*, the enumeration
    * begins at the first port name which corresponds with this wildcard. After all available port names were
    * returned back, the method returns with 0 and a subsequent call of the WkbGetLastError method returns the
    * WKB_ERROR_INVALID_PORT error code.
    */
   public static final long WKB_EPT_NEXT        = 0x0010;


   // ******************************************************************************************************** //
   // NATIVE CALL WkbListPort2
   // ******************************************************************************************************** //

   /**
    * This method lists the Serial Numbers of all WIBU-BOXes, which are found at the specified WIBU-BOX port.
    *
    * @param hwkbsys contains the handle of a valid accessed WibuKey subsystem. To list a port of the standard
    * local subsystem the predefined handle value HWKB_LOCAL can be used.
    *
    * @param flCtrl contains flags which control the operation of the method.
    *
    * @param pszPort contains one single port specification. 
    *
    * @param pvDest receives the list information. For this version of the method a 
    * <A HREF = "WkJavaApi.WKBBOXLIST.html">WKBBOXLIST</A> class with an array of ten 
    * <A HREF = "WkJavaApi.WKBSERIAL.html">WKBSERIAL</A> classes in awkbsr[] is used.
    * 
    * @param cbDest not used in Java. Has to be set to any integer value, e.g. 0.
    * <!--<font color="#FF00FF"><b> ??? </b> Nicht mehr benutzt. Vorschlag: Neue Funktion ohne cbDest, alte deprecated. </font>
    * -->
    *
    * @return 0 if any error occurred.
    * In this case a subsequent call of the <A HREF = "WkJavaApi.html#WkbGetLastError()">WkbGetLastError</A> method 
    * returns the corresponding error code.
    */
   public static native int WkbListPort2(long hwkbsys, long flCtrl, String pszPort, Object pvDest, int cbDest);
  
   /**
    * This member is returned by the method used by WkbListPort2 method and specifies the number of the
    * found WIBU-BOXes and their Serial Numbers at the specified WibuKey port.
    */
   public static class WKBBOXLIST
   {
        /**
         * contains flags which control the interpretation of the structure. Structure specific flags are
         * currently not used and therefore this parameter must be set to 0 except standard structure flags are set.
         */
        public long flCtrl;
       
        /**
         * contains the number of WIBU-BOXes which are found at the specified WibuKey port. This value is
         * independent of the number of actually stored WIBU-BOX Serial Numbers into this structure.
         */
        public short cBox;
       
        /**
         * and reserved for future use and must be set to 0.
         */
        public short usReserved;
       
        /**
         * contains a dynamic length array which contains the Serial Numbers of the found WIBU-BOXes.
         * The Serial Numbers are specified by the WKBSERIAL structure.
         */
        public WKBSERIAL[] awkbsr = new WKBSERIAL[1];
   }
       
       
  // ******************************************************************************************************** //
  // NATIVE CALL WkbListBox2
  // ******************************************************************************************************** //

  /**
   * This method determines the contents of the specified WIBU-BOX.
   *
   * @param hwkbsys contains the handle of a valid accessed WibuKey subsystem. To list the contents of a WIBU-BOX at 
   * the standard local subsystem the predefined handle value HWKB_LOCAL can be used.
   *
   * @param flCtrl contains flags which control the operation of the method. The flag WKB_LIST_DATA will be set by this method.
   * <ul>
   *   <li> This parameter contains also standard function control flags. 
   * </ul>
   *
   * @param pszPort contains one single port specification.
   *
   * @param pwkbsr contains Serial Number of the WIBU-BOX in a 
   * <A HREF = "WkJavaApi.WKBSERIAL.html">WKBSERIAL</A> class.
   *
   * @param pvDest receives the list information. For this variant of the method a
   * <A HREF = "WkJavaApi.WKBBOXDATA.html">WKBBOXDATA</A> class is used.
   *
   * @return 0 if any error occurred.
   * In this case a subsequent call of the <A HREF = "WkJavaApi.html#WkbGetLastError()">WkbGetLastError</A> method 
   * returns the corresponding error code.
   */
  public static long WkbListBox2(long hwkbsys, long flCtrl, String pszPort, WKBSERIAL pwkbsr, WKBBOXDATA pvDest)
  {
    return WkbListBox2(hwkbsys, flCtrl, 0, pszPort, pwkbsr, pvDest);
  }

  /**
   * This method determines the contents of the specified WIBU-BOX. 
   * This variant gets the configuration data of the specified WIBU-BOX.
   *
   * @param hwkbsys contains the handle of a valid accessed WibuKey subsystem. To list the contents of a WIBU-BOX at 
   * the standard local subsystem the predefined handle value HWKB_LOCAL can be used.
   *
   * @param flCtrl contains flags which control the operation of the method. The flag WKB_LIST_CONFIG will be set by this method.
   * <ul>
   *   <li> This parameter contains also standard function control flags. 
   * </ul>
   *
   * @param pszPort contains one single port specification.
   *
   * @param pwkbsr contains Serial Number of the WIBU-BOX in a 
   * <A HREF = "WkJavaApi.WKBSERIAL.html">WKBSERIAL</A> class.
   *
   * @param pvDest receives the list information. For this variant of the method a
   * <A HREF = "WkJavaApi.WKBBOXCONFIG.html">WKBBOXCONFIG</A> class is used.
   *
   * @return 0 if any error occurred.
   * In this case a subsequent call of the <A HREF = "WkJavaApi.html#WkbGetLastError()">WkbGetLastError</A> method 
   * returns the corresponding error code.
   */
  public static long WkbListBox2(long hwkbsys, long flCtrl, String pszPort, WKBSERIAL pwkbsr, WKBBOXCONFIG pvDest)
  {
    return WkbListBox2(hwkbsys, flCtrl, 0, pszPort, pwkbsr, pvDest);
  }

  /**
   * This method determines the extended memory contents of the specified WIBU-BOX.
   *
   * @param hwkbsys contains the handle of a valid accessed WibuKey subsystem. To list the contents of a WIBU-BOX at 
   * the standard local subsystem the predefined handle value HWKB_LOCAL can be used.
   *
   * @param flCtrl contains flags which control the operation of the method. The flag WKB_LIST_EXTMEM will be set by this method.
   * <ul>
   *   <li> This parameter contains standard function control flags. 
   * </ul>
   *
   * @param pszPort contains one single port specification.
   *
   * @param pwkbsr contains Serial Number of the WIBU-BOX in a 
   * <A HREF = "WkJavaApi.WKBSERIAL.html">WKBSERIAL</A> class.
   *
   * @param pvDest receives the extended memory. For this variant of the method a
   * <A HREF = "WkJavaApi.WKBEXTMEM.html">WKBEXTMEM</A> class is used.
   *
   * @return 0 if any error occurred.
   * In this case a subsequent call of the <A HREF = "WkJavaApi.html#WkbGetLastError()">WkbGetLastError</A> method 
   * returns the corresponding error code.
   *
   */
  public static native long WkbListBox2(long hwkbsys, long flCtrl, String pszPort, WKBSERIAL pwkbsr, WKBEXTMEM pvDest);

  /**
   * This method determines the contents of the specified WIBU-BOX.
   *
   * @param hwkbsys contains the handle of a valid accessed WibuKey subsystem. To list the contents of a WIBU-BOX at 
   * the standard local subsystem the predefined handle value HWKB_LOCAL can be used.
   *
   * @param flCtrl contains flags which control the operation of the method. The flag WKB_LIST_DATA will be set by this method.
   * <ul>
   *   <li> WKB_PORTNUMBER (0x0008) is set to specify a port number instead of a port name. 
   *   <li> This parameter contains also standard function control flags. 
   * </ul>
   *
   * @param lPort contains a valid port specification number, which is compatible with WibuKey version 1 (used when the flag 
   * WKB_PORTNUMBER is set).
   *
   * @param pszPort contains one single port specification. (used when the flag WKB_PORTNUMBER is NOT set).
   *
   * @param pwkbsr contains Serial Number of the WIBU-BOX in a 
   * <A HREF = "WkJavaApi.WKBSERIAL.html">WKBSERIAL</A> class.
   *
   * @param pvDest receives the list information. For this variant of the method a
   * <A HREF = "WkJavaApi.WKBBOXDATA.html">WKBBOXDATA</A> class is used.
   *
   * @return 0 if any error occurred.
   * In this case a subsequent call of the <A HREF = "WkJavaApi.html#WkbGetLastError()">WkbGetLastError</A> method 
   * returns the corresponding error code.
   *
   * @deprecated This method is obsolete and replaced by <A HREF = 
   * "WkJavaApi.html#WkbListBox2(long, long, java.lang.String, com.wibu.WkJavaApi.WKBSERIAL, com.wibu.WkJavaApi.WKBBOXDATA)">
   * WkbListBox2</A>
   */
  public static native long WkbListBox2(long hwkbsys, long flCtrl, long lPort, String pszPort, WKBSERIAL pwkbsr, WKBBOXDATA pvDest);

  /**
   * This method determines the contents of the specified WIBU-BOX.
   *
   * @param hwkbsys contains the handle of a valid accessed WibuKey subsystem. To list the contents of a WIBU-BOX at 
   * the standard local subsystem the predefined handle value HWKB_LOCAL can be used.
   *
   * @param flCtrl contains flags which control the operation of the method. The flag WKB_LIST_DATA will be set by this method.
   * <ul>
   *   <li> WKB_PORTNUMBER (0x0008) is set to specify a port number instead of a port name. Has to be set for this variant of this
   *        method. 
   *   <li> This parameter contains also standard function control flags. 
   * </ul>
   *
   * @param lPort contains a valid port specification number, which is compatible with WibuKey version 1.
   *
   * @param pwkbsr contains Serial Number of the WIBU-BOX in a 
   * <A HREF = "WkJavaApi.WKBSERIAL.html">WKBSERIAL</A> class.
   *
   * @param pvDest receives the list information. For this variant of the method a
   * <A HREF = "WkJavaApi.WKBBOXDATA.html">WKBBOXDATA</A> class is used.
   *
   * @return 0 if any error occurred.
   * In this case a subsequent call of the <A HREF = "WkJavaApi.html#WkbGetLastError()">WkbGetLastError</A> method 
   * returns the corresponding error code.
   *
   * @deprecated This method is obsolete and replaced by <A HREF = 
   * "WkJavaApi.html#WkbListBox2(long, long, java.lang.String, com.wibu.WkJavaApi.WKBSERIAL, com.wibu.WkJavaApi.WKBBOXDATA)">
   * WkbListBox2</A>
   */
  public static long WkbListBox2(long hwkbsys, long flCtrl, long lPort, WKBSERIAL pwkbsr, WKBBOXDATA pvDest)
  {
    return WkbListBox2(hwkbsys, flCtrl, lPort, null, pwkbsr, pvDest);
  }

  /**
   * This method determines the contents of the specified WIBU-BOX. 
   * This variant gets the configuration data of the specified WIBU-BOX.
   *
   * @param hwkbsys contains the handle of a valid accessed WibuKey subsystem. To list the contents of a WIBU-BOX at 
   * the standard local subsystem the predefined handle value HWKB_LOCAL can be used.
   *
   * @param flCtrl contains flags which control the operation of the method. The flag WKB_LIST_CONFIG will be set by this method.
   * <ul>
   *   <li> WKB_PORTNUMBER (0x0008) is set to specify a port number instead of a port name. 
   *   <li> This parameter contains also standard function control flags. 
   * </ul>
   *
   * @param lPort contains a valid port specification number, which is compatible with WibuKey version 1 (used when the flag 
   * WKB_PORTNUMBER is set).
   *
   * @param pszPort contains one single port specification (used when the flag WKB_PORTNUMBER is NOT set).
   *
   * @param pwkbsr contains Serial Number of the WIBU-BOX in a 
   * <A HREF = "WkJavaApi.WKBSERIAL.html">WKBSERIAL</A> class.
   *
   * @param pvDest receives the list information. For this variant of the method a
   * <A HREF = "WkJavaApi.WKBBOXCONFIG.html">WKBBOXCONFIG</A> class is used.
   *
   * @return 0 if any error occurred.
   * In this case a subsequent call of the <A HREF = "WkJavaApi.html#WkbGetLastError()">WkbGetLastError</A> method 
   * returns the corresponding error code.
   *
   * @deprecated This method is obsolete and replaced by <A HREF = 
   * "WkJavaApi.html#WkbListBox2(long, long, java.lang.String, com.wibu.WkJavaApi.WKBSERIAL, com.wibu.WkJavaApi.WKBBOXCONFIG)">
   * WkbListBox2</A>
   */
  public static native long WkbListBox2(long hwkbsys, long flCtrl, long lPort, String pszPort, WKBSERIAL pwkbsr, WKBBOXCONFIG pvDest);

  /**
   * This method determines the contents of the specified WIBU-BOX. 
   * This variant gets the configuration data of the specified WIBU-BOX.
   *
   * @param hwkbsys contains the handle of a valid accessed WibuKey subsystem. To list the contents of a WIBU-BOX at 
   * the standard local subsystem the predefined handle value HWKB_LOCAL can be used.
   *
   * @param flCtrl contains flags which control the operation of the method. The flag WKB_LIST_CONFIG will be set by this method.
   * <ul>
   *   <li> WKB_PORTNUMBER (0x0008) is set to specify a port number instead of a port name. Has to be set for this variant of this
   *        method. 
   *   <li> This parameter contains also standard function control flags. 
   * </ul>
   *
   * @param lPort contains a valid port specification number, which is compatible with WibuKey version 1.
   *
   * @param pwkbsr contains Serial Number of the WIBU-BOX in a 
   * <A HREF = "WkJavaApi.WKBSERIAL.html">WKBSERIAL</A> class.
   *
   * @param pvDest receives the list information. For this variant of the method a
   * <A HREF = "WkJavaApi.WKBBOXCONFIG.html">WKBBOXCONFIG</A> class is used.
   *
   * @return 0 if any error occurred.
   * In this case a subsequent call of the <A HREF = "WkJavaApi.html#WkbGetLastError()">WkbGetLastError</A> method 
   * returns the corresponding error code.
   *
   * @deprecated This method is obsolete and replaced by <A HREF = 
   * "WkJavaApi.html#WkbListBox2(long, long, java.lang.String, com.wibu.WkJavaApi.WKBSERIAL, com.wibu.WkJavaApi.WKBBOXCONFIG)">
   * WkbListBox2</A>
   */
  public static long WkbListBox2(long hwkbsys, long flCtrl, long lPort, WKBSERIAL pwkbsr, WKBBOXCONFIG pvDest)
  {
    return WkbListBox2(hwkbsys, flCtrl, lPort, null, pwkbsr, pvDest);
  }
  
  /**
   * (0x0000) returns the complete contents of the entries of the WIBU-BOX into a WKBBOXDATA structure at pvDest.
   */
  public static final long WKB_LIST_DATA        = 0x0000;
 
  /**
   * (0x0010) returns the configuration data of the WIBU-BOX into a WKBBOXCONFIG structure at pvDest.
   */                                                   
  public static final long WKB_LIST_CONFIG      = 0x0010;
 
  /**
   * (0x0020) returns the contents of the extended memory storage pages into a WKBEXTMEM structure at pvDest.
   */                                                   
  public static final long WKB_LIST_EXTMEM      = 0x0020;

  /**
   * (0x0008) is set to specify a port number instead of a port name.
   */
  public static final long WKB_PORTNUMBER       = 0x0008;


  /**
   * This member is returned by the method used by the WkbListPort2 or by the
   * <A HREF = "WkJavaApi.html#WkbQueryEntry2(long, long, com.wibu.WkJavaApi.WKBBOXENTRY)">WkbQueryEntry2</A>
   * method and specifies the entry contents of a complete WIBU-BOX or of all opened entries.
   */
  public static class WKBBOXDATA
  {
          /**
           * contains flags which control the interpretation of the structure. Structure specific flags are currently
           * not used and therefore this parameter must be set to 0 except standard structure flags are set.
           */
          public long flCtrl;
          
          /**
           * contains the version of the read WIBU-BOX. It is 1 for the classic silver WIBU-BOX and 3, 4 or 5 for
           * all other WIBU-BOXes. The WIBU-BOX/U has always 4 as version and the WIBU-BOX/ST always 5 as version.
           */
          public short usVersion;
         
          /**
           * contains the number of filled or opened base entries of the WIBU-BOX. The contents of these entries
           * are specified in the awkbbxe member.
           */
          public short cEntry;
         
          /**
           * contains the contents of up to 10 base entries of the WIBU-BOX. Each entry is stored as a
           * WKBBOXENTRY class.
           */
          public WKBBOXENTRY[] awkbbxe = new WKBBOXENTRY[10];
  }


  /**
   * This member contains the configuration contents of a WIBU-BOX which is returned by the WkbListBox2
   * method and can be changed via the WkbProgram2 method.
   */
  public static class WKBBOXCONFIG
  {
          /**
           * contains flags which control the interpretation of the structure. Structure specific flags are
           * currently not used and therefore this parameter must be set to 0 except standard structure
           * flags are set.
           */
          public long flCtrl;
         
          /**
           * contains the version of the read WIBU-BOX. It is 1 for the classic silver WIBU-BOX and 3 or 4
           * for the slim turquoise-blue WIBU-BOX.
           */
          public short usVersion;
         
          /**
           * and reserved for future use and must be set to 0.
           */
          public short usReserved;
         
          /**
           * contains the value of the WIBU-BOX internal programming counter which is increased by every
           * signature protected programming operation.
           */
          public long ulProgramCount;
         
          /**
           * contains the configuration values of the WIBU-BOX. The current version of the WIBU-BOX contains
           * only 4 configuration flags, which corresponds with bit 0 to 3 of this entry. Bit 4 up to 31 are
           * always set to 0.
           */
          public long ulConfig;

          /**
           *  number of pages in the ExtMemSize totally 
           **/
          public short usExtMemSize;   

          /** 
           * WIBU-BOX ASIC build number 
           **/
	        public short usAsicBuild;    

  } // class WKBBOXCONFIG


  // ******************************************************************************************************** //
  // NATIVE CALL WkbClose2
  // ******************************************************************************************************** //

  /**
   * This method closes the WIBU-BOX entry, which was opened by a previous call of the
   * <A HREF = "WkJavaApi.html#WkbOpen2(long, long, java.lang.String, long, long, java.lang.Object)">WkbOpen2</A> method.
   * This method cancels also all asynchronously started but not completed method calls regarding the
   * closed WIBU-BOX entry.
   *
   * @param hwkbe contains the handle of an opened WIBU-BOX entry.
   *
   * @return TRUE if the operation was successful and FALSE if an error occurred.
   * In this case a subsequent call of the <A HREF = "WkJavaApi.html#WkbGetLastError()">WkbGetLastError</A> method 
   * returns the corresponding error code.
   */
  public static native boolean WkbClose2(long hwkbe);


  // ******************************************************************************************************** //
  //                                    Subsystem (Network) Management
  // ******************************************************************************************************** //

  // ******************************************************************************************************** //
  // NATIVE CALL WkbAccess2
  // ******************************************************************************************************** //

  /**
   * This method accesses a WibuKey in a local or network based subsystem. A subsystem is determined locally by the
   * specified Firm and User Code or by a WkLAN or WkNet server and one or more WIBU-BOXes with doubled entries:
   * The Firm and User Code of the first entry must be the same like the specified Firm and User Code and is the
   * <i>access entry</i>; the entry with the same Firm Code and the next higher User Code limits the concurrent access to
   * this subsystem. If only one entry is specified in the WIBU-BOX or the difference between the User Code in the
   * access entry and the next higher User Code is more than 255 (the maximum user limit), then the limit of
   * concurrent accesses is set to 1 (single user operation).
   *
   * @param flCtrl contains a combination of flags which controls the execution of the method. Following command codes are possible:
   * <ul>
   *   <li> WKB_ACC_CREATE (0x00) creates a new subsystem. 
   *   <li> For the WKB_ACC_CREATE command, one or more of the following option flags must be used in combination: 
   *   <ul>
   *     <li> WKB_ACC_LOCAL (0x01) tries to open a local WIBU-BOX with the specified Firm and User Code. If such a 
   *          WIBU-BOX is found, a subsystem is used which is compatible to the standard subsystem, addressed by the HWKB_LOCAL 
   *          constant, except that the default setting of the standard bits in the control flag parameters and members may be 
   *          specified and the returned subsystem handle has not the value of HWKB_LOCAL. 
   *     <li> WKB_ACC_WKLAN (0x02) tries to access a WkLAN network subsystem, which corresponds with the specified 
   *          control parameters. 
   *     <li> WKB_ACC_WKNET (0x04) tries to access a WkNet network subsystem, which corresponds with the specified 
   *          control parameters. 
   *     <li> For all other commands, none of these flags may be used. The restore command sets the same subsystem 
   *          which status data were determined via a call of the WkbQuerySystem2 method. 
   *     <li> If two or more of these flags are set, the method tries at first the local WIBU-BOX, then the WkLAN 
   *          subsystem and at least the WkNet subsystem. This ranking guarantees the selection of the fastest possible 
   *          subsystem. 
   *     <li> Following flag is only used in combination with the WKB_ACC_WKNET flag: 
   *     <ul>
   *       <li> WKB_ACC_DIRECT (0x100000) addresses the specified network subsystem directly without allocating any 
   *            user. This option is used to control the management of the subsystem, usually via the WkbControlSystem2 method. 
   *     </ul>
   *   </ul>
   *   <li> One of the following flag may be used with the WKB_ACC_CREATE option: 
   *   <ul>
   *     <li> WKB_ACC_NONETLIMIT (0x200000) forces that any WkbOpen2 operation using the created subsystem 
   *          automatically uses the WKB_OPEN_NONETLIMIT option. If the WkLAN server receives a login with this option, it 
   *          allocates no slot in a network cluster. Using this option permits the creating of a local subsystem if the used 
   *          WIBU-BOX entry is already used by a WkbOpen2 operation with the WKB_OPEN_GLOBAL option. 
   *     <li> WKB_ACC_STATIONSHARE (0x800000) forces that any WkbOpen2 operation using the created subsystem 
   *          automatically uses the WKB_OPEN_STATIONSHARE option. If the WkLAN server receives a login with this option, it 
   *          allocates a slot in a network cluster only if not another process on the same client computer (same network 
   *          address) has already allocated such a slot. Otherwise this slot is also used for the new slot ('Station Share' 
   *          condition). 
   *     <li> If none of these options is specified, the Network User Quantity Limitation is active with an allocated 
   *          user per started application. 
   *     <li> Following flag may be used with any subsystem variant, but is ignored for a WkNet subsystem. It must not 
   *          be combined with the WKC_ACC_STATIONSHARE flag: 
   *     <ul>
   *       <li> WKB_ACC_EXCLUSIVE (0x400000) activates the Exclusive Mode for the accessed WIBU-BOX entry. Each WIBU-
   *            BOX entry may be accessed only once at same time for a specific Firm Code and User Code if this flag is set. This 
   *            flag must be set if an application should be started only once within its local or WkLAN subsystem for its specific 
   *            Firm Code and User Code. 
   *     </ul>
   *   </ul>
   *   <li> This parameter contains also standard function control flags. 
   * </ul>
   *
   * @param pvCtrl contains a <A HREF = "WkJavaApi.WKBACCESS.html">WKBACCESS</A> class, which contains further 
   * information to specify the type of accessing. 
   *
   * @return handle (hwkbsys) or 0 if any error occurred.
   * In this case a subsequent call of the <A HREF = "WkJavaApi.html#WkbGetLastError()">WkbGetLastError</A> method 
   * returns the corresponding error code.
   */
  public static native long WkbAccess2(long flCtrl, final WKBACCESS pvCtrl);


  /**
   * (0x20) creates a new local subsystem, which is not protected by a WIBU-BOX entry. Only the Firm and User Code
   * in the WKBACCESS are stored into the subsystem to use later by <A HREF = "WkJavaApi.html#WkbOpen2(long, long,
   * java.lang.String, long, long, java.lang.Object)">WkbOpen2</A> calls with the WKB_ACCESS_CODE constant.
   * A shell based subsystem can be used to cleanup a class of allocated resources like opened WIBU-BOXes or allocated
   * memory blocks with a simple <A HREF = "WkJavaApi.html#WkbRelease2(long)">WkbRelease2</A> call, where all these resources
   * were created via this subsystem.
   */
  public static final long WKB_ACC_SHELL      = 0x20;  
 
  /**
   * (0x00) creates a new subsystem.
   */
  public static final long WKB_ACC_CREATE      = 0x00;
  
  /**
   * (0x01) tries to open a local WIBU-BOX with the specified Firm and User Code. If such a WIBU-BOX is found,
   * a subsystem is used which is compatible to the standard subsystem, addressed by the HWKB_LOCAL constant, except
   * that the default setting of the standard bits in the control flag parameters and members may be specified and
   * the returned subsystem handle has not the value of HWKB_LOCAL.
   */
  public static final long WKB_ACC_LOCAL      = 0x01;
 
  /**
   * (0x02) tries to access a WkLAN network subsystem, which corresponds with the specified control parameters.
   */
  public static final long WKB_ACC_WKLAN      = 0x02;
 
  /**
   * (0x04) WKNET not yet implemented in Java version
   */
  public static final long WKB_ACC_WKNET      = 0x04;  

  /**
   * (0x100000) direct access to WkNet management 
   */
  public static final long WKB_ACC_DIRECT       = 0x100000;
  
  /**
   * (0x200000) no touch of net user limitation
   */
  public static final long WKB_ACC_NONETLIMIT   = 0x200000;
  /**
   * (0x400000) exclusive access mode
   */
  public static final long WKB_ACC_EXCLUSIVE    = 0x400000;
  /**
   * (0x800000) station share access mode
   */
  public static final long WKB_ACC_STATIONSHARE = 0x800000;

  /**
   * This structure controls the creation of a specific WibuKey subsystem. It can be used in combination with
   * the WKB_ACC_CREATE flag and the <A HREF = "WkJavaApi.html#WkbAccess2(long, com.wibu.WkJavaApi.WKBACCESS)">WkbAccess2</A> method.
   * ATTENTION. It is only the minimal structure for local WkLAN. Attributes for WkNet are not implemented yet.
   */
  public static class WKBACCESS
  {
   
    /**
     * contains further control flags.
     */
    public long flCtrl = 0; //!I:jo BUG JAV009: WKB_USEOFFSETS;
   
    /**
     * contains the Firm Code which is used for the WkLAN or the WkNet access entry to access the local subsystem.
     */
    public long lFirmCode = 0;
   
    /**
     * contains the User Code which is used for the WkLAN or the WkNet access entry or to access the local subsystem.
     */
    public long lUserCode = 0;
   
    /**
     * contains the default setting of the bits 8 up to 19 of the flCtrl parameter or in the flCtrl member of a
     * transferred structure of any WibuKey-API base method which is called for the created subsystem
     * (see Standard Function Control Flags). The bit area from 8 to 19 contains the level byte and the Unicode (UC),
     * Asynchronous Mode (AM) and Pointer Mode (PM) flags. If a method wants to override these default setting,
     * then the exclusive Mode (XM) flag must be set additionally in the flCtrl parameter of this method or the
     * transferred structure.
     */
    public long flStdCtrl = WKB_STDCTRL;
   
    /**
     * contains the minor version number of the needed minimum subsystem version. If 0 is specified, no
     * specific version is required.
     */
    public byte bVersionLow = 0;               
   
    /**
     * contains the major version number (currently 1 or 2) of the needed minimum subsystem version. If 0 is
     * specified, no specific version is required.
     */
    public byte bVersionHigh = 0;              
   
    /**
     * contains the user identification of the desired net user. If this value is set to WKB_NETUSER_REG (0xFFFE),
     * this number is read from the [Net User] section of the driver setting (see below), which is not implemented
     * under DOS. If this value is set to WKB_NETUSER_FIND (0xFFFF), this number is not fixed but the first free user
     * identification is chosen, locked and returned.
     */
    public int idNetUser = WKB_NETUSER_FIND;
   
    /**
     * points to a string with the name of the application which is protected. This name is used for information
     * purposes, for example in the WkLAN administration list of locked user. This name is also used to read the
     * user number (if WKB_NETUSER_REG is specified) and optionally the complete path of the WkNet Server file from
     * the WIBUKEY.INI file, which is not implemented under DOS. If this name is not used, NULL must be specified
     * in this member.
     */
    public StringBuffer pszAppName = null;
   
    /**
     * points to a string with the name of a specific WkLAN server which should be addressed. If this member is set
     * to NULL, no specific WkLAN server is addressed but all available servers are scanned. To specify a name here
     * is useful to select a specific server for listing and programming operations or other administration purposes.
     */
    public StringBuffer pszServerName = null;
  } // class WKBACCESS
 
 
  /**
   * (0x0001) This option is only valid if the WKB_ACC_EXCLUSIVE option in the flCtrl parameter of
   * <A HREF = "WkJavaApi.html#WkbAccess2(long, com.wibu.WkJavaApi.WKBACCESS)">WkbAccess2</A> is
   * set or if a fixed user-identification is stored in the idNetUser member of this structure. The option forces
   * the automatic cancellation of the user in the server cluster which allocates a slot from the same client computer
   * (WKB_ACC_EXCLUSIVE) or with the same user identification, so that the process, which calls
   * <A HREF = "WkJavaApi.html#WkbAccess2(long, com.wibu.WkJavaApi.WKBACCESS)">WkbAccess2</A> with
   * WKB_ACC_AUTOCANCEL can allocate now this previously locked slot.
   */
  public static final long WKB_ACC_AUTOCANCEL   = 0x0001;
 
  /**
   * (0x0002) does not display a message box on Windows during a WkLAN server is searched. This option is only valid
   * and may be specified only if in the calling of the
   * <A HREF = "WkJavaApi.html#WkbAccess2(long, com.wibu.WkJavaApi.WKBACCESS)">WkbAccess2</A> method the level is set to 1 or higher.
   */
  public static final long WKB_ACC_NO_DIALOG    = 0x0002;
 
  /**
   * (0x0004) does not display a Cancel button in a message box on Windows during a WkLAN server is searched. This
   * option is only valid and may be specified only if in the calling of the <A HREF = 
   * "WkJavaApi.html#WkbAccess2(long, com.wibu.WkJavaApi.WKBACCESS)">WkbAccess2</A> method the level is set
   * to 1 or higher. The option has no meaning if WKB_ACC_NO_DIALOG is set too.
   */
  public static final long WKB_ACC_NO_CANCEL    = 0x0004;
 
  /**
   * This value is WKB_NETUSER_FIND (0xFFFF) if this subsystem was accessed directly.
   */
  public static final int WKB_NETUSER_FIND      = 0xFFFF;
 
  /**
   * If this value is set to WKB_NETUSER_REG (0xFFFE), this number is read from the [Net User] section of the driver
   * setting (see below), which is not implemented under DOS. If this value is set to WKB_NETUSER_FIND (0xFFFF), this
   * number is not fixed but the first free user identification is chosen, locked and returned.
   */
  public static final int WKB_NETUSER_REG       = 0xFFFE;


  // ******************************************************************************************************** //
  // NATIVE CALL WkbRelease2
  // ******************************************************************************************************** //
 
  /**
   * This method releases the specified WibuKey subsystem.
   * 
   * @param hwkbsys contains the handle of a valid accessed WibuKey subsystem. The method does nothing if the 
   * predefined HWKB_LOCAL constant is specified here.
   *
   * @return TRUE if the operation was succsessfully and FALSE if an error occurred. 
   * In this case a subsequent call of the <A HREF = "WkJavaApi.html#WkbGetLastError()">WkbGetLastError</A> method 
   * returns the corresponding error code.
   */
  public static native boolean WkbRelease2(long hwkbsys);


  // ******************************************************************************************************** //
  // NATIVE CALL WkbQuerySystem2
  // ******************************************************************************************************** //

  /**
   * This method receives the system status in a <A HREF = "WkJavaApi.WKBSYSTEM.html">WKBSYSTEM</A> class. 
   * This is the status when the <A HREF = "WkJavaApi.html#WkbAccess2(long, com.wibu.WkJavaApi.WKBACCESS)">WkbAccess2</A> 
   * method was called or the current status if the WKB_QST_UPDATE option is set.
   * 
   * @param hwkbsys contains the handle of a valid accessed WibuKey subsystem. To address the standard
   * local subsystem the predefined handle value HWKB_LOCAL can be used.
   *
   * @param flCtrl contains flags which controls the execution of the method. WKB_QSY_SYSTEM will be set by this method.
   * <ul>
   *   <li> WKB_QSY_UPDATE (0x0001) returns the current state of the WKBSYSTEM structure. This includes the number of 
   *        available and free users. 
   *   <li> This parameter contains also standard function control flags. 
   * </ul>
   *
   * @param pvDest receives the state of the accessed subsystem in a WKBSYSTEM class.
   *
   * @param cbDest not used in Java. Has to be set to any integer value, e.g. 0.
   * <!--<font color="#FF00FF"><b> ??? Methode als deprecated, neue ohne cbDest?</b>not used.</font>
   * -->
   *
   * @return 0 if any error occurred.
   * In this case a subsequent call of the <A HREF = "WkJavaApi.html#WkbGetLastError()">WkbGetLastError</A> method 
   * returns the corresponding error code.
   */
  public static native int WkbQuerySystem2(long hwkbsys, long flCtrl, WKBSYSTEM pvDest, int cbDest);
 
 
   /**
   * This method receives the readable TCP/IP address of the addressed subsystem. 
   * The format of the address string is operating system dependent, for example 192.168.35.62.
   * 
   * @param hwkbsys contains the handle of a valid accessed WibuKey subsystem. To address the standard
   * local subsystem the predefined handle value HWKB_LOCAL can be used.
   *
   * @param flCtrl contains flags which controls the execution of the method. WKB_QSY_IPADDR has to be set.
   * <ul>
   *   <li> WKB_QSY_IPADDR (0x20) returns the readable TCP/IP address of the addressed subsystem.
   *   <li> This parameter contains also standard function control flags. 
   * </ul>
   *
   * @param pvDest receives the readable TCP/IP address of the addressed subsystem in a string.
   *
   * @param cbDest not used in Java. Has to be set to any integer value, e.g. 0.
   * <!--<font color="#FF00FF"><b> ??? Methode als deprecated, neue ohne cbDest?</b>not used.</font>
   * -->
   *
   * @return 0 if any error occurred.
   * In this case a subsequent call of the <A HREF = "WkJavaApi.html#WkbGetLastError()">WkbGetLastError</A> method 
   * returns the corresponding error code.
   */
  public static native int WkbQuerySystem2(long hwkbsys, long flCtrl, Object pvDest, int cbDest);
 
 
  /**
   * This structure is returned by the <A HREF = 
   * "WkJavaApi.html#WkbQuerySystem2(long, long, com.wibu.WkJavaApi.WKBSYSTEM, int)">WkbQuerySystem2</A> method and specifies 
   * partially the internal status of a WibuKey subsystem. The first entry specifies with one the WKB_ACC flag the type 
   * of the subsystem, the further members of the structure depend on this type.
   */
  public static class WKBSYSTEM
  {
        /**
         * contains further status flags.
         */
        public long flCtrl;
       
        /**
         * holds to the system name of the specified server.
         */
        public StringBuffer pszServerName = new StringBuffer("");
       
        /**
         * contains the installed network user which corresponds with the access to the specified subsystem.
         * This value is WKB_NETUSER_FIND (0xFFFF) if this subsystem was accessed directly.
         */
        public short idNetUser;
       
        /**
         * contains the maximum available users in the specified WkNet or WkLAN subsystem.
         */
        public short cMaxUser;
       
        /**
         * contains the number of free users in the specified WkNet or WkLAN subsystem at the time of the subsystem
         * access operation (create or refresh).
         */
        public short cFreeUser;
       
        /**
         * ontains the maximum length of a data block which can be encrypted via the WkNet or WkLAN subsystem.
         * Under WkNet this length depends on the specified length of data transfer in the WkNet server file.
         */
        public short cbMaxData;
       
        /**
         * contains the time distance to the last proper WkNet file update in refresh time intervals. The driver
         * calculates this value when it checks whether the server has updated the WkNet file. If so, this value is
         * set to 0. Otherwise, it is increased with every new expected refresh interval by 1. If this value is greater
         * than 1, a call to a WkNet specific API method fails with the WKB_ERROR_NET_NOT_READY error.
         */
        public short cRefreshError = 0;
       
        /**
         * is reserved for future use and should be set to 0.
         */
        public short usReserved1 = 0;
  }
 
  /**
   *
   */
  public static final long WKB_USEPOINTERS      = 0x00000;
 
  /**
   *
   */
  public static final long WKB_UNICODE          = 0x10000;
 
  /**
   * use exclusive setting for this call
   */
  public static final long WKB_EXCLMODE         = 0x40000;
 
  /**
   * should never be used in JAVA!
   */
  public static final long WKB_USEOFFSETS       = 0x80000;
  
  /**
   * (0x00) returns the system status in a WKBSYSTEM structure. This is the status if the
   * <A HREF = "WkJavaApi.html#WkbAccess2(long, com.wibu.WkJavaApi.WKBACCESS)">WkbAccess2</A> method was
   * called or the current status if the WKB_QST_UPDATE option is set (see below).
   */
  public static final long WKB_QSY_SYSTEM       = 0x00;
 
  /**
   * (0x10) returns the full status of the system status in an unspecified byte buffer, which must be long enough to
   * hold the complete information. This buffer can be used later to restore the subsystem via the
   * <A HREF = "WkJavaApi.html#WkbAccess2(long, com.wibu.WkJavaApi.WKBACCESS)">WkbAccess2</A> method
   * and the WKB_ACC_RESTORE option.
   */
  public static final long WKB_QSY_STATUS       = 0x10;
 
  /**
   * (0x20) returns a string with the readable TCP/IP address of the addressed subsystem. The format of the address
   * string is operating system dependent, for example 192.168.35.62.
   */
  public static final long WKB_QSY_IPADDR       = 0x20;
 
  /**
   * (0x30) returns a WKBOPENENTRY structure, which contains the information of the WIBU-BOX entry, which is the base
   * of the specified subsystem.
   */  
  public static final long WKB_QSY_OPENENTRY    = 0x30;        
 
  /**
   * (0x0001) returns the current state of the WKBSYSTEM structure. This includes the number of available and free users.
   */
  public static final long WKB_QSY_UPDATE       = 0x0001;      
 
  /**
   * (0x0002) displays the required information additionally in a dialog box at the screen. The method returns not
   * before the user has closed this dialog box. The dialog displays the type, name and version of the subsystem and
   * contains also buttons to open the WibuKey control panel applet and the WibuKey using help. With WKB_DLG_xxx
   * options (see below), some of these extensions may be suppressed. The dialog is also displayed when the destination
   * buffer is too small to receive the returned data.
   */
  public static final long WKB_QSY_DIALOG       = 0x0002;
 
  /**
   * (0x10000000) the version of the subsystem is not displayed in the dialog.
   */
  public static final long WKB_DLG_NO_VERSION   = 0x10000000;
 
  /**
   * (0x20000000) the dialog button, which accesses the WibuKey control panel applet, is hidden.
   */
  public static final long WKB_DLG_NO_PANEL     = 0x20000000;
 
  /**
   * (0x40000000) the dialog button, which opens the WibuKey using help text, is hidden.
   */
  public static final long WKB_DLG_NO_HELP      = 0x40000000;
 

  // ******************************************************************************************************** //
  // NATIVE CALL WkbEnumServers2
  // ******************************************************************************************************** //
 
  /**
   * This method returns a list of all available WibuKey network server systems as names or addresses
   * @param flCtrl  contains flags which control the execution of the operation. Following command codes are currently supported: 
   * <ul>
   *   <li> WKB_ESV_LIST (0x0000) The buffer specified by the pszDest parameter receives a string containing the names 
   *        or addresses of all available WibuKey network servers, separated by the vertical bar and terminated by a 0 
   *        character. 
   *   <li> WKB_ESV_NEXT (0x0010) The buffer specified by the pszDest parameter receives a string which corresponds 
   *        with the next name or address of an available WibuKey network server which follows in the name ordering after the 
   *        name which is specified in the pszDest buffer if the method is called. If the latter name is an empty string, the 
   *        first available WibuKey network server name is returned. After all available WkLAN server names were returned 
   *        back, the method returns with 0 and a subsequent call of the <A HREF = "WkJavaApi.html#WkbGetLastError()">
   *        WkbGetLastError</A> method returns the WKB_ERROR_SERVER_NOT_FOUND error code. 
   *   <li> Following options may be specified additionally: 
   *   <ul>
   *     <li> WKB_ESV_NAME (0x0000, default) The user-readable network name of a server is determined. These names are 
   *          returned from the WibuKey servers and may differ from the proper network name of a used server computer. 
   *     <li> WKB_ESV_IPADDR (0x0001) The user-readable TCP/IP address of a server is determined. These names are 
   *          determined directly from the local operating system; normally they can be used always for a proper access to the 
   *          WibuKey server. The format of the address string is operating system dependent, for example 192.168.35.62. 
   *     <li> WKB_ESV_NO_DIALOG (0x0002) does not display a message box on Windows during the WkLAN servers are 
   *          searched. This option is only valid and may be specified only if the level is set to 1 or higher. 
   *     <li> WKB_ESV_NO_CANCEL (0x0004) does not display a Cancel button in a message box on Windows during the WkLAN 
   *          servers are searched. This option is only valid and may be specified only if the level is set to 1 or higher. The 
   *          option has no meaning if WKB_ESV_NO_DIALOG is set too. 
   *   </ul>
   *   <li> This parameter contains also standard function control flags. 
   * </ul>
   *
   * @param pszDest receives the string which corresponds with the list of all available server names or 
   * with the next server name. In the latter case, the buffer must contain the preceding server name string before the 
   * method is called or an empty string if the first name should be returned. The stored string is terminated by a 0-
   * character.
   *
   * @param cchDest not used in Java. Has to be set to any integer value, e.g. 0.
   * <!-- <font color="#FF00FF"><b> ??? Methode als deprecated, neue ohne cchDest?</b>not used.</font>
   * -->
   *
   * @return 0 if any error occurred.
   * In this case a subsequent call of the <A HREF = "WkJavaApi.html#WkbGetLastError()">WkbGetLastError</A> method 
   * returns the corresponding error code.
   */
  public static native int WkbEnumServers2(long flCtrl, StringBuffer pszDest, int cchDest);


  /**
   * (0x0000) The buffer specified by the pszDest parameter receives a string containing the names or addresses
   * of all available WibuKey network servers, separated by the vertical bar and terminated by a 0 character.
   */
  public static final long WKB_ESV_LIST         = 0x0000;
 
  /**
   * (0x0010) The buffer specified by the pszDest parameter receives a string which corresponds with the next
   * name or address of an available WibuKey network server which follows in the name ordering after the name
   * which is specified in the pszDest buffer if the method is called. If the latter name is an empty string,
   * the first available WibuKey network server name is returned. After all available WkLAN server names were
   * returned back, the method returns with 0 and a subsequent call of the WkbGetLastError method returns
   * the WKB_ERROR_SERVER_NOT_FOUND error code.
   */
  public static final long WKB_ESV_NEXT         = 0x0010;
 
  /**
   * (0x0000, default) The user-readable network name of a server is determined. These names are returned from
   * the WibuKey servers and may differ from the proper network name of a used server computer.
   */
  public static final long WKB_ESV_NAME         = 0x0000;
 
  /**
   * (0x0001) The user-readable TCP/IP address of a server is determined. These names are determined directly
   * from the local operating system; normally they can be used always for a proper access to the WibuKey server.
   * The format of the address string is operating system dependent, for example 192.168.35.62.
   */
  public static final long WKB_ESV_IPADDR       = 0x0001;
 
  /**
   * (0x0002) does not display a message box on Windows during the WkLAN servers are searched. This option is only
   * valid and may be specified only if the level is set to 1 or higher.
   */
  public static final long WKB_ESV_NO_DIALOG    = 0x0002;
 
  /**
   * (0x0004) does not display a Cancel button in a message box on Windows during the WkLAN servers are searched.
   * This option is only valid and may be specified only if the level is set to 1 or higher. The option has no
   * meaning if WKB_ESV_NO_DIALOG is set too.
   */
  public static final long WKB_ESV_NO_CANCEL    = 0x0004;

 
  // ******************************************************************************************************** //
  // NATIVE CALL WkbApplicationInfo2
  // ******************************************************************************************************** //
  
  /**
   * (0x0000) sets the contents of the WKBAPPINFO structure at the pvInfo parameter as new client application
   * control information. Only those parts of the structure are interpreted where the corresponding selection
   * flags in the flCtrl member of the structure are set. If the version is selected, the method executes a
   * version check; if this fails and the WKB_AI_DIALOG option is set, an error message box is displayed.
   */
  public static final long WKB_AI_SETDATA       = 0x0000;
 
  /**
   * (0x0010) returns the application setting information for the specified subsystem into the WKBAPPINFO buffer.
   * The method returns the true version of the calling driver and the really used language.
   */
  public static final long WKB_AI_GETDATA       = 0x0010;
 
  /**
   * (0x0020) interprets the specified list of Windows language identifiers and returns the first identifier
   * which language is supported by the current driver installation. If no language is accepted, LANG_ENGLISH
   * is returned.
   */
  public static final long WKB_AI_GETNEARLANG   = 0x0020;
 
  /**
   * (0x0030) returns an array of Windows language identifiers, which describes all supported languages of the
   * current driver installation. The first item in this array contains the language, which is used by default
   * (determined by the users language setting).
   */
  public static final long WKB_AI_GETLANGLIST   = 0x0030;
 
  /**
   * (0x0040) displays a message box with the specified text at screen. The dialog may be modified by the
   * WKB_DLG_xxx constants. Within the text, placeholders may be specified with surrounding %-characters.
   * Currently only two placeholders are supported: %ErrorText% is replaced by the text of the last occurred
   * error; %ErrorCode% is replaced by the number of the last occurred error.
   */
  public static final long WKB_AI_MESSAGE       = 0x0040;


  /**
   * This structure specifies information to control an application specific behavior of a driver client.
   * The structure may be specified via the WKBACCESS structure of a <A HREF = 
   * "WkJavaApi.html#WkbAccess2(long, com.wibu.WkJavaApi.WKBACCESS)">WkbAccess2</A> call or modified by the
   * WkbApplicationInfo2 method.
   */
  public static class WKBAPPINFO
  {
        /**
         * contains further control flags.
         */
        public long flCtrl;
       
        /**
         * points to the name of the application, which is displayed in error message boxes and other dialog
         * information. This member is only used if the WKB_AI_SEL_NAME flag is set in flCtrl.
         */
        public StringBuffer pszAppName = new StringBuffer("");
       
        /**
         * contains the minor value (for example 53 for 2.53) of the minimum version, which is required for the
         * driver access. This member is only used if the WKB_AI_SEL_VERSION flag is set. The version check is
         * done for the calling driver and for all subsystem variants, which are specified in fsAccessMask.
         */
        public byte bVersionLow = 0;
       
        /**
         * contains the major value (for example 2 for 2.53) of the minimum version, which is required for the
         * driver access. This member is only used if the WKB_AI_SEL_VERSION flag is set. The version check is
         * done for the calling driver and for all subsystem variants, which are specified in fsAccessMask.
         */
        public byte bVersionHigh = 0;
       
        /**
         * specifies as a combination of following subsystem variant flags.
         * The following flags may be set:
         *      - WKB_ACC_LOCAL (0x01) checks that the local (kernel based) subsystem has as minimum the version
         *                      in bVersionLow and bVersionLow.
         *      - WKB_ACC_WKLAN (0x02) checks that the WKLAN network subsystem has as minimum the version in
         *                      bVersionLow and bVersionLow.
         *      - WKB_ACC_WKNET (0x04) checks that the WkNet network subsystem has as minimum the version in
         *                      bVersionLow and bVersionLow.
         */
        public short fsAccessMask;
       
        /**
         * specifies the handle of the driver client application main window. This member is only used if the
         * WKB_AI_SEL_MENU flag is set in flCtrl.
         */
        public long hwndMainWindow;
       
        /**
         * specifies additional flags for the application window WibuKey menu. The future to use this member is
         * currently not implemented. This member is only used if the WKB_AI_SEL_MENU flag is set in flCtrl.
         */
        public long flMenuItemCtrl;
       
        /**
         * contains the Windows identification constant of the language, which is used for the subsystem dialog and
         * error text communication. This member is only used if the WKB_AI_SEL_LANG flag is set in flCtrl.
         */
        public short idLanguage;
       
        /**
         * is reserved for future use and must be set to 0.
         */
        public short usReserved1 = 0;
       
        /**
         * is reserved for future use and must be set to 0.
         */
        public long ulReserved2 = 0;
       
        /**
         * is reserved for future use and must be set to 0.
         */
        public long ulReserved3 = 0;
  }


  /**
   * (0x0001) selects the data in the pszAppName member as valid.
   */
  public static final long WKB_AIS_SEL_NAME     = 0x0001;
 
  /**
   * (0x0002) selects the data in the bVersionLow, bVersionHigh and fsAccessMask members of the structure as valid.
   */
  public static final long WKB_AIS_SEL_VERSION  = 0x0002;
 
  /**
   * (0x0004) selects the data in the hWndMainWnd and flMenuItemCtrl member of the structure as valid.
   */
  public static final long WKB_AIS_SEL_MENU     = 0x0004;
 
  /**
   * (0x0008) selects the data in the idLanguage member of the structure as valid.
   */
  public static final long WKB_AIS_SEL_LANG     = 0x0008;
 
  /**
   * (0x000F) selects the data in all members of the structure as valid.
   */
  public static final long WKB_AIS_SEL_ALL      = 0x000F;
 
  /**
   * (0x0008) is used in combination with the WKB_AI_SETDATA command: If the specified version does not correspond
   * with the desired subsystem parts, an error message with the required version is displayed. This option may be
   * specified optionally with the WKB_AI_MESSAGE command but does not influence the behavior of this method. For
   * all other commands, this option must not be specified.
   */
  public static final long WKB_AI_DIALOG        = 0x0008;
 


  // ******************************************************************************************************** //
  //                                    General Error and Execution Management
  // ******************************************************************************************************** //

  // ******************************************************************************************************** //
  // NATIVE CALL WkbGetLastError
  // ******************************************************************************************************** //
 
  /**
   * Gets the last set WibuKey error code.
   *
   * @return last set WibuKey error code.
   */
  public static native int WkbGetLastError();


  // ******************************************************************************************************** //
  // NATIVE CALL WkbSetLastError
  // ******************************************************************************************************** //

  /**
   * Sets the specified code as the driver internal error code.
   *
   * @param idError contains the error code which is set.
   */
  public static native void WkbSetLastError(int idError);
 
 
  /**
   * Receives the status of an asynchronous started method.
   *
   * @param nActionID contains the action code which was returned of a method which was called asynchronously.
   *
   * @param pidReturn return value of the method which was executed and terminated asynchronously.
   *
   * @return If the method, which corresponds with the specified action code, has completed, the WkbIsComplete
   *         mathod returns TRUE and the variable, which is pointed by the second parameter, receives the return
   *         value of the terminated method. This can be a status code, a handle or any other method dependent
   *         value. If this value corresponds with an error condition, the error code can be returned via the
   *         WkbGetLastError method.
   *         If the specified method has not terminated the WkbIsComplete method returns FALSE and a subsequent
   *         call of WkbGetLastError returns WKB_ERROR_IN_ACTION. If the specified action is undefined, the
   *         WKB_ERROR_UNDEFINED_ACTION error code is returned otherwise.
   */
  public static native boolean WkbIsComplete(int nActionID, int pidReturn);


  // *********************************************************************************************************
  //                                            Driver Status and Control
  // *********************************************************************************************************
 
  // ******************************************************************************************************** //
  // NATIVE CALL WkbGetVersion3
  // ******************************************************************************************************** //
 
  /**
   * Retrieves the version of the WibuKey-driver.
   *
   * @param hwkbsys contains the handle of a valid accessed WibuKey subsystem. To determine the version of the
   *                standard local subsystem the predefined handle value HWKB_LOCAL can be used. To determine the
   *                version of the driver part, which contains the code of this method, 0 must be specified.
   *                This returned value differs from the HWKB_LOCAL specification only if the kernel of the driver
   *                is dynamically linked (on Windows 95/98 res. NT/2000/XP).
   * @return 
   * <ul>
   *   <li> main number of the version in the higher byte (bit 8 to 15) and the minor number in the lower byte (bit 0 
   *        to 7). Under the WK32 API, the bits 16 to 31 of the value are set all to 0. 
   *   <li> 0 if any error occurred.
   *        In this case a subsequent call of the <A HREF = "WkJavaApi.html#WkbGetLastError()">WkbGetLastError</A> method 
   *        returns the corresponding error code.
   * </ul>
   */
  public static native int WkbGetVersion3(long hwkbsys);

  
  // ******************************************************************************************************** //
  // NATIVE CALL WkbGetVersion2
  // ******************************************************************************************************** //
 
  /**
   * This method retrieves the version of the WibuKey-driver.
   * @param hwkbsys contains the handle of a valid accessed WibuKey subsystem. To determine the version of the
   *                standard local subsystem the predefined handle value HWKB_LOCAL can be used. To determine the
   *                version of the driver part, which contains the code of this method, 0 must be specified.
   *                This returned value differs from the HWBK_LOCAL specification only if the kernel of the driver
   *                is dynamically linked (on Windows 95/98 res. NT/2000).
   * @return 
   * <ul>
   *   <li> number of the version in the lower byte (bit 0 to 7) and the minor number in the next byte (bit 8 to 15). Under the WK32 API, the bits 16 to 31 of the value are set all to 0. 
   *   <li> 0 if any error occurred.
   *        In this case a subsequent call of the <A HREF = "WkJavaApi.html#WkbGetLastError()">WkbGetLastError</A> method 
   *        returns the corresponding error code.
   * </ul>
   *
   * @deprecated This method is obsolete and replaced by <A HREF = "WkJavaApi.html#WkbGetVersion3(long)">WkbGetVersion3</A>
   */
  public static native int WkbGetVersion2(long hwkbsys);
  
  // ******************************************************************************************************** //
  // NATIVE CALL WkbControlSystem2
  // ******************************************************************************************************** //
 
  /**
   * Controls the specified WibuKey subsystem.
   * <br> <I>This method is not tested for compatibility with Java and may not work. <B>Usage is not recommended.</B></I>
   * <!--<font color="#FF00FF"><b> ??? Diese Methode scheint noch nie benutzt und ob davon irgendwas richtig vfunktioniert. Was tun?</b></font>
   * -->
   * @param hwkbsys contains the handle of a valid accessed WibuKey subsystem.
   *
   * @param flCtrl contains flags which controls the execution of the method. Currently following command codes are defined:
   * <ul>
   *   <li> WKB_CTS_RESET (0x00) resets the specified system. Only valid for the local subsystem. The reset operation 
   *        modes are defined in the cbCtrl parameter. The pvCtrl parameter must be set to 0. 
   *   <li> WKB_CTS_CANCEL_USERS (0x10) cancels the specified users in the WkNet or WkLAN subsystem. Canceled users 
   *        cannot be longer use the network server via their own subsystem without a new call of the WkbAccess2 method. 
   *   <li> WKB_CTS_FREE_ACCESS (0x20) frees the specified subsystem without releasing the user in the network. 
   *        This 
   *        operation is normally done in combination with saving the internal subsystem state via calling the WkbQuerySystem2 
   *        method with the WKB_QSY_STATUS command and the WkbAccess2 method with the WKB_ACC_RESTORE command. </font>
   *   <li> WKB_CTS_KILL_ACTION (0x30) kills the actions of the specified subsystem which identification is stored in 
   *        the cbCtrl parameter. If this value is 0 all actions of the specified subsystem are killed. This kill all action 
   *        variant must not be called asynchronously.
   *        If the WKB_CTS_KILL_ACTION operation is selected, this parameter contains the action code of the action, 
   *        which should be killed. If 0 is specified instead, all actions of the specified subsystem are killed. This kill all 
   *        action variant must not be called asynchronously.   
   *   <li> This parameter contains also standard function control flags. 
   * </ul>
   * @param pvCtrl controls the execution of the method. For the 
   *               WKB_CTS_CANCEL_USERS this is an array containing USHORT (16 bits) values. Two values in this
   *               array specify a pair of minimum and maximum user identifications in which range (including the
   *               boundaries) all users are freed in the WkNet file or in the WkLAN server structure. If a single
   *               user entry is specified at the end of the array, only this one user is canceled. The length of the
   *               array in bytes is specified by the cbCtrl parameter. For all other WKB_CTS_... operations, this
   *               value must be set to 0.
   *
   * @param cbCtrl contains the length of the data structure which is specified via the pvCtrl if this
   *               pointer is not 0. If it is 0, the meaning of this parameter depends on the specified
   *               WKB_CTS_... option. For the WKB_CTS_RESET operation, this parameter represents a bitmap of flags with
   *               following meaning: TO BE CONTINUED... </font>
   *
   * @return 0 if any error occurred.
   * In this case a subsequent call of the <A HREF = "WkJavaApi.html#WkbGetLastError()">WkbGetLastError</A> method 
   * returns the corresponding error code.
   */
  public static native int WkbControlSystem2(long hwkbsys, long flCtrl, Object pvCtrl, long cbCtrl);


  /**
   * (0x00) reset the specified system. This method is identical with the WkbReset method of API 1.X and
   * only valid for the local subsystem. The reset operation modes are defined in the cbCtrl parameter. The pvCtrl
   * parameter must be set to NULL.
   */
  public static final long WKB_CTS_RESET        = 0x0000;
 
  /**
   * (0x10) cancels the specified users in the WkNet or WkLAN subsystem. Canceled users cannot be longer use the network
   * server via their own subsystem without a new call of the <A HREF = 
   * "WkJavaApi.html#WkbAccess2(long, com.wibu.WkJavaApi.WKBACCESS)">WkbAccess2</A> method..
   */
  public static final long WKB_CTS_CANCEL_USERS = 0x1000;
 
  /**
   * (0x20) frees the specified subsystem without releasing the user in the network. This operation is normally
   * done in combination with saving the internal subsystem state via calling the
   * <A HREF = "WkJavaApi.html#WkbQuerySystem2(long, long, com.wibu.WkJavaApi.WKBSYSTEM, int)">WkbQuerySystem2</A> method with
   * the WKB_QSY_STATUS command and the <A HREF = 
   * "WkJavaApi.html#WkbAccess2(long, com.wibu.WkJavaApi.WKBACCESS)">WkbAccess2</A> method with the WKB_ACC_RESTORE command.
   */
  public static final long WKB_CTS_FREE_ACCESS  = 0x2000;
 
  /**
   * (0x30) kills the actions of the specified subsystem which identification is stored in the cbCtrl parameter.
   * If this value is 0 all actions of the specified subsystem are killed. This kill all action variant must not
   * be called asynchronously.
   */
  public static final long WKB_CTS_KILL_ACTION  = 0x3000;

  /**
   * (0x01) is reserved for future use and must be set to 0.
   */
  public static final long WKB_RESET_SYSTEM     = 0x0100;
 
  /**
   * (0x02) rereads the addressing specifications in the profile (WIBUKEY.INI or environment).
   */
  public static final long WKB_RESET_ADDRESS    = 0x0200;
 
  /**
   * (0x04) recalculates the complete WIBU-BOX timing.
   */
  public static final long WKB_RESET_TIMING     = 0x0400;
 
  /**
   * (0x08) restarts the elementary scanning of all WIBU-BOXes if the next access method is started. This option
   * is only used for the Macintosh-ADB-variant of the driver.
   */
  public static final long WKB_RESET_BOXSCAN    = 0x0800;
 
  /**
   * (0x10) resets the locking flag of all opened WIBU-BOXes. The locking flag is set by the
   * <A HREF = "WkJavaApi.html#WkbSelect2(long, long, long, java.lang.Object)">WkbSelect2</A> method
   * and closed normally via the <A HREF = "WkJavaApi.html#WkbUnSelect2(long)">WkbUnSelect2</A> or
   * <A HREF = "WkJavaApi.html#WkbClose2(long)">WkbClose2</A> method. The locking can be remained unlimited if an
   * application crashes or forgets to call the WkbSelect2 or
   * <A HREF = "WkJavaApi.html#WkbClose2(long)">WkbClose2</A> operation and avoids then any further
   * selection of the WIBU-BOX. After the locking flag of an WIBU-BOX is reset, the latter cannot be longer used
   * in a <A HREF = "WkJavaApi.html#WkbCrypt2(long, long, byte[], com.wibu.WkJavaApi.WKBAREA, WkJavaApi.pInt)">WkbCrypt2</A> call without
   * a further WkbSelect2 call.
   */
  public static final long WKB_RESET_BOXLOCKS   = 0x1000;
 
  /**
   * (0xFE) is a mask, which forces all current and future reset options.
   */
  public static final long WKB_RESET_FULL       = 0xFE00;


  // ******************************************************************************************************** //
  // NATIVE CALL WkbQueryInfo2
  // ******************************************************************************************************** //

  /**
   * This method returns version and error information for a specific subsystem or for the calling driver.
   * @param hwkbsys contains the handle of a valid accessed WibuKey subsystem. To address the standard local
   *                subsystem the predefined handle value HWKB_LOCAL may be used. If 0 is specified in this parameter,
   *                WKB_QI_DRIVER will return the version information of the calling driver. This parameter is
   *                ignored when the WKB_QI_ERRTEXT command is specified.
   *
   * @param flCtrl contains flags which control the operation of the method. Following command codes are currently supported: 
   * <ul>
   *   <li> WKB_QI_ERRTEXT (0x0000) returns an error text, which corresponds with the last set error code (which is 
   *        returned by <A HREF = "WkJavaApi.html#WkbGetLastError()">WkbGetLastError</A>). The text is automatically set in the 
   *        selected language of the process and if the calling driver supports this language; otherwise US-English is used by 
   *        default. To return the text of a specific error code, you may set this error code via the 
   *        <A HREF = "WkJavaApi.html#WkbSetLastError(int)">WkbSetLastError</A> method. This flag can currently only be used on Windows! 
   *   <li> WKB_QI_VERVALUE (0x0010) returns version information in one <A HREF = 
   *        "WkJavaApi.WKBVERSION.html">WKBVERSION</A> class, which specifies binary version and release data information about 
   *        the specified subsystem or of the calling driver. The type of version is specified by further options (see below).  
   *   <li> WKB_QI_VERTEXT (0x0020) returns a text, which specifies the version and the release date of the specified 
   *        subsystem or of the calling driver. The type of version is specified by further options (see below). If several 
   *        operations are specified, multiple strings are stored subsequently, separated by a vertical bar character ('|'). 
   *   <li> Several flags control the operation of the WKB_QI_VERVALUE and WKB_QI_VERTEXT command codes. 
   *        Only one of these can be used 
   *   <ul>
   *     <li> WKB_QI_DRIVER (0x0001) returns the version information of the driver part of a local or remote 
   *          subsystem. If hwkbsys contains 0, the version information of the calling driver is returned. 
   *     <li> WKB_QI_SERVER (0x0002) returns the version information of the server application of a remote subsystem. 
   *          This flag is ignored if a local subsystem is available or the server application version information is not 
   *          available. 
   *     <li> WKB_QI_JNI (0x0004) returns the version information of the JNI library (wibuKJni32.dll/wibuKJni.so/wibuKJni.dylib).
   *          This flag is only available in the JAVA API.
   *   </ul>
   *   <li> This parameter contains also standard function control flags. 
   * </ul>
   *
   * @param pvDest for flag WKB_QI_VERVALUE it contains a WKBVERSION class, for other flags a String.
   *
   * @param cbDest not used in Java. Has to be set to any integer value, e.g. 0.
   *
   * @return 0 if any error occurred.
   * In this case a subsequent call of the <A HREF = "WkJavaApi.html#WkbGetLastError()">WkbGetLastError</A> method 
   * returns the corresponding error code.
   */
  public static native int WkbQueryInfo2(long hwkbsys, long flCtrl, Object pvDest, int cbDest);
 
 
  /**
   * (0x0000) returns an error text, which corresponds with the last set error code
   * (which is returned by WkbGetLastError). The text is automatically set in the selected language of
   * the process and if the calling driver supports this language; otherwise US-English is used by default.
   * To return the text of a specific error code, you may set this error code via the WkbSetLastError method.
   */
  public static final long WKB_QI_ERRTEXT       = 0x0000;
 
  /**
   * (0x0010) returns one or more WKBVERSION structures, which specifies binary version and release data information
   * about the specified subsystem or of the calling driver. The type of version is specified by further options
   * (see below). If several operations are specified, multiple WKBVERSION structures are stored subsequently.
   */
  public static final long WKB_QI_VERVALUE      = 0x0010;
  
  /**
   * (0x0001) returns the driver version information.
   */
  public static final long WKB_QI_DRIVER      	= 0x0001;
  
  /**
   * (0x0002) returns the server version information.
   */
  public static final long WKB_QI_SERVER      	= 0x0002;
 
  /**
   * (0x0004) returns the wibuKJni version information.
   */
  public static final long WKB_QI_JNI      	  = 0x0004;
 
  /**
    * (0x0008) display information in dialog box - only supported on Windows.
    */
  public static final long WKB_QI_DIALOG       = 0x0008;

  /**
   * (0x0020) returns a text, which specifies the version and the release date of the specified subsystem or of the
   * calling driver. The type of version is specified by further options (see below). If several operations are
   * specified, multiple strings are stored subsequently, separated by a vertical bar character ("|").
   */
  public static final long WKB_QI_VERTEXT       = 0x0020;
 
 
  /**
   * This structure describes the version of a (remote) WibuKey subsystem or of a kernel driver part. It is
   * returned by the command of the WkbQueryInfo2 method.
   */
  public static class WKBVERSION
  {
        /**
         * contains the major version value of the WibuKey driver component or subsystem, for example
         * 2 of a version 2.52.
         */
        public short usVersion;
       
        /**
         * contains the minor version value of the WibuKey component or subsystem, for example
         * 52 of a version 2.52.
         */
        public short usSubVersion;
       
        /**
         * contains the continuous build value of the WibuKey component or subsystem. This value is
         * currently not used and set to 0.
         */
        public short usBuild;
       
        /**
         * contains the count value, which extends the major and minor version value. The range of this value
         * specifies also the completion level of a product:
         * - 0 to 99: concept: all features and interfaces may be changed.
         * - 100 to 199: alpha: most features and interfaces may be changed.
         * - 200 to 299: beta: features and interfaces are very complete, minor parts may be changed.
         * - 300 to 399: gamma: features and interfaces are final; the implementation is currently tested.
         * - 400 to 499: reserved for future use
         * - 500 to 599: final: 500 is first release, 501 "a", 502 "b" etc.
         * - > 599: reserved for future use.
         */
        public short usCount;
       
        /**
         * contains the year value of the release date. This is a value between 1980 and 2099.
         */
        public short usYear;
       
        /**
         * contains the month value of the release date. This is 1 for January, 2 for February, etc.
         */
        public byte bMonth;
       
        /**
         * contains the day value of the release data. This is a value between 1 and 31.
         */
        public byte bDay;
       
        /**
         * and reserved for future use and must be set to 0.
         */
        public long ulReserved1;
  }


  // ******************************************************************************************************** //
  // NATIVE CALL WkbQueryStatus2
  // ******************************************************************************************************** //

  /**
   * This method returns the status and specifications of the accessed WibuKey driver.
   * @param hwkbsys contains the handle of a valid accessed WibuKey subsystem. To address the standard local
   *                subsystem the predefined handle value HWKB_LOCAL can be used.
   * @param flCtrl contains flags which control the operation of the method. Following command codes are currently supported: 
   * <ul>
   *   <li> WKB_QST_DRIVER (0x0000) returns driver status specific information.
   *   <li> This parameter contains also standard function control flags. 
   * </ul>
   *
   * @param pwkbst receives the status data in a <A HREF = "WkJavaApi.WKBSTATUS.html">WKBSTATUS</A> class.
   *
   * @param cbDest not used in Java. Has to be set to any integer value, e.g. 0.
   * <!--<font color="#FF00FF"><b> ??? Methode als deprecated, neue ohne cbDest?</b>not used.</font>
   * -->
   *
   * @return 0 if any error occurred.
   * In this case a subsequent call of the <A HREF = "WkJavaApi.html#WkbGetLastError()">WkbGetLastError</A> method 
   * returns the corresponding error code.
   */
  public static native int WkbQueryStatus2(long hwkbsys, long flCtrl, WKBSTATUS pwkbst, int cbDest);
 
  /**
   * (0x0000) returns driver status specific information.
   */
  public static final long WKB_QST_DRIVER       = 0x0000;
 
 
  /**
   * The structure contains status values of the used drivers and is returned by the WkbQueryStatus2 method.
   */
  public static class WKBSTATUS
  {
         /**
          * contains flags which specify the status of the driver.
          */
         public short usFlags;
        
         /**
          * is only valid if the WKB_ST_INTDRV flag in the usFlags member is specified and contains the used
          * interrupt number in the range between 0 up to 255.
          */
         public short usDrvInt;
  }

  /**
   * The driver is called internally via interrupt. The used interrupt number is specified in the usDrvInt member.
   */
  public static final short WKB_ST_INTDRV       = 0x0001;
 
  /**
   * On Windows NT/2000 or Windows 95/98, a true kernel driver is used.
   */  
  public static final short WKB_ST_KRNL         = 0x0002;
 
  /**
   * The driver is called internally by the WkDos calling mechanism (via interrupt 0x2F).
   */  
  public static final short WKB_ST_WKDOS        = 0x0004;
 
  /**
   * The driver supports the synchronous mode (default).
   */  
  public static final short WKB_SYNCHRONOUS     = 0x0000;
 
  /**
   * The driver supports the asynchronous mode.
   */  
  public static final short WKB_ST_ASYNCHRONOUS = 0x0010;
 
  /**
   * The driver supports the diagnosis mode.
   */
  public static final short WKB_ST_DIAGNOSIS    = 0x0020;
 
  /**
   * The driver supports the standard indirect encryption (Knuth 20).
   */
  public static final short WKB_ST_STDINDCRYPT  = 0x0040;
 
  /**
   * The driver supports all indirect encryptions including FEAL and permutation.
   */  
  public static final short WKB_ST_EXTINDCRYPT  = 0x0080;
 
  /**
   * The driver supports the encryption via the WKBAREA structure.
   */  
  public static final short WKB_ST_AREACRYPT    = 0x0100;
 
  /**
   * The driver supports Expiration Dates in WIBU-BOX listing and entry selection methods.
   */  
  public static final short WKB_ST_EXPDATE      = 0x0200;
 
  /**
   * The driver supports the access of the local WibuKey subsystem and the driver setting does not
   * deactivate this subsystem.
   */  
  public static final short WKB_ST_LOCAL        = 0x1000;
 
  /**
   * The driver supports the access of a server via the WkLAN subsystem and the driver setting does not
   * deactivate this subsystem.
   */
  public static final short WKB_ST_WKLAN        = 0x2000;      
 
  /**
   * The driver supports the access of a server via the WkNet subsystem and the driver setting does not
   * deactivate this subsystem.
   */
  public static final short WKB_ST_WKNET        = 0x4000;      

 
  // *********************************************************************************************************
  //                                            Port Status and Control
  // *********************************************************************************************************
 
  // ******************************************************************************************************** //
  // NATIVE CALL WkbQueryPort2
  // ******************************************************************************************************** //
 
  /**
   * This method returns characteristic values of the specified WibuKey port.
   * <br> <I>This method is not tested for compatibility with Java and may not work. <B>Usage is not recommended.</B></I>
   *
   * @param hwkbsys contains the handle of a valid accessed WibuKey subsystem. To address the standard local
   *                subsystem the predefined handle value HWKB_LOCAL can be used.
   *
   * @param flCtrl contains flags which control the operation of the method. Following command codes are currently supported: 
   * <ul>
   *   <li> WKB_QP_PHYS (0x0000) returns port specific physical control and status data. 
   *   <li> WKB_QP_NAME (0x0010) returns the number and the port name of the specified port. 
   *   <li> WKB_QP_CONFIG_NAMES (0x0020) returns the string of the configuration bits of this port. 
   *   <li> This parameter contains also standard function control flags. 
   * </ul>
   *
   * @param pszPort contains a single port specification.
   *
   * @param pvDest receives the port data. For the flag WKB_QP_PHYS a <A HREF = "WkJavaApi.WKBPORT.html">WKBPORT</A> 
   * class is used. For the flag WKB_QP_NAME a <A HREF = "WkJavaApi.WKBPORTNAME.html">WKBPORTNAME</A> class is used. For 
   * the flag WKB_QP_CONFIG_NAMES a 0-terminated string is used.
   *
   * @param cbDest not used in Java. Has to be set to any integer value, e.g. 0.
   *
   * @return 0 if any error occurred.
   * In this case a subsequent call of the <A HREF = "WkJavaApi.html#WkbGetLastError()">WkbGetLastError</A> method 
   * returns the corresponding error code.
   */
  public static native int WkbQueryPort2(long hwkbsys, long flCtrl, String pszPort, Object pvDest, int cbDest);
 
 
  /**
   * (0x0000) returns port specific physical control and status data.
   */
  public static final long WKB_QP_PHYS          = 0x0000;
 
  /**
   * (0x0010) returns the number and the port name of the specified port.
   */
  public static final long WKB_QP_NAME          = 0x0010;
 
  /**
   * (0x0020) returns the string of the configuration bits of this port.
   */
  public static final long WKB_QP_CONFIG_NAMES  = 0x0020;
 
 
  /**
   * This structure contains the physical characteristics of the specified WibuKey port. It is returned by the
   * WkbQueryPort2 method.
   */
  public static class WKBPORT
  {
        /**
         * contains the physical port address which is used to address the WIBU-BOX via this port.
         */
        public long ulAddress;
  } 
 
 
  /**
   * This structure contains the logical characteristics of the specified WibuKey port. It is returned by the
   * WkbQueryPort2 method.
   */
  public static class WKBPORTNAME
  {
        /**
         * contains flags which control the interpretation of the structure.
         */
        public long flCtrl;
       
        /**
         * contains the logical port number which is used to address the port by a number with the WKB_PORTNUMBER option.
         */
        public long lPort;
       
        /**
         * points to the logical port name as a 0-terminated string.
         */
        public String pszPort;
  }
 
 
  // ******************************************************************************************************** //
  // NATIVE CALL WkbSetPort2
  // ******************************************************************************************************** //
 
  /**
   * This method sets the port specific control data.
   * <br> <I>This method is not tested for compatibility with Java and may not work. <B>Usage is not recommended.</B></I>
   * @param hwkbsys contains the handle of a valid accessed WibuKey subsystem. To address the standard
   *                local subsystem the predefined handle value HWKB_LOCAL can be used.
   * @param flCtrl contains flags which control the operation of the method.
   * @param pszPort points to a string which contains one single port specification. If the flag WKB_PORTNUMBER is
   *                set, this parameter is not interpreted as pointer but must contain a valid port specification
   *                number, which is compatible with WibuKey version 1.
   * @param pbData points to a structure which contains the port control data.
   * @param cbData contains the length of the port control data.
   * @return The method returns TRUE if the operation was successful and FALSE if an error occurred. In this case a
   *         subsequent call of the WkbGetLastError method returns the corresponding error code.
   * @deprecated This method is obsolete and not supported.
   */
  public static native int WkbSetPort2(long hwkbsys, long flCtrl, String pszPort, byte[] pbData, int cbData);
 
 
  // *********************************************************************************************************
  //                                            Programming WIBU-BOXes
  // *********************************************************************************************************
 
  // ******************************************************************************************************** //
  // NATIVE CALL WkbProgram2
  // ******************************************************************************************************** //
 
  /**
   * This method modifies the contents of a WIBU-BOX.
   * 
   * @param hwkbsys contains the handle of a valid accessed WibuKey subsystem. To program a WIBU-BOX at the standard 
   * local subsystem the predefined handle value HWKB_LOCAL can be used. 
   *
   * @param flCtrl contains flags which control the listing of the port. Following command codes and option flags are 
   * currently supported: 
   * <ul>
   *   <li> WKB_PGM_CLEAR (0x0000) clears an existing entry completely. 
   *   <li> WKB_PGM_NEW (0x0010) creates a new Base Entry. 
   *   <li> WKB_PGM_ADD (0x0020) adds an Added Entry to an existing Base Entry. 
   *   <li> WKB_PGM_MODIFY (0x0030) modifies an existing Base Entry. This is possibly only for User Data Entries; to 
   *        modify any other entry type, the entry must be cleared and created again. 
   *   <li> WKB_PGM_MODIFY_ADDED (0x0040) modifies the data contents of an Added Entry. 
   *   <li> WKB_PGM_LOCK (0x0090) lock WIBU-BOX for programming. 
   *   <li> WKB_PGM_UNLOCK (0x00A0) unlock locked WIBU-BOX. 
   *   <li> WKB_PGM_EXTMEM (0x00B0) program ExtMem storage pages
   *   <li> This parameter contains also standard function control flags. 
   * </ul>
   *
   * @param pszPort contains one single port specification.
   *
   * @param pwkbsr contains the Serial Number in a <A HREF = "WkJavaApi.WKBSERIAL.html">WKBSERIAL</A> class and 
   * specifies the WIBU-BOX which entry is programmed.
   *
   * @param pvPgmData contains programming data in a <A HREF = "WkJavaApi.WKBPROGRAM.html">WKBPROGRAM</A> class. 
   *
   * @return 0 if any error occurred.
   * In this case a subsequent call of the <A HREF = "WkJavaApi.html#WkbGetLastError()">WkbGetLastError</A> method 
   * returns the corresponding error code.
   */
  public static native boolean WkbProgram2(long hwkbsys, long flCtrl, String pszPort, WKBSERIAL pwkbsr,
                                                                                        WKBPROGRAM pvPgmData);
 
  /**
   * This method modifies the contents of a WIBU-BOX.
   * 
   * @param hwkbsys contains the handle of a valid accessed WibuKey subsystem. To program a WIBU-BOX at the standard 
   * local subsystem the predefined handle value HWKB_LOCAL can be used. 
   *
   * @param flCtrl contains flags which control the listing of the port. Following command codes and option flags are 
   * currently supported: 
   * <ul>
   *   <li> WKB_PGM_CONFIG (0x0080) modifies the WIBU-BOX Circuit Configuration data. 
   *   <li> WKB_PGM_LOCK (0x0090) lock WIBU-BOX for programming. 
   *   <li> WKB_PGM_UNLOCK (0x00A0) unlock locked WIBU-BOX. 
   *   <li> This parameter contains also standard function control flags. 
   * </ul>
   *
   * @param pszPort contains one single port specification.
   *
   * @param pwkbsr contains the Serial Number in a <A HREF = "WkJavaApi.WKBSERIAL.html">WKBSERIAL</A> class and 
   * specifies the WIBU-BOX which entry is programmed.
   *
   * @param pvPgmData contains the configuration data for programming in a 
   * <A HREF = "WkJavaApi.WKBBOXCONFIG.html">WKBBOXCONFIG</A> class.
   *
   * @return 0 if any error occurred.
   * In this case a subsequent call of the <A HREF = "WkJavaApi.html#WkbGetLastError()">WkbGetLastError</A> method 
   * returns the corresponding error code.
   */
  public static native boolean WkbProgram2(long hwkbsys, long flCtrl, String pszPort, WKBSERIAL pwkbsr,
                                                                                        WKBBOXCONFIG pvPgmData);
 
  /**
   * This method modifies the contents of a WIBU-BOX.
   * 
   * @param hwkbsys contains the handle of a valid accessed WibuKey subsystem. To program a WIBU-BOX at the standard 
   * local subsystem the predefined handle value HWKB_LOCAL can be used. 
   *
   * @param flCtrl contains flags which control the listing of the port. Following command codes and option flags are 
   * currently supported: 
   * <ul>
   *   <li> WKB_PGM_EXTMEM (0x00B0) modifies the WIBU-BOX Extended Memory. 
   *   <li> This parameter contains also standard function control flags. 
   * </ul>
   *
   * @param pszPort contains one single port specification.
   *
   * @param pwkbsr contains the Serial Number in a <A HREF = "WkJavaApi.WKBSERIAL.html">WKBSERIAL</A> class and 
   * specifies the WIBU-BOX which entry is programmed.
   *
   * @param pvExtMem contains the extended memory data for programming  
   * <A HREF = "WkJavaApi.WKBEXTMEM.html">WKBEXTMEM</A> class.
   *
   * @return 0 if any error occurred.
   * In this case a subsequent call of the <A HREF = "WkJavaApi.html#WkbGetLastError()">WkbGetLastError</A> method 
   * returns the corresponding error code.
   */
  public static native boolean WkbProgram2(long hwkbsys, long flCtrl, String pszPort, WKBSERIAL pwkbsr,
    WKBEXTMEM pvExtMem);
 
 
  /**
   * (0x0000) clear an existing entry completely.
   */
  public static final long WKB_PGM_CLEAR        = 0x0000;
 
  /**
   * (0x0010) creates a new Base Entry.
   */
  public static final long WKB_PGM_NEW          = 0x0010;
 
  /**
   * (0x0020) add an Added Entry to an existing Base Entry.
   */
  public static final long WKB_PGM_ADD          = 0x0020;
 
  /**
   * (0x0030) modifies an existing Base Entry. This is possibly only for User Data Entries; to modify any
   * other entry type, the entry must be cleared and created again.
   */
  public static final long WKB_PGM_MODIFY       = 0x0030;
 
  /**
   * (0x0040) modifies the data contents of an Added Entry.
   */
  public static final long WKB_PGM_MODIFY_ADDED = 0x0040;
 
  /**
   * (0x0080) modifies the WIBU-BOX Circuit Configuration data.
   */
  public static final long WKB_PGM_CONFIG       = 0x0080;
 
  /**
   * (0x0090) lock WIBU-BOX for programming.
   */
  public static final long WKB_PGM_LOCK         = 0x0090;
 
  /**
   * (0x00A0) unlock locked WIBU-BOX.
   */
  public static final long WKB_PGM_UNLOCK       = 0x00A0;

  /**
   * (0x00B0) program ExtMem storage pages
   */
  public static final long WKB_PGM_EXTMEM       = 0x00B0;

  /**
   * constant for <flCtrl> member of WKBPROGRAM structure
   * lowest byte contains box version number
   */
  public static final long WKB_SET_BOXVERSION   = 0x0100;
  
  /**
   * This structure specifies a WIBU-BOX entry, which is inserted, modified or cleared by the WkbProgram2 method.
   */
  public static class WKBPROGRAM
  {
        /**
         * This member specifies the version of the WIBU-BOX and further algorithm options which is used to change
         * the specified entry. The following flag must be used to add the version value of the programmed WIBU-BOX:
         */
        public long flCtrl;
       
        /**
         * specifies the entry which is changed as a WKBBOXENTRY structure. If the entry is cleared, only the entry
         * index must be specified in the flCtrl member of this structure. Otherwise the new contents must be stored
         * completely. If an entry is added to an existing base entry, the index of the base entry is used and the
         * Firm and User Code of the base entry must be specified.
         */
        public WKBBOXENTRY wkbbxe;
       
        /**
         * contains the signature which is needed to store or modify Firm and User Code based entries. If a user data
         * entry (WKB_BE_DATA) is inserted, modified or deleted, this member is ignored. If the programming algorithm of
         * WibuKey version 1 is used, this member is 47 bytes long, if the programming algorithm of WibuKey version 2
         * is used only 8 bytes.
         */
        public byte[] abSignature = new byte[8];       
  }
 
 
  /**
   * (0x0000) The pvCtrl parameter contains a pointer to an initialized structure, which contains additional data to 
   * control the encryption. If pvCtrl is NULL, no such control data are specified and the selection must be done by
   * the obsolete WkbSelect2 method. 
   */
   public static final int WKB_CRYPT_CTRL            = 0x0000;

  // ********************************************************************************************************* //
  //                                         Base WIBU-BOX Open and Encryption
  // ********************************************************************************************************* //

  // ********************************************************************************************************* //
  // NATIVE CALL WkbCrypt2
  // ********************************************************************************************************* //

  /**
   * This method encrypts or decrypts data in the memory via the specified WIBU-BOX entry.
   * The algorithm may be specified directly in a <A HREF = "WkJavaApi.WKBAREA.html">WKBAREA</A> class or it is 
   * executed in dependence of a previous selection of the opened entry via the obsolete 
   * <A HREF = "WkJavaApi.html#WkbSelect2(long, long, long, java.lang.Object)">WkbSelect2</A> method.
   *
   * @param hwkbe contains the handle of an opened WIBU-BOX entry.
   *
   * @param flCtrl contains flags which controls the execution of the operation. Following base options are currently supported: 
   * <ul>
   *   <li> WKB_CRYPT_CTRL (0x0000) The pvCtrl parameter contains a class, which contains additional data to control 
   *        the encryption. If pvCtrl is 0, no such control data are specified and the selection must be done by the 
   *        obsolete <A HREF = "WkJavaApi.html#WkbSelect2(long, long, long, java.lang.Object)">WkbSelect2</A> method. 
   *   <li> This parameter contains also standard function control flags. 
   * </ul>
   * @param pvDest contains the data to be encrypted and receives the encrypted data.
   *
   * @param pvCtrl contains a <A HREF = "WkJavaApi.WKBAREA.html">WKBAREA</A> class, which contains additional control 
   * data for the execution of the method.
   *
   * @param pcbDest contains a <A HREF = "WkJavaApi.pInt.html">pInt</A> (helper) class; its member receives the number 
   * of en/decrypted bytes.
   *
   * @return 0 if any error occurred.
   * In this case a subsequent call of the <A HREF = "WkJavaApi.html#WkbGetLastError()">WkbGetLastError</A> method 
   * returns the corresponding error code.
   */
  public static native int WkbCrypt2(long hwkbe, long flCtrl, byte[] pvDest, WKBAREA pvCtrl, pInt pcbDest);

 
  /**
   * This structure specifies an extended encryption/decryption area and is used as an optional parameter of
   * the <A HREF = "WkJavaApi.html#WkbCrypt2(long, long, byte[], com.wibu.WkJavaApi.WKBAREA, WkJavaApi.pInt)">WkbCrypt2</A> method.
   */
  public static class WKBAREA
  {
    /**
     * controls the encryption/decryption of the area and overwrites all WKB_AREA_XXX flags which are specified
     * in the flCtrl parameter of the W
     */
    public long flCtrl = WKB_STDCTRL | 0x0001 ;
   
    /**
     * is only used if the WKB_AREA_SELECT flag is set and contains the selection flags which are normally set in the
     * flCtrl parameter of the <A HREF = "WkJavaApi.html#WkbSelect2(long, long, long, java.lang.Object)">WkbSelect2</A> method.
     * If the WKB_AREA_SELECT flag is not set, this value must be set to 0.
     */
    public long flSelCtrl = 0;
   
    /**
     * is only used if the WKB_AREA_SELECT flag is set and contains the Selection Code which is normally set in the
     * ulSelectCode parameter of the <A HREF = "WkJavaApi.html#WkbSelect2(long, long, long, java.lang.Object)">WkbSelect2</A>
     * method. If the WKB_AREA_SELECT flag is not set, this value must be set to 0.
     */
    public long ulSelectCode = 0;
   
    /**
     * contains the 16-bit CRC value which is used to verify an direct encryption or decryption.
     */
    public long ulCrc;
   
    /**
     * is reserved for future use and must be set to 0
     */
    public long ulReserved;
   
    /**
     * points to a bitmap in which all bits are set (1) which correspond with bytes in the data area which should
     * be encrypted or decrypted. Bytes which corresponding bit is not set (0) are skipped and not modified. In a
     * single byte of this bitmap, the lowest bit (0) corresponds with the lowest byte within an 8 byte group in
     * the area and the highest bit (7) corresponds with the highest byte within this group.
     */
    public byte[] pbGapMap = null;
   
    /**
     * is only used if the WKB_AREA_SELECT flag is set and points to the WKBSELECT structure for the indirect selection
     * which is normally pointed by the pvCtrl parameter of the <A HREF = 
     * "WkJavaApi.html#WkbSelect2(long, long, long, java.lang.Object)">WkbSelect2</A> method. If the WKB_AREA_SELECT flag is
     * not set, this value must be set to NULL.
     */
    public WKBSELECT pvCtrl;
  }
 
 
  /**
   * (0x0000) standard data are addressed which can be freely modified. On Windows 3.11, Windows 95/98,
   * Windows NT/2000 and other protected operating systems, only write-permitted data may be addressed with this
   * (default) option.
   */
  public static final long WKB_AREA_DATA        = 0x0000;

  /**
   * (0x0001) The encryption/decryption includes a previous selection and subsequent un-selection. The 
   * selection flags are specified in the flSelCtrl member, the Selection Code in ulSelectCode. The 
   * WKBSELECT structure for the indirect selection is addressed by the pvCtrl member. Normally this 
   * flag must be used except the WkbCrypt2 method is called after the obsolete WkbSelect2 method.
   */
  public static final long WKB_AREA_SELECT		= 0x0001;
 
  /**
   * (0x0010) executable code is addressed which must be modified with care and activates additional driver operations
   * under some operating systems or programming languages.
   */
  public static final long WKB_AREA_CODE        = 0x0010;
 
  /**
   * (0x0080) doesn't encrypt any part of the specified buffer. This option is used for test purposes during the
   * development process.
   */
  public static final long WKB_AREA_NONE        = 0x0080;
 
  /**
   * (0x0002) After the encryption/decryption phase the 16-bit CRC of the result is calculated and compared with the
   * value ulCrc. If the values are not identical, the error code WKB_ERROR_CRC_VERIFY (15) is returned. If the
   * specified data to encrypt or decrypt has already these CRC value before the encryption or decryption process,
   * the driver assumes that this process is already made and is ignored. In the case of ignore, the
   * <A HREF = "WkJavaApi.html#WkbCrypt2(long, long, byte[], com.wibu.WkJavaApi.WKBAREA, WkJavaApi.pInt)">WkbCrypt2</A> method
   * returns 0 for the number of encrypted or decrypted bytes. If this option is combined with a specified gap table in
   * the pbGapMap parameter, skipped bytes do not influence the CRC calculation. If an indirect encryption algorithm is
   * selected, the CRC checking after the encryption is ignored. This option is ignored if WKB_AREA_NONE is specified.
   */
  public static final long WKB_AREA_CHKCRC      = 0x0002;
 
  /**
   * (0x0004) Before the specified area is encrypted or decrypted, the 16-bit CRC of this area is calculated and
   * stored into the ulCrc member. If this option is combined with WKB_AREA_CHKCRC, the new CRC value is stored not
   * before the previously stored value was used for the check operation. If this option is combined with a specified
   * gap table in the pbGapMap parameter, skipped bytes do not influence the CRC calculation.
   */
  public static final long WKB_AREA_CALCCRC     = 0x0004;
 
  /**
   * direct encryption/decryption via the WIBU-BOX
   */
  public static final long WKB_SEL_DIRECT       = 0x0000;
 
  /**
   * indirect Knuth encrypt/decryption
   */
  public static final long WKB_SEL_KNUTH20      = 0x0010;
 
  /**
   * indirect original FEAL decryption
   */
  public static final long WKB_SEL_DEC_FEAL     = 0x0020;
 
  /**
   * indirect original FEAL encryption
   */
  public static final long WKB_SEL_ENC_FEAL     = 0x0030;
 
  /**
   * indirect permutation decryption
   */
  public static final long WKB_SEL_DEC_PERM     = 0x0040;
 
  /**
   * indirect permutation encryption
   */
  public static final long WKB_SEL_ENC_PERM     = 0x0050;
 
  /**
   * indirect symmetric FEAL encryption/decryption
   */
  public static final long WKB_SEL_SYMFEAL      = 0x0060;
 
  /**
   *  Use cached Selection-init-data (WKB_SEL_SET_CACHE)
   */
  public static final long WKB_SEL_USE_CACHE    = 0x00F0;
  
  /**
   * command mask for <flCtrl> 
   */
  public static final long WKB_SEL_MASK		= 0x00F0;

  /**
   * (bit 31) reduces the Limit Counter of the entry by 1 if such a counter exists.
   */
  public static final long WKB_SC_BOXLIMIT      = 0x80000000;
 
  /**
   * (bit 30) touches the user limitation in a WkNet or WkLAN subsystem. This flag is only permitted, if
   * the <A HREF = "WkJavaApi.html#WkbOpen2(long, long, java.lang.String, long, long, java.lang.Object)">WkbOpen2</A> call,
   * which has opened the used entry, has not set the WKB_OPEN_NONETLIMIT option.
   */
  public static final long WKB_SC_NETLIMIT      = 0x40000000;
 
  /**
   * (bit 29) checks the expiration date of the opened WIBU-BOX entry if it has such an added entry.
   */
  public static final long WKB_SC_EXPDATE         = 0x20000000;

  /**
   * (bit 28) check for Station Share.
   */
  public static final long WKB_SC_STATIONSHARE    = 0x10000000;

  /**
   * mask for get Option bits
   */
  public static final long WKB_SC_EXTMASK         = 0xF0000000;

  /**
   * mask to get Core bits of Selection Code value
   */
  public static final long WKB_SC_COREMASK        = 0x0FFFFFFF;

  /**
   * This structure specifies additional information for the selection of the indirect encryption or decryption
   * by the pvCtrl member of a <A HREF = "WkJavaApi.WKBAREA.html">WKBAREA</A> class or by the obsolete 
   * <A HREF = "WkJavaApi.html#WkbSelect2(long, long, long, java.lang.Object)">WkbSelect2</A> method.
   */
  public static class WKBSELECT
  {
    /**
     * This member contains several flags which are separated in different groups.
     */
    public long flCtrl;
   
    /**
     * receives an identification value after the call of <A HREF = 
     * "WkJavaApi.html#WkbSelect2(long, long, long, java.lang.Object)">WkbSelect2</A> with a selected indirect encryption or
     * decryption and a set WKB_SEL_SET_CACHE option in the flCtrl member. This identification value points to the
     * driver internally stored initialization data of this indirect encryption. This identification can be specified
     * in the ulSelectCode parameter of a later <A HREF = 
     * "WkJavaApi.html#WkbSelect2(long, long, long, java.lang.Object)">WkbSelect2</A> call with the WKB_SEL_USE_CACHE 
     * option set in its flCtrl parameter to speed up the repeated execution of the same indirect selection.
     */
    public long ulCacheId; //returns Cache when WKB_SEL_SET_CACHE is set
   
    /**
     * contains the 16-bit CRC value which is used to verify the initialization of the indirect encryption or decryption.
     */
    public long ulCrc;
   
    /**
     * contains the length of the initialization data of the abInit member. This can be a value in the range 1
     * between 1 up to 20.
     */
    public int cbInit = 1; //length of abInit = 1 ... 20
   
    /**
     * contains the initialization data which is sent to the WIBU-BOX and encrypted directly via the specified
     * Selection Code and used to initialize indirect encryption/decryption algorithm. The length of the initialization
     * data in this member depends on the value in the cbInit member.
     */
    public byte[] abInit = { 0 };
  }
 
 
    /**
     * (0x0000) use the FEAL 8 algorithm (8 encipher rounds).
     */
    public static final long WKB_SEL_FEAL8      = 0x0000;
   
    /**
     * (0x0001) use the FEAL 16 algorithm (16 encipher rounds).
     */
    public static final long WKB_SEL_FEAL16     = 0x0001;
   
    /**
     * (0x0002) use the FEAL 32 algorithm (32 encipher rounds)
     */
    public static final long WKB_SEL_FEAL32     = 0x0002;
 
    /**
     * (0x0000) uses an operand length of 1 byte for the exchange operations of the permutation.
     */
    public static final long WKB_SEL_OPLEN1     = 0x0000;
   
    /**
     * (0x0001) uses an operand length of 2 bytes for the exchange operations of the permutation.
     */
    public static final long WKB_SEL_OPLEN2     = 0x0001;
   
    /**
     * (0x0002) uses an operand length of 4 bytes for the exchange operations of the permutation.
     */
    public static final long WKB_SEL_OPLEN4     = 0x0002;
   
    /**
     * (0x0003) uses an operand length of 8 bytes for the exchange operations of the permutation.
     */
    public static final long WKB_SEL_OPLEN8     = 0x0003;
   
    /**
     * increases security for original FEAL, use it
     */
    public static final long WKB_SEL_CBC_CRYPT  = 0x0008;
   
    /**
     * Caches Selection-init-data
     */
    public static final long WKB_SEL_SET_CACHE  = 0x0010;
   
    /**
     * Calc CRC of Selection Process
     */
    public static final long WKB_SEL_CALCCRC    = 0x0040;
   
    /**
     * Check CRC from a same Selection with WKB_SEL_CALCCRC flag set.
     */
    public static final long WKB_SEL_CHKCRC     = 0x0020;


  /**
   * This is a helping method for <A HREF = 
   * "WkJavaApi.html#WkbCrypt2(long, long, byte[], com.wibu.WkJavaApi.WKBAREA, WkJavaApi.pInt)">WkbCrypt2.</A>
   */
  public static class pInt
  {
    /**
     *
     */
    public int value;
  }


  // *********************************************************************************************************
  //                                         Driver Local Memory Management
  // *********************************************************************************************************
 
  // ********************************************************************************************************* //
  // NATIVE CALL WkbAllocMem
  // ********************************************************************************************************* //
 
  /**
   * This method allocates a memory block within the WibuKey driver.
   * <br> <I>This method is not tested for compatibility with Java and may not work. <B>Usage is not recommended.</B></I>
   * @param hwkbsys contains the handle of the subsystem for which the memory block is allocated. If this
   *                subsystem is closed by <A HREF = "WkJavaApi.html#WkbRelease2(long)">WkbRelease2</A>, this memory block
   *                is freed automatically. To allocate memory for the local subsystem the predefined handle
   *                value HWKB_LOCAL can be used.
   * @param cbAlloc contains the length of allocated memory block. This must be a value between 0 and 65500.
   * @return This method returns a valid handle if the operation was successful and NULL if an error occurred.
   *         In this case a subsequent call of the <A HREF = "WkJavaApi.html#WkbGetLastError()">WkbGetLastError</A>
   *         method returns the corresponding error code.
   */
  public static native long WkbAllocMem(long hwkbsys, long cbAlloc);

  
  // ********************************************************************************************************* //
  // NATIVE CALL WkbFreeMem
  // ********************************************************************************************************* //
 
  /**
   * This method destroys a memory block within the WibuKey driver.
   * <br> <I>This method is not tested for compatibility with Java and may not work. <B>Usage is not recommended.</B></I>
   * @param hwkbmem contains the handle of a memory block which was allocated by
            <A HREF = "WkJavaApi.html#WkbAllocMem(long,long)">WkbAllocMem.</A>
   * @return The method returns TRUE if the operation was successful and FALSE if an error occurred.
   *         In this case subsequent call of the <A HREF = "WkJavaApi.html#WkbGetLastError()">WkbGetLastError</A>
   *         method returns the corresponding error code.
   */
  public static native boolean WkbFreeMem(long hwkbmem);


  // ********************************************************************************************************* //
  // NATIVE CALL WkbWriteMem
  // ********************************************************************************************************* //

  /**
   * This method writes data from a memory address into an allocated driver memory block.
   * <br> <I>This method is not tested for compatibility with Java and may not work. <B>Usage is not recommended.</B></I>
   * @param hwkbmem contains the handle of a memory block which was allocated by
   *                <A HREF = "WkJavaApi.html#WkbAllocMem(long,long)">WkbAllocMem.</A>
   * @param pvSrc points to the data which are written into the memory block. This must contain at least
   *              cbData or more bytes. This parameter may also contain a memory handle of another
   *              created memory block.
   * @param cbData contains the number of bytes which should be transferred. There are never more bytes
   *               transferred than the allocated memory block can hold.
   * @return The method returns the number of actually transferred bytes. If this value is 0, an error occurred.
   *         In this case a subsequent call of the <A HREF = "WkJavaApi.html#WkbGetLastError()">WkbGetLastError</A>
   *         method returns the corresponding error code.
   */
  public static native long WkbWriteMem(long hwkbmem, byte[] pvSrc, long cbData);
  
  
  // ********************************************************************************************************* //
  // NATIVE CALL WkbReadMem
  // ********************************************************************************************************* //
  
  /**
   * This method reads data from an allocated driver memory block.
   * <br> <I>This method is not tested for compatibility with Java and may not work. <B>Usage is not recommended.</B></I>
   * @param hwkbmem contains the handle of a memory block which was allocated by
   *                <A HREF = "WkJavaApi.html#WkbAllocMem(long,long)">WkbAllocMem.</A>
   * @param pvDest points to a memory buffer which receives the data from the memory block. The memory buffer must
   *               be hold at least cbDest or more bytes. This parameter may also contain a memory handle of another
   *               created memory block.
   * @param cbDest contains the number of bytes which should be transferred. There are never more bytes transferred
   *               than the allocated memory block can hold.
   * @return The method returns the number of actually transferred bytes. If this value is 0, an error occurred.
   *         In this case a subsequent call of the <A HREF = "WkJavaApi.html#WkbGetLastError()">WkbGetLastError</A>
   *         method returns the corresponding error code.
   */
  public static native long WkbReadMem(long hwkbmem, byte[] pvDest, long cbDest);


  // ******************************************************************************************************** //
  // NATIVE CALL WibuKJniGetVersionString()
  // ******************************************************************************************************** //
 
  /**
   * This method retrieves the version of the WibuKey JNI library.
   *
   * @param pszDest receives the version string.
   *
   * @return length of pszDest. 0 if any error occurred.
   * In this case a subsequent call of the <A HREF = "WkJavaApi.html#WkbGetLastError()">WkbGetLastError</A> method 
   * returns the corresponding error code.
   * @deprecated This method is obsolete and replaced by the WKB_QI_JNI flag
   * which can be specified in a call to <A HREF = "WkJavaApi.html#WkbQueryInfo2(long, long, java.lang.Object, int)">WkbQueryInfo2</A>
   * 
   */
  public static native int WibuKJniGetVersionString(StringBuffer pszDest);
  
  /* =======================================================================
   * WibuKey Extended Memory Support (Classes, Constants)
   * =======================================================================
   */

  /**
   *  flag used in WKBEXTMEM - user data in ExtMem Page: unprotected 
   */
  public static final int WKB_XM_USER       = 0;

  /** 
   * flag used in WKBEXTMEM - protected data in ExtMem page: Access Signature 
   */
  public static final int WKB_XM_PROT       = 1;

  /* -------------------
   * WKBEXTMEM structure
   * -------------------
   * specification to read or write a single ExtMem Page
   * used by WkbListBox2() and WkbProgram2()
   * @see <A HREF = "WkJavaApi.html#WkbListBox2(long, long, java.lang.String, com.wibu.WkJavaApi.WKBSERIAL, com.wibu.WkJavaApi.WKBBOXDATA)">WkbListBox2</A>
   * @see <A HREF = "WkJavaApi.html#WkbProgram2(long, long, java.lang.String, com.wibu.WkJavaApi.WKBSERIAL, com.wibu.WkJavaApi.WKBPROGRAM)">WkbProgram2</A>
   */

  public static class WKBEXTMEM 
  {
    /**
     *  contains WKB_XM_xxx 
     */
    public short  fsCtrl;       
 
    /** 
     * contains index of the ExtMem page 
     */
    public short iPage;  
        
    /** 
     * contains access signature for WKB_XM_PROT 
     */
    public byte [] abSignature = new byte[8];

    /** 
     * receives or contains data of ExtMem page 
     */
    public byte [] abData = new byte[32];   
  
} ; /* prefix "wkbxm" */

  // ********************************************************************************************************* //
  // NATIVE CALL WkbGetFC
  // ********************************************************************************************************* //
 
  /**
   * This function returns the current set User Code. 
   * Internal method for use in KEY4J - do not use explicitly!
   * @return The method returns the Firm Code of the last decrypted class file.
   */
  public static native int WkbGetFC();

  // ********************************************************************************************************* //
  // NATIVE CALL WkbGetUC
  // ********************************************************************************************************* //
 
  /**
   * This function returns the current set User Code.
   * Internal method for use in KEY4J - do not use explicitly!
   * @return The method returns the User Code of the last decrypted class file.
   */
  public static native int WkbGetUC();

  // ********************************************************************************************************* //
  //                     Special Functions for KEY4J (STEP2E)
  // ********************************************************************************************************* //

  // ********************************************************************************************************* //
  // NATIVE CALL LoadClass
  // ********************************************************************************************************* //
 
  /**
   * Internal method for use in KEY4J - do not use explicitly!
   */
  public static native Class LoadClass(String strClassName, Object classLoader, byte [] bEncryptedClassFile,
    int cbData);

  /**
   * Internal method for use in KEY4J
   */
  public static native int LastLoadClassError();

  // ********************************************************************************************************* //
  // NATIVE CALL EncryptClass
  // ********************************************************************************************************* //
 
  /**
   * Internal method for use in KEY4J - obosolete!
   */
  public static native int  EncryptClass(int nSecurityMode, int nAccessMode, byte [] bHeaderData, 
    byte [] bClassData, long lFirmCode, long lUserCode, long lSelectionCode);

  // ********************************************************************************************************* //
  // NATIVE CALL InitKey4JPool
  // ********************************************************************************************************* //
 
  /**
   * Internal method for use in KEY4J - do not use explicitly
   */
  public static native boolean InitKey4JPool(int nPoolSize);
  
  
  // ********************************************************************************************************* //
  //                                                 entry code
  // ********************************************************************************************************* //

  static
  {
    String  strJniLib = "wibuKJni";
    String	strOsName = System.getProperty("os.name");
    
    // load the wibuJNI dll
    try
    {

      if (System.getProperty ("com.wibu.wibukjni.path") != null) {
        // Applet or application providing it's own wibuKJni lib in 
        // a separate location
        System.load (new java.io.File (System.getProperty ("com.wibu.wibukjni.path"), 
          System.mapLibraryName (strJniLib)).getAbsolutePath ());
      } else if (strOsName.compareTo("Linux") != 0) {
        // default location on Windows/Mac
        System.loadLibrary(strJniLib);
      } else {
        // in linux we have 2 tries
        boolean fLoaded = true;
        try 
        {
          // 1st try: default path
          System.loadLibrary(strJniLib);
        } catch ( UnsatisfiedLinkError ex ) {
          // ignore this try
          fLoaded = false;
        } // try/catch

        if ( ! fLoaded ) {
          System.load("/usr/lib/lib" + strJniLib + ".so");
        } // if
      } // if/else
      // System.out.println("library loaded.. ");
    } 
    catch (SecurityException se)
    {
      System.err.print("Security Error ");
      System.err.println("[unable to load program library " + System.mapLibraryName(strJniLib) + "!]");
      // System.exit(0);
      throw se;
    } 
    catch (UnsatisfiedLinkError uslk)
    {
      System.err.print("Link Error ");
      System.err.println("[unable to load program library " + System.mapLibraryName(strJniLib) + "!]");
      // System.exit(0);
      throw uslk;
    } // try/catch
  } // static
} // class WkJavaApi

// =========================================================
// wk/dev/lib/java/WkJavaApi: end of com.wibu.WkJavaApi.java
// =========================================================
