/*****************************************************************************
wibuKJni                                      WibuKey API Java Native Library
==============================================================================
wibuKJni.c                          Wrapper functions to call the WibuKey API
==============================================================================

This C++ file contains all required wrapper functions which are called from 
the WkJavaApi class and themselves call the WibuKey API library (wkwin32.dll,
WkMacLib or libwklin.so)

==============================================================================


==============================================================================

Copyright  2001-2008 by WIBU-SYSTEMS AG
           Rueppurrer Strasse 54, D-76137 Karlsruhe, Germany/Europe
           Website: http://www.wibu.com, Email: info@wibu.com
           Phone: +49-721-93172-0, Fax: +49-721-93172-22
           All rights reserved.

Version .. 4.30 of 2008Oct02
Project .. wibu/wk/dev/lib/java
Author.... Juergen Ockert (WIBU), Heiko Abraham (WIBU)
System ... MacOS X, Linux
Tools .... gcc

Change Report
03Sep15 4.00 jo: Initial version
08Oct02 4.30 ha: adaptation for Arm/Linux armv4l

*****************************************************************************/

#if ! defined(__PROJ_H__)
#define __PROJ_H__


#define DRV_VERSION         4
#define DRV_SUBVERSION      30
#define DRV_VER_BUILD       42
#define DRV_VER_COUNT       0

#define DRV_VER_DAY         14
#define DRV_VER_MONTH       4
#define DRV_VER_YEAR        8


#if defined(_DEBUG)
  #define JNI_DEBUG         1
#else
  #define JNI_DEBUG         0
#endif


// include local jni header
#if defined(__linux__) || defined(__APPLE__)
  #if !defined(_UNIX)
    #define _UNIX 1
  #endif

  #include "jni.h"
#endif

#include "../wk.h"


/*****************************************************************************

---- Function prototypes -----------------------------------------------------

*****************************************************************************/


#ifdef __cplusplus
extern "C" {
#endif
  // helper functions for conversions
  void J2C_WKBSERIAL(JNIEnv *env, jobject serial, WKBSERIAL *wkbsr);
  void J2C_String(JNIEnv *env, jobject jstrSource, char *pszDest, int cbDest);
  int  C2J_AppendToStringBuffer(JNIEnv *ev, jobject pszDest, const char *charDest, 
                                const char *errMessage);
  void  J2C_ByteArray(JNIEnv *ev, jbyteArray jbaSource, BYTE *pbDest, LONG cbDest);
  // debug output
  #if JNI_DEBUG 
    void PrintDbgWkbSerial(WKBSERIAL *wkbsr, const char *pszPrefix);
    void PrintDbgMsg(const char *pszMsg, int nLine);
    void PrintDbgWkbArea(WKBAREA *pwka, int nLine);
  #endif
#ifdef __cplusplus
};
#endif

#if JNI_DEBUG 
  #define DUMP_WKBACCESS(arg)           PrintDbgAccess(&(arg), __LINE__)
  #define DUMP_WKBOPENENTRY(arg, port)  PrintDbgOpenEntry(&(arg), port, __LINE__)
  #define DUMP_WKBAREA(arg)             PrintDbgWkbArea(&arg, __LINE__)
  #define DUMB_WKBSERIAL(arg)           PrintDbgWkbSerial(&arg, "")
#else
  #define DUMP_WKBACCESS(arg);
  #define DUMP_WKBOPENENTRY(arg, port);
  #define DUMP_WKBAREA(arg);
  #define DUMB_WKBSERIAL(arg);
#endif // JNI_DEBUG


#endif // __PROJ_H__
// =======================================
// wk/dev/lib/java/wibuKJni: end of proj.h
// =======================================
