/*****************************************************************************
wibuKJni                                      WibuKey API Java Native Library
==============================================================================
wibuKJni.c                          Wrapper functions to call the WibuKey API
==============================================================================

This C++ file contains all required wrapper functions which are called from 
the WkJavaApi class and themselves call the WibuKey API library (wkwin32.dll,
WkMacLib or libwklin.so)

==============================================================================


==============================================================================

Copyright  2001-2008 by WIBU-SYSTEMS AG
           Rueppurrer Strasse 54, D-76137 Karlsruhe, Germany/Europe
           Website: http://www.wibu.com, Email: info@wibu.com
           Phone: +49-721-93172-0, Fax: +49-721-93172-22
           All rights reserved.

Version .. 4.20 of 2008Oct02
Project .. wibu/wk/dev/lib/java
Author.... Reto Mettler (Stomedia), Freddy Kuehne (Stomedia), 
           Juergen Ockert (WIBU), Heiko Abraham (WIBU)
System ... Windows 95/98/Me, Windows NT 3.51-5.10, MacOS X, Linux
Tools .... Microsoft Visual C++ 7.10 / gcc 3.3.x

Change Report
02Feb19 3.20 jo: changed class name from com.stomedia.wibu.Api to 
                 com.wibu.WkJavaApi
02Feb21 3.20 jo: replaced 'WkbUnSelect' call with 'WkbUnSelect2'
02Feb22 3.20 jo: bugfix in WkbListPort2, added version funcktions
02Mar07 3.21 jo: implemented 'WkbProgram2'
02Mar19 3.21 jo: WkbListPort2: init port list
02Apr23 3.21 jo: bugfix WkbQueryEntry2 / WKB_QE_OPEN
02Sep19 3.30 jo: WkbProgram2: Firm/User Code included
02Sep27 3.31 jo: remove WKB_USEOFFSETS flag from WKBACCESS.flCtrl
02Oct21 3.31 jo: WkbCrypt2: Check if WKB_AREA_SELECT is set in c_area.flCtrl
03Feb27 3.32 jo: added WkbQueryEntry2(WKB_QE_MULTIENTRY) (B-JAV012)
03Apr04 3.32 jo: WkbOpen2 understand WKB_OPEN_ENTRY flag
03Apr23 3.32 jo: WkbOpen2: WKB_OPEN_ENTRY fixed
03Aug06 3.32 jo: Bugfix in WkbQueryInfo2/WKB_QI_ERRTEXT
03Aug21 4.00 jo: Support of Extended Memory
04Mai18 4.10 jo: Bugfix in WkbListBox2 (WKB_LIST_EXTMEM), memory leaks in 
                 WkbListBox2()
08Oct02 4.30 ha: addaptation for Linux/Arm JamVM


*****************************************************************************/

#include "proj.h"
#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include <assert.h>


// Organize exports
#if ! defined(DLL_EXPORT)
  #if __GNUC__ >= 4
    #define DLL_EXPORT  __attribute__((visibility("default")))
  #else
    #define DLL_EXPORT 
  #endif
#endif

#if defined(min)
  #define wbs_min min
#else
  #define wbs_min(a,b) ((a)<(b)? (a):(b))
#endif

#define   WKB_QI_JNI  0x0004

#if JNI_DEBUG // defined(JNI_DEBUG)
void PrintDbgMsg(const char *pszMsg, int nLine)
{
  fprintf(stdout, "%s (#%d)\n", pszMsg, nLine);
  fflush(stdout);
} // PrintDbgMsg()

static void PrintDbgAccess(WKBACCESS* pwkba, int nLine)
{
  fprintf(stdout, "Dumping WKBACCESS at line %d:\n", nLine);
  fprintf(stdout, " pwkba->flCtrl        : %x\n", pwkba->flCtrl);
  fprintf(stdout, " pwkba->lFirmCode     : %x\n", pwkba->lFirmCode);
  fprintf(stdout, " pwkba->lUserCode     : %x\n", pwkba->lUserCode);
  fprintf(stdout, " pwkba->flStdCtrl     : %x\n", pwkba->flStdCtrl);
  fprintf(stdout, " pwkba->bVersionLow   : %x\n", pwkba->bVersionLow);
  fprintf(stdout, " pwkba->bVersionHigh  : %x\n", pwkba->bVersionHigh);
  fprintf(stdout, " pwkba->idNetUser     : %x\n", pwkba->idNetUser);
  fprintf(stdout, " pwkba->pszAppName    : %x\n", (void*) pwkba->pszAppName);
  fprintf(stdout, " pwkba->pszServerName : %x\n", (void*) pwkba->pszServerName);
  fflush(stdout);
} // PrintDbgAccess()

void PrintDbgWkbSerial(WKBSERIAL *wkbsr, const char *pszPrefix)
{
  const char *szPrefix = pszPrefix ? pszPrefix : "";

  if ( pszPrefix == 0 ) {
    fprintf(stdout, "Dumping WKBSERIAL:\n");
  } // if

  fprintf(stdout, "%swkbsr           : 0x%x\n", szPrefix, wkbsr);
  if ( wkbsr != 0 ) {
    fprintf(stdout, "%swkbsr->usMask   : %d\n", szPrefix, wkbsr->usMask);
    fprintf(stdout, "%swkbsr->ulSerial : %d\n", szPrefix, wkbsr->ulSerial);
  } // if
} // PrintDbgWkbSerial()

static void PrintDbgWkbSelect(WKBSELECT *pwkbsel, const char *pszPrefix)
{
  int         i;
  const char *szPrefix = pszPrefix ? pszPrefix : "";

  if ( pszPrefix == 0 ) {
    fprintf(stdout, "Dumping WKBSELECT:\n");
  } // if

  fprintf(stdout, "%swkbsel            : 0x%x\n", szPrefix, pwkbsel);
  if ( pwkbsel != 0 ) {
    fprintf(stdout, "%swkbsel->flCtrl    : 0x%x\n", szPrefix, pwkbsel->flCtrl);
    fprintf(stdout, "%swkbsel->ulCacheId : %d\n", szPrefix, pwkbsel->ulCacheId);
    fprintf(stdout, "%swkbsel->ulCrc     : 0x%x\n", szPrefix, pwkbsel->ulCrc);
    fprintf(stdout, "%swkbsel->cbInit    :%d\n", szPrefix, pwkbsel->cbInit);
    fprintf(stdout, "%swkbsel->abInit[]  : ", szPrefix);
    for ( i=0; i<pwkbsel->cbInit; i++) {
      fprintf(stdout, "%02x ", (unsigned char) pwkbsel->abInit[i]);
    } // for
    fprintf(stdout, "\n");
  } // if

  return;
} // PrintDbgWkbSelect()

static void PrintDbgMemory(const char *pszDescription, int nLine,
                           unsigned char *pbData, int cbData)
{
  int   i;
  fprintf(stdout, "Dumping WKBOPENENTRY (BINARY) at line %d:\n", nLine);
  for ( i=0; i<cbData; i++) {
    fprintf(stdout, "%02X ", pbData[i]);
    fflush(stdout);
  } // for
  fprintf(stdout, "\n");
  fflush(stdout);
} // PrintDbgMemory()

static void PrintDbgOpenEntry(WKBOPENENTRY *pwkoe, char *pszPort, int nLine)
{
  fprintf(stdout, "Dumping WKBOPENENTRY at line %d:\n", nLine);
  fprintf(stdout, " pwkoe->flCtrl          : 0x%x\n", pwkoe->flCtrl);
  fprintf(stdout, " pwkoe->hwkbsys         : 0x%x\n", pwkoe->hwkbsys);
  fprintf(stdout, " pwkoe->lPort           : 0x%x\n", pwkoe->lPort);
  fprintf(stdout, " pwkoe->pszPort         : 0x%x\n", pwkoe->pszPort);
  fprintf(stdout, " szport                 : 0x%x\n", pszPort);
  if ( pwkoe->pszPort ) {
    fprintf(stdout, " pwkoe->pszPort         : %s\n", pwkoe->pszPort);
  } // if
  if ( pszPort ) {
    fprintf(stdout, " szport                 : '%s'\n", pszPort);
  } // if
  fprintf(stdout, " pwkoe->usVersion       : 0x%x\n", pwkoe->usVersion);
  fprintf(stdout, " pwkoe->ulFirmCode      : %d\n", pwkoe->ulFirmCode);
  fprintf(stdout, " pwkoe->ulUserCode      : %d\n", pwkoe->ulUserCode);
  PrintDbgWkbSerial(&(pwkoe->wkbsr), " pwkoe->");
} // PrintDbgOpenEntry()

void PrintDbgWkbArea(WKBAREA *pwka, int nLine)
{
  fprintf(stdout, "Dumping WKBAREA at line %d:\n", nLine);
  fprintf(stdout, " pwka->flCtrl        : 0x%x\n", pwka->flCtrl);
  fprintf(stdout, " pwka->flSelCtrl     : 0x%x\n", pwka->flSelCtrl);
  fprintf(stdout, " pwka->pvCtrl        : 0x%x\n", pwka->pvCtrl);
  fprintf(stdout, " pwka->ulSelectCode  : %d\n", pwka->ulSelectCode);
  fprintf(stdout, " pwka->ulCrc         : %d\n", pwka->ulCrc);
  fprintf(stdout, " pwka->pbGapMap      : 0x%x\n", pwka->pbGapMap);
  fprintf(stdout, " pwka->pvCtrl        : 0x%x\n", pwka->pvCtrl);
  fprintf(stdout, " pwka->pvCtrl        : 0x%x\n", pwka->pvCtrl);

  if ( pwka->pvCtrl != 0 ) {
    if ( pwka->flCtrl & WKB_AREA_SELECT ) {
      WKBSELECT *pwkbsel = (WKBSELECT *) pwka->pvCtrl;
      PrintDbgWkbSelect(pwkbsel, " pwka->pvCtrl->");
    } // if
  } // if
} // PrintDbgWkbArea()

#endif

// ***************************************************************************************
// HELPING FUNCTIONS
// ***************************************************************************************

/**
 * creates a c string from a java StringBuffer
 * return 1 if call was successfull, else 0
 */
static int Create_C_Str_From_Java(JNIEnv *ev,jobject pszDest, char *charDest, char *errMessage)
{
	jclass objectCls;
	jstring str;
	jmethodID mid_toString;
	int len = 0;

	objectCls = (*ev)->FindClass(ev, "java/lang/StringBuffer");
	
	if (objectCls == NULL)
		printf("error getting class; [%s]\n", errMessage);	
	
	if (!(pszDest == NULL))
	{
		// method ID's from the StringBuffer Class
		mid_toString = (*ev)->GetMethodID(ev, objectCls, "toString","()Ljava/lang/String;"); 
		
		// check if an error dedect
		if (mid_toString == NULL)
			printf("error by getting toString method ID; [%s]\n",errMessage);
		
		// mapping StringBuffer -> String
		str = (*ev)->CallObjectMethod(ev, pszDest, mid_toString);
		
		len = (*ev)->GetStringLength(ev, str);
		(*ev)->GetStringUTFRegion(ev, str, 0, len, charDest);
	
		return 1;
	}
	
	return 0;
} // Create_C_Str_From_Java()

/**
 * setting a java StringBuffer to length zero
 * return 1 if call was successfull, else 0
 */
static int Delete_Str_In_Buffer(JNIEnv *ev,jobject pszDest, char *errMessage)
{
	jclass objectCls;
	jmethodID mid_setLength;
	
	objectCls = (*ev)->FindClass(ev, "java/lang/StringBuffer");
	
	if (objectCls == NULL) {
		printf("error getting class; [%s]\n",errMessage);
		return 0;
	}	
	
	if ( pszDest != NULL )	{
		// method ID's from the StringBuffer Class
		mid_setLength = (*ev)->GetMethodID(ev, objectCls, "setLength", "(I)V");
		
    if (mid_setLength == NULL) {
			printf("error by getting setLength method ID; [%s]\n", errMessage);
			return 0;
		} // if
		
		(*ev)->CallVoidMethod(ev, pszDest, mid_setLength, 0);
		
		return 1;
	} // if

	return 0;	
} // Delete_Str_In_Buffer()

/**
 * append a string to a Buffer
 * return 1 if call was successfull, else 0
 */
int C2J_AppendToStringBuffer(JNIEnv *ev, jobject pszDest, const char *charDest, 
                                const char *errMessage)
{
	jclass objectCls;
	jobject returnObj;
	jmethodID mid_append;
	
	objectCls = (*ev)->FindClass(ev, "java/lang/StringBuffer");
	if (objectCls == NULL) {
		printf("error getting class; [%s]\n", errMessage);
		return 0;
	}	

	if (!(pszDest == NULL))	{
		// method ID's from the StringBuffer Class
		mid_append = (*ev)->GetMethodID(ev, objectCls, "append","(Ljava/lang/String;)Ljava/lang/StringBuffer;");
		
		// check if an error dedect
		if (mid_append == NULL)	{
			printf("error by getting append method ID; [%s]\n", errMessage);
			return 0;
		}
			
		// append 
		// mapping c -> java
		returnObj = (*ev)->CallObjectMethod(ev, pszDest, mid_append, (*ev)->NewStringUTF(ev, charDest));
	
		return 1;
	}
	
	return 0;
} // C2J_AppendToStringBuffer()

void J2C_ByteArray(JNIEnv *jenv, jbyteArray jbaSource, BYTE *pbDest, LONG cbDest)
{
  jbyte   *jbp;
  jbp = (*jenv)->GetByteArrayElements(jenv, jbaSource, NULL);
  memcpy(pbDest, jbp, cbDest);
  (*jenv)->ReleaseByteArrayElements(jenv, jbaSource, jbp, JNI_ABORT);
} // J2C_ByteArray()


/*****************************************************************************
J2C_String
==============================================================================

copies a Java String (not StringBuffer!) into a C char array

Parameters
  env           JNI environment
  jstrSource    JAVA source string
  szDest        destination buffer
  cbDest        size of destination buffer

*****************************************************************************/

void J2C_String(JNIEnv *env, jobject jstrSource, char *pszDest, int cbDest)
{
  // get string length
  int nSize = wbs_min((int) (*env)->GetStringLength(env, jstrSource), cbDest-1);

  if ( nSize ) {
    // get string buffer from JAVA
    const char *pszStr = (*env)->GetStringUTFChars(env, jstrSource, 0);
    // copy string buffer to return value
    strncpy(pszDest, pszStr, nSize);
    pszDest[nSize] = 0;

    // release string buffer
    (*env)->ReleaseStringUTFChars(env, jstrSource, pszStr);
  } else {
    *pszDest = 0;
  } // if/else

} // J2C_String();

// ******************************************************************************************************/
// ClassIDs and FieldIDs of all mapped java-classes with initialisation-procedure
// ******************************************************************************************************/

struct
{
  jfieldID  flCtrl;
  jfieldID  lFirmCode;
  jfieldID  lUserCode;
  jfieldID  flStdCtrl;
  jfieldID  bVersionLow;
  jfieldID  bVersionHigh;
  jfieldID  idNetUser;
  jfieldID  pszAppName;
  jfieldID  pszServerName;
}access_ID;

jclass accessEntryCls;
static BOOL accessIsInit = FALSE;


/*****************************************************************************
InitAccess
==============================================================================

initialize the access_IDs

Parameters
   ev

Return
  static void 
  
*****************************************************************************/

static void InitAccess(JNIEnv* ev)
{
  if(!accessIsInit)
  {
    accessEntryCls = (*ev)->NewGlobalRef(ev, (*ev)->FindClass(ev, "com/wibu/WkJavaApi$WKBACCESS"));
    access_ID.flCtrl = (*ev)->GetFieldID(ev, accessEntryCls, "flCtrl", "J");
    access_ID.lFirmCode = (*ev)->GetFieldID(ev, accessEntryCls, "lFirmCode", "J");
    access_ID.lUserCode = (*ev)->GetFieldID(ev, accessEntryCls, "lUserCode", "J");
    access_ID.flStdCtrl = (*ev)->GetFieldID(ev, accessEntryCls, "flStdCtrl", "J");
    access_ID.bVersionLow = (*ev)->GetFieldID(ev, accessEntryCls, "bVersionLow", "B");
    access_ID.bVersionHigh = (*ev)->GetFieldID(ev, accessEntryCls, "bVersionHigh", "B");
    access_ID.idNetUser = (*ev)->GetFieldID(ev, accessEntryCls, "idNetUser", "I");
    access_ID.pszAppName = (*ev)->GetFieldID(ev, accessEntryCls, "pszAppName", "Ljava/lang/StringBuffer;");
    access_ID.pszServerName = (*ev)->GetFieldID(ev, accessEntryCls, "pszServerName", "Ljava/lang/StringBuffer;");
    accessIsInit = TRUE;
  }

} // InitAccess()


/***** WKBSERIAL *****/
struct
{
  jfieldID ulSerial;
  jfieldID usMask;
  jfieldID usReserved;
} serial_ID;

jclass serialCls;
static BOOL serialIsInit = FALSE;

/*****************************************************************************
InitSerial
==============================================================================

initialize the serial_IDs

Parameters
   ev

Return
  static void 
  
*****************************************************************************/

static void InitSerial(JNIEnv* ev)
{
  if(!serialIsInit)
  {
    serialCls = (*ev)->NewGlobalRef(ev, (*ev)->FindClass(ev, "com/wibu/WkJavaApi$WKBSERIAL"));
    serial_ID.ulSerial = (*ev)->GetFieldID(ev, serialCls, "ulSerial", "J");
    serial_ID.usMask = (*ev)->GetFieldID(ev, serialCls, "usMask", "S");
    serial_ID.usReserved = (*ev)->GetFieldID(ev, serialCls, "usReserved", "S");
    serialIsInit = TRUE;
  }
} // InitSerial()


/***** WKBBOXENTRY ******/
struct
{
  jfieldID  flCtrl;
  jfieldID  ulFirmCode;
  jfieldID  ulUserCode;
  jfieldID  ulData1;
  jfieldID  ulData2;
} boxEntry_ID;

jclass boxEntryCls;
static BOOL boxEntryIsInit = FALSE;

static void InitBoxEntry(JNIEnv* ev)
{
  if(!boxEntryIsInit)
  { 
    boxEntryCls = (*ev)->NewGlobalRef(ev, (*ev)->FindClass(ev, "com/wibu/WkJavaApi$WKBBOXENTRY"));  
    boxEntry_ID.flCtrl = (*ev)->GetFieldID(ev, boxEntryCls, "flCtrl", "J");
    boxEntry_ID.ulFirmCode = (*ev)->GetFieldID(ev, boxEntryCls, "ulFirmCode", "J");
    boxEntry_ID.ulUserCode = (*ev)->GetFieldID(ev, boxEntryCls, "ulUserCode", "J");
    boxEntry_ID.ulData1 = (*ev)->GetFieldID(ev, boxEntryCls, "ulData1", "J");
    boxEntry_ID.ulData2 = (*ev)->GetFieldID(ev, boxEntryCls, "ulData2", "J");
    boxEntryIsInit = TRUE;
  }
}

/***** WKBBOXDATA *****/
struct
{
  jfieldID  flCtrl;
  jfieldID  usVersion;
  jfieldID  cEntry;
  jfieldID  awkbbxe; //WKBBOXENTRY[10]
} boxData_ID;

jclass boxDataCls;
jmethodID newBoxEntry_ID;
static BOOL boxDataIsInit = FALSE;

static void InitBoxData(JNIEnv* ev)
{
  if(!boxDataIsInit)
  {
    InitBoxEntry(ev);
    boxDataCls            = (*ev)->NewGlobalRef(ev, (*ev)->FindClass(ev, "com/wibu/WkJavaApi$WKBBOXDATA"));
    newBoxEntry_ID        = (*ev)->GetMethodID(ev, boxEntryCls,"<init>","()V");
    boxData_ID.flCtrl     = (*ev)->GetFieldID(ev, boxDataCls, "flCtrl", "J");
    boxData_ID.usVersion  = (*ev)->GetFieldID(ev, boxDataCls, "usVersion", "S");
    boxData_ID.cEntry     = (*ev)->GetFieldID(ev, boxDataCls, "cEntry", "S");
    boxData_ID.awkbbxe    = (*ev)->GetFieldID(ev, boxDataCls, "awkbbxe", "[Lcom/wibu/WkJavaApi$WKBBOXENTRY;");
    boxDataIsInit = TRUE;
  }
} // InitBoxData()


/***** WKBBOXCONFIG ******/
struct 
{
  jfieldID flCtrl;
  jfieldID usVersion;
  jfieldID usReserved;
  jfieldID ulProgramCount;
  jfieldID ulConfig;
  jfieldID usExtMemSize; 
  jfieldID usAsicBuild;
} boxConfig_ID;

jclass boxConfigCls;
static BOOL boxConfigIsInit = FALSE;

static void InitBoxConfig(JNIEnv* ev)
{
  if ( ! boxConfigIsInit ) {
    boxConfigCls = (*ev)->NewGlobalRef(ev, (*ev)->FindClass(ev, "com/wibu/WkJavaApi$WKBBOXCONFIG"));
    boxConfig_ID.flCtrl = (*ev)->GetFieldID(ev, boxConfigCls, "flCtrl", "J");
    boxConfig_ID.usVersion = (*ev)->GetFieldID(ev, boxConfigCls, "usVersion", "S");
    boxConfig_ID.usReserved = (*ev)->GetFieldID(ev, boxConfigCls, "usReserved", "S");
    boxConfig_ID.ulProgramCount = (*ev)->GetFieldID(ev, boxConfigCls, "ulProgramCount", "J");
    boxConfig_ID.ulConfig = (*ev)->GetFieldID(ev, boxConfigCls, "ulConfig", "J");
    boxConfig_ID.usExtMemSize = (*ev)->GetFieldID(ev, boxConfigCls, "usExtMemSize", "S");
    boxConfig_ID.usAsicBuild = (*ev)->GetFieldID(ev, boxConfigCls, "usAsicBuild", "S");
    boxConfigIsInit = TRUE;
  } // if
} // InitBoxConfig()

/***** WKBVERSION ******/
struct
{
  jfieldID usVersion;
  jfieldID usSubVersion;
  jfieldID usBuild;
  jfieldID usCount;
  jfieldID usYear;
  jfieldID bMonth;
  jfieldID bDay;
  jfieldID ulReserved1;
} version_ID;

static jclass queryInfoCls = NULL;
static BOOL queryInfoIsInit = FALSE;

static void InitVersion(JNIEnv* ev)
{
  if(!queryInfoIsInit)
  { 
    jclass objectCls;
    objectCls = (*ev)->FindClass(ev, "com/wibu/WkJavaApi$WKBVERSION"); 
    if (objectCls == NULL) {
      // -- FindClass(WkJavaApi$WKBVERSION) failed
    } else {
      queryInfoCls = (*ev)->NewGlobalRef(ev, objectCls );
      version_ID.usVersion = (*ev)->GetFieldID(ev, queryInfoCls, "usVersion", "S");
      version_ID.usSubVersion = (*ev)->GetFieldID(ev, queryInfoCls, "usSubVersion", "S");
      version_ID.usBuild = (*ev)->GetFieldID(ev, queryInfoCls, "usBuild", "S");
      version_ID.usCount = (*ev)->GetFieldID(ev, queryInfoCls, "usCount", "S");
      version_ID.usYear = (*ev)->GetFieldID(ev, queryInfoCls, "usYear", "S");
      version_ID.bMonth = (*ev)->GetFieldID(ev, queryInfoCls, "bMonth", "B");
      version_ID.bDay = (*ev)->GetFieldID(ev, queryInfoCls, "bDay", "B");
      version_ID.ulReserved1 = (*ev)->GetFieldID(ev, queryInfoCls, "ulReserved1", "J");
      queryInfoIsInit = TRUE;
    }
  }
} // InitVersion()


/***** WKBSTATUS ******/
struct
{
  jfieldID usFlags;
  jfieldID usDrvInt;
} queryStatus_ID;

jclass queryStatusCls;
static BOOL queryStatusIsInit = FALSE;

static void InitQueryStatus(JNIEnv* ev)
{
  if(!queryStatusIsInit)
  {
  	queryStatusCls = (*ev)->NewGlobalRef(ev, (*ev)->FindClass(ev, "com/wibu/WkJavaApi$WKBSTATUS"));
  	queryStatus_ID.usFlags = (*ev)->GetFieldID(ev, queryStatusCls, "usFlags", "S");
  	queryStatus_ID.usDrvInt = (*ev)->GetFieldID(ev, queryStatusCls, "usDrvInt", "S");
  	queryStatusIsInit = TRUE;
  }	
}



/***** WKBPORT ******/
struct
{
  jfieldID ulAddress;
} port_ID;

jclass portCls;
static BOOL portIsInit = FALSE;

static void InitPort(JNIEnv* ev)
{
  if(!portIsInit)
  {
  	portCls = (*ev)->NewGlobalRef(ev, (*ev)->FindClass(ev, "com/wibu/WkJavaApi$WKBPORT"));
  	port_ID.ulAddress = (*ev)->GetFieldID(ev, portCls, "ulAddress", "J");
  	portIsInit = TRUE;
  }	
}


/***** WKBPORTNAME ******/
struct 
{
  jfieldID flCtrl;
  jfieldID lPort;
  jfieldID pszPort;

} portname_ID;

jclass portnameCls;
static BOOL portnameIsInit = FALSE;

static void InitPortName(JNIEnv* ev)
{
  if(!portnameIsInit)
  {
    portnameCls = (*ev)->NewGlobalRef(ev, (*ev)->FindClass(ev, "com/wibu/WkJavaApi$WKBPORTNAME"));
    portname_ID.flCtrl = (*ev)->GetFieldID(ev, portnameCls, "flCtrl", "J");
    portname_ID.lPort = (*ev)->GetFieldID(ev, portnameCls, "lPort", "J");
    portname_ID.pszPort = (*ev)->GetFieldID(ev, portnameCls, "pszPort", "Ljava/lang/String;");
    portnameIsInit = TRUE;	
  }
}


/***** WKBPROGRAMM ******/
struct
{
  jfieldID flCtrl;
  jfieldID wkbbxe;
  jfieldID abSignature;
} program_ID;

jclass programCls;
jmethodID newPrgBoxEntry_ID;
static BOOL programIsInit = FALSE;


static void InitProgram(JNIEnv* ev)
{
  if(!programIsInit)
  {
    InitBoxEntry(ev);
    programCls = (*ev)->NewGlobalRef(ev, (*ev)->FindClass(ev, "com/wibu/WkJavaApi$WKBPROGRAM"));
    newPrgBoxEntry_ID = (*ev)->GetMethodID(ev, boxEntryCls,"<init>","()V");
    program_ID.flCtrl = (*ev)->GetFieldID(ev, programCls, "flCtrl", "J");
    program_ID.wkbbxe =  (*ev)->GetFieldID(ev, programCls, "wkbbxe", "Lcom/wibu/WkJavaApi$WKBBOXENTRY;");
    program_ID.abSignature = (*ev)->GetFieldID(ev, programCls, "abSignature", "[B");
  
    programIsInit = TRUE;
  }

}


/***** WKBBOXLIST ******/
struct
{
  jfieldID flCtrl;
  jfieldID cBox;
  jfieldID usReserved;
  jfieldID awkbsr;
} boxlist_ID;

jclass boxListCls;
jmethodID newSerial_ID;
static BOOL boxListIsInit = FALSE;

static void InitBoxList(JNIEnv* ev)
{
  if(!boxListIsInit)
  {
    InitSerial(ev);
    boxListCls = (*ev)->NewGlobalRef(ev, (*ev)->FindClass(ev, "com/wibu/WkJavaApi$WKBBOXLIST"));
    newSerial_ID = (*ev)->GetMethodID(ev, serialCls,"<init>","()V");
    boxlist_ID.flCtrl = (*ev)->GetFieldID(ev, boxListCls, "flCtrl", "J");
    boxlist_ID.cBox = (*ev)->GetFieldID(ev, boxListCls, "cBox", "S");
    boxlist_ID.usReserved = (*ev)->GetFieldID(ev, boxListCls, "usReserved", "S");
    boxlist_ID.awkbsr = (*ev)->GetFieldID(ev, boxListCls, "awkbsr", "[Lcom/wibu/WkJavaApi$WKBSERIAL;");
    boxListIsInit = TRUE;	
  }
} // InitBoxList()


/***** WKBAPPINFO ******/
struct 
{ 
  jfieldID flCtrl;
  jfieldID pszAppName;
  jfieldID bVersionLow;
  jfieldID bVersionHigh;
  jfieldID fsAccessMask;
  jfieldID hwndMainWindow;
  jfieldID flMenuItemCtrl;
  jfieldID idLanguage;
  jfieldID usReserved1;
  jfieldID ulReserved2;
  jfieldID ulReserved3;
} appInfo_ID;



/***** WKBSYSTEM ******/
struct
{
  jfieldID flCtrl;
  jfieldID pszServerName;
  jfieldID idNetUser;
  jfieldID cMaxUser;
  jfieldID cFreeUser;
  jfieldID cbMaxData;
  jfieldID cRefreshError;
  jfieldID usReserved1;
} system_ID;

jclass systemCls;
static BOOL systemIsInit = FALSE;

static void InitSystem(JNIEnv* ev)
{
  if(!systemIsInit)
    {
      systemCls = (*ev)->NewGlobalRef(ev, (*ev)->FindClass(ev, "com/wibu/WkJavaApi$WKBSYSTEM"));
      system_ID.flCtrl = (*ev)->GetFieldID(ev, systemCls, "flCtrl", "J");
      system_ID.pszServerName = (*ev)->GetFieldID(ev, systemCls, "pszServerName", "Ljava/lang/StringBuffer;");
      system_ID.idNetUser = (*ev)->GetFieldID(ev, systemCls, "idNetUser", "S");
      system_ID.cMaxUser = (*ev)->GetFieldID(ev, systemCls, "cMaxUser", "S");
      system_ID.cFreeUser = (*ev)->GetFieldID(ev, systemCls, "cFreeUser", "S");
      system_ID.cbMaxData = (*ev)->GetFieldID(ev, systemCls, "cbMaxData", "S");
      system_ID.cRefreshError = (*ev)->GetFieldID(ev, systemCls, "cRefreshError", "S");
      system_ID.usReserved1 = (*ev)->GetFieldID(ev, systemCls, "usReserved1", "S");
      systemIsInit = TRUE;
  }
}

/***** WKBOPENENTRY ******/
struct
{
  jfieldID flCtrl;
  jfieldID hwkbsys;
  jfieldID lPort;
  jfieldID pszPort;
  jfieldID wkbsr;
  jfieldID usVersion;
  jfieldID usEntry;
  jfieldID ulFirmCode;
  jfieldID ulUserCode;
} openEntry_ID;

jclass openEntryCls;
static BOOL openEntryIsInit = FALSE;

static void InitOpenEntry(JNIEnv* ev)
{
  if(!openEntryIsInit)
  { 
    InitSerial(ev);
    openEntryCls            = (*ev)->NewGlobalRef(ev, (*ev)->FindClass(ev, "com/wibu/WkJavaApi$WKBOPENENTRY"));
    openEntry_ID.flCtrl     = (*ev)->GetFieldID(ev, openEntryCls, "flCtrl", "J");
    openEntry_ID.hwkbsys    = (*ev)->GetFieldID(ev, openEntryCls, "hwkbsys", "J");
    openEntry_ID.lPort      = (*ev)->GetFieldID(ev, openEntryCls, "lPort", "J");
    openEntry_ID.pszPort    = (*ev)->GetFieldID(ev, openEntryCls, "pszPort", "Ljava/lang/StringBuffer;");
    openEntry_ID.wkbsr      = (*ev)->GetFieldID(ev, openEntryCls,"wkbsr", "Lcom/wibu/WkJavaApi$WKBSERIAL;");
    openEntry_ID.usVersion  = (*ev)->GetFieldID(ev, openEntryCls, "usVersion", "S");
    openEntry_ID.usEntry    = (*ev)->GetFieldID(ev, openEntryCls, "usEntry", "S");
    openEntry_ID.ulFirmCode = (*ev)->GetFieldID(ev, openEntryCls, "ulFirmCode", "J");
    openEntry_ID.ulUserCode = (*ev)->GetFieldID(ev, openEntryCls, "ulUserCode", "J");
    openEntryIsInit = TRUE;
  }
} // InitOpenEntry()


/***** WKBAREA *****/
struct
{
  jfieldID flCtrl;
  jfieldID flSelCtrl;
  jfieldID ulSelectCode;
  jfieldID ulCrc;
  jfieldID ulReserved;
  jfieldID pbGapMap;
  jfieldID pvCtrl;
} area_ID;

jclass areaCls;
static BOOL areaIsInit = FALSE;

void InitArea(JNIEnv* ev)
{
  if(!areaIsInit) {
    areaCls               = (*ev)->NewGlobalRef(ev, (*ev)->FindClass(ev, "com/wibu/WkJavaApi$WKBAREA"));
    area_ID.flCtrl        = (*ev)->GetFieldID(ev, areaCls, "flCtrl", "J");
    area_ID.flSelCtrl     = (*ev)->GetFieldID(ev, areaCls, "flSelCtrl", "J");
    area_ID.ulSelectCode  = (*ev)->GetFieldID(ev, areaCls, "ulSelectCode", "J");
    area_ID.ulCrc         = (*ev)->GetFieldID(ev, areaCls, "ulCrc", "J");
    area_ID.ulReserved    = (*ev)->GetFieldID(ev, areaCls, "ulReserved", "J");
    area_ID.pbGapMap      = (*ev)->GetFieldID(ev, areaCls, "pbGapMap", "[B");
    area_ID.pvCtrl        = (*ev)->GetFieldID(ev, areaCls, "pvCtrl", "Lcom/wibu/WkJavaApi$WKBSELECT;");
    areaIsInit = TRUE;
  } // if
} // InitArea()

/***** WKBSELECT *****/
struct
{
  jfieldID flCtrl;
  jfieldID ulCacheId;
  jfieldID ulCrc;
  jfieldID cbInit;
  jfieldID abInit;
} select_ID;

jclass selectCls;
static BOOL selectIsInit = FALSE;

void InitSelect(JNIEnv* ev)
{
  if(!selectIsInit)
  {
    selectCls = (*ev)->NewGlobalRef(ev, (*ev)->FindClass(ev, "com/wibu/WkJavaApi$WKBSELECT"));
    select_ID.flCtrl = (*ev)->GetFieldID(ev, selectCls, "flCtrl", "J");
    select_ID.ulCacheId = (*ev)->GetFieldID(ev, selectCls, "ulCacheId", "J");
    select_ID.ulCrc = (*ev)->GetFieldID(ev, selectCls, "ulCrc", "J");
    select_ID.cbInit = (*ev)->GetFieldID(ev, selectCls, "cbInit", "I");
    select_ID.abInit = (*ev)->GetFieldID(ev, selectCls, "abInit", "[B");
    selectIsInit = TRUE;
  }
} // InitSelect()

/***** int* *****/
jfieldID pInt_ID;
static BOOL pIntIsInit = FALSE;
jclass pIntCls;

static void InitPInt(JNIEnv* ev)
{
  if(!pIntIsInit)
  {
    pIntCls = (*ev)->NewGlobalRef(ev, (*ev)->FindClass(ev,"com/wibu/WkJavaApi$pInt"));
    pInt_ID = (*ev)->GetFieldID(ev, pIntCls, "value", "I");
    pIntIsInit = TRUE;
  }
} // InitPInt()

/***** WKBIPADDR *****/
struct
{
  jfieldID  aus;
} ipAddr_ID;

jclass      ipAddrCls;
jmethodID   newIpAddr_ID;
static BOOL ipAddrIsInit = FALSE;

static void InitIpAddr(JNIEnv* ev)
{
  if(!ipAddrIsInit)
  {
    ipAddrCls     = (*ev)->NewGlobalRef(ev, (*ev)->FindClass(ev, "com/wibu/WkJavaApi$WKBIPADDR"));
    newIpAddr_ID  = (*ev)->GetMethodID(ev, ipAddrCls,"<init>","()V");

    ipAddr_ID.aus = (*ev)->GetFieldID(ev, ipAddrCls, "aus", "[S");

    ipAddrIsInit = TRUE;
  } // if
} // InitIpAddr()

/***** WKMSERVER *****/
struct
{
  jfieldID  ulCtrl;          
  jfieldID  pszName;        
  jfieldID  pszDescription; 
  jfieldID  wkbia;           
  jfieldID  idUpdate;        
  jfieldID  cAvailCluster;   
  jfieldID  cUsedCluster;    
} wkmServer_ID;

jclass      wkmServerCls;
jmethodID   newWkmServer_ID;
static BOOL wkmServerIsInit = FALSE;


/*****************************************************************************
InitWkmServer
==============================================================================

initialize the wkmServer_ID helper class

Parameters
   ev
    
*****************************************************************************/

static void InitWkmServer(JNIEnv* ev)
{
  if( ! wkmServerIsInit )
  {
    wkmServerCls    = (*ev)->NewGlobalRef(ev, (*ev)->FindClass(ev, "com/wibu/WkJavaApi$WKMSERVER"));
    newWkmServer_ID = (*ev)->GetMethodID(ev, wkmServerCls,"<init>","()V");

    wkmServer_ID.ulCtrl         = (*ev)->GetFieldID(ev, wkmServerCls, "ulCtrl", "J");
    wkmServer_ID.pszName        = (*ev)->GetFieldID(ev, wkmServerCls, "pszName", "Ljava/lang/StringBuffer;");
    wkmServer_ID.pszDescription = (*ev)->GetFieldID(ev, wkmServerCls, "pszDescription", "Ljava/lang/StringBuffer;");
    wkmServer_ID.wkbia          = (*ev)->GetFieldID(ev, wkmServerCls, "wkbia", "Lcom/wibu/WkJavaApi$WKBIPADDR;");
    wkmServer_ID.idUpdate       = (*ev)->GetFieldID(ev, wkmServerCls, "idUpdate", "J");
    wkmServer_ID.cAvailCluster  = (*ev)->GetFieldID(ev, wkmServerCls, "cAvailCluster", "S");
    wkmServer_ID.cUsedCluster   = (*ev)->GetFieldID(ev, wkmServerCls, "cUsedCluster", "S");

    wkmServerIsInit = TRUE;
  } // if
} // InitWkmServer()

/***** WKMCLUSTERMAP *****/
struct
{
  jfieldID  ulCtrl;
  jfieldID  idUpdate;
  jfieldID  abMap;
} clusterMap_ID;

jclass      clusterMapCls;
jmethodID   newclusterMap_ID;
static BOOL clusterMapIsInit = FALSE;

static void InitClusterMap(JNIEnv* ev)
{
  if(!clusterMapIsInit)
  {
    clusterMapCls     = (*ev)->NewGlobalRef(ev, (*ev)->FindClass(ev, "com/wibu/WkJavaApi$WKMCLUSTERMAP"));
    newclusterMap_ID  = (*ev)->GetMethodID(ev, clusterMapCls, "<init>", "()V");

    clusterMap_ID.ulCtrl    = (*ev)->GetFieldID(ev, clusterMapCls, "ulCtrl", "J");
    clusterMap_ID.idUpdate  = (*ev)->GetFieldID(ev, clusterMapCls, "idUpdate", "J");
    clusterMap_ID.abMap     = (*ev)->GetFieldID(ev, clusterMapCls, "abMap", "[B");

    clusterMapIsInit = TRUE;
  } // if
} // InitClusterMap()

/***** WKMCLUSTER *****/
struct
{
  jfieldID  ulCtrl;
  jfieldID  szName;
  jfieldID  ulFirmCode;
  jfieldID  ulUserCode;
  jfieldID  idCluster;
  jfieldID  cMaxUser;
  jfieldID  cFreeUser;
  jfieldID  cFreeUserWkNet;
  jfieldID  idUpdate;
  jfieldID  abMap;
} cluster_ID;

jclass      clusterCls;
jmethodID   newcluster_ID;
static BOOL clusterIsInit = FALSE;

static void InitCluster(JNIEnv* ev)
{
  if(!clusterIsInit)
  {
    clusterCls     = (*ev)->NewGlobalRef(ev, (*ev)->FindClass(ev, "com/wibu/WkJavaApi$WKMCLUSTER"));
    newcluster_ID  = (*ev)->GetMethodID(ev, clusterCls, "<init>", "()V");

    cluster_ID.ulCtrl         = (*ev)->GetFieldID(ev, clusterCls, "ulCtrl", "J");
    cluster_ID.szName         = (*ev)->GetFieldID(ev, clusterCls, "szName", "J");
    cluster_ID.ulFirmCode     = (*ev)->GetFieldID(ev, clusterCls, "ulFirmCode", "J");
    cluster_ID.ulUserCode     = (*ev)->GetFieldID(ev, clusterCls, "ulUserCode", "J");
    cluster_ID.idCluster      = (*ev)->GetFieldID(ev, clusterCls, "idCluster", "J");
    cluster_ID.cMaxUser       = (*ev)->GetFieldID(ev, clusterCls, "cMaxUser", "S");
    cluster_ID.cFreeUser      = (*ev)->GetFieldID(ev, clusterCls, "cFreeUser", "S");
    cluster_ID.cFreeUserWkNet = (*ev)->GetFieldID(ev, clusterCls, "cFreeUserWkNet", "S");

    clusterIsInit = TRUE;
  } // if
} // InitClusterMap()
/***** WKMCLUSTERLIST *****/
struct
{
  jfieldID  ulCtrl;
  jfieldID  idUpdate;
  jfieldID  cEntry;
  jfieldID  cSubsequent;
  jfieldID  awkmclu;
} clusterList_ID;

jclass      clusterListCls;
jmethodID   newclusterList_ID;
static BOOL clusterListIsInit = FALSE;

static void InitClusterList(JNIEnv* ev)
{
  if(!clusterListIsInit)
  {
    clusterListCls     = (*ev)->NewGlobalRef(ev, (*ev)->FindClass(ev, "com/wibu/WkJavaApi$WKMCLUSTERLIST"));
    newclusterList_ID  = (*ev)->GetMethodID(ev, clusterListCls, "<init>", "()V");

    clusterList_ID.ulCtrl       = (*ev)->GetFieldID(ev, clusterListCls, "ulCtrl", "J");
    clusterList_ID.idUpdate     = (*ev)->GetFieldID(ev, clusterListCls, "idUpdate", "J");
    clusterList_ID.cEntry       = (*ev)->GetFieldID(ev, clusterListCls, "cEntry", "S");
    clusterList_ID.cSubsequent  = (*ev)->GetFieldID(ev, clusterListCls, "cSubsequent", "S");
    clusterList_ID.awkmclu      = (*ev)->GetFieldID(ev, clusterListCls, "awkmclu", "[Lcom/wibu/WkJavaApi$WKMCLUSTER;");

    clusterListIsInit = TRUE;
  } // if
} // InitClusterList()

/*****************************************************************************
J2C_WKBSERIAL
==============================================================================

mapping of WKBSERIAL structure Java to C

Parameters
  *ev
  serial
  *wkbsr

Return
  static void 
  
*****************************************************************************/

void J2C_WKBSERIAL(JNIEnv *ev, jobject serial, WKBSERIAL *wkbsr)
{
  InitSerial(ev);
  memset(wkbsr, 0, sizeof(WKBSERIAL));

  wkbsr->ulSerial = (ULONG) (*ev)->GetLongField(ev, serial, serial_ID.ulSerial);
  wkbsr->usMask = (*ev)->GetShortField(ev, serial, serial_ID.usMask);
  wkbsr->usReserved = (*ev)->GetShortField(ev, serial, serial_ID.usReserved);
} // J2C_WKBSERIAL()

/*****************************************************************************
J2C_WKBOPENENTRY
==============================================================================

mapping of a WKBOPENENTRY structure from Java to C

Parameters
  *ev
  jobj
  *pwkboe

Return
  - 
  
*****************************************************************************/

static void J2C_WKBOPENENTRY(JNIEnv *ev, jobject jobj, WKBOPENENTRY *pwkboe)
{
  jobject   serial;

  InitOpenEntry(ev);
  memset(pwkboe, 0, sizeof(WKBOPENENTRY));

  pwkboe->flCtrl  = (ULONG) (*ev)->GetLongField(ev, jobj, openEntry_ID.flCtrl);
  pwkboe->hwkbsys = (HWKBSYSTEM) (*ev)->GetLongField(ev, jobj, openEntry_ID.hwkbsys);
  pwkboe->lPort = (LONG) (*ev)->GetLongField(ev, jobj, openEntry_ID.lPort);
  pwkboe->pszPort = 0;

  //WKBSERIAL java -> c
  serial = (*ev)->GetObjectField(ev, jobj, openEntry_ID.wkbsr);
  J2C_WKBSERIAL(ev, serial, &pwkboe->wkbsr);

  pwkboe->usVersion = (unsigned short) (*ev)->GetShortField(ev, jobj, openEntry_ID.usVersion);
  pwkboe->usEntry   = (unsigned short) (*ev)->GetShortField(ev, jobj, openEntry_ID.usEntry);
  pwkboe->ulFirmCode = (ULONG) (*ev)->GetLongField(ev, jobj, openEntry_ID.ulFirmCode);
  pwkboe->ulUserCode = (ULONG) (*ev)->GetLongField(ev, jobj, openEntry_ID.ulUserCode);
  DUMP_WKBOPENENTRY(*pwkboe, "---");
} // J2C_WKBOPENENTRY()

jclass extMemCls;
static BOOL extMemIsInit = FALSE;

/***** WKBEXTMEM ******/
struct
{
  jfieldID fsCtrl;
  jfieldID iPage;
  jfieldID abSignature;
  jfieldID abData;
} extMem_ID;

static void InitExtMem(JNIEnv *ev)
{
  if(!extMemIsInit) { 
    extMemCls             = (*ev)->NewGlobalRef(ev, (*ev)->FindClass(ev, "com/wibu/WkJavaApi$WKBEXTMEM"));
    extMem_ID.fsCtrl      = (*ev)->GetFieldID(ev, extMemCls, "fsCtrl", "S");
    extMem_ID.iPage       = (*ev)->GetFieldID(ev, extMemCls, "iPage", "S");
    extMem_ID.abSignature = (*ev)->GetFieldID(ev, extMemCls, "abSignature", "[B");
    extMem_ID.abData      = (*ev)->GetFieldID(ev, extMemCls, "abData", "[B");
    extMemIsInit = TRUE;
  } // if
} // InitExtMem()


/*****************************************************************************
J2C_WKBEXTMEM
==============================================================================

mapping of WKBEXTMEM structure from Java to C

Parameters
  ev
  jobj
  pkwbem

Return
  static void 
  
*****************************************************************************/

static void J2C_WKBEXTMEM(JNIEnv *ev, jobject jobj, WKBEXTMEM *pkwbem)
{
  jbyteArray  jbSignature;
  jbyteArray  jbData;

  // init class constants
  InitExtMem(ev);

  // init destination
  memset(pkwbem, 0, sizeof(WKBEXTMEM));

  pkwbem->fsCtrl   = (unsigned short) (*ev)->GetShortField(ev, jobj, extMem_ID.fsCtrl);
  pkwbem->iPage   = (unsigned short) (*ev)->GetShortField(ev, jobj, extMem_ID.iPage);
    
  jbSignature = (*ev)->GetObjectField(ev, jobj, extMem_ID.abSignature);
  jbData      = (*ev)->GetObjectField(ev, jobj, extMem_ID.abData);
  J2C_ByteArray(ev, jbSignature, pkwbem->abSignature, (LONG)sizeof(pkwbem->abSignature));
  J2C_ByteArray(ev, jbData, pkwbem->abData, (LONG)sizeof(pkwbem->abData));
} // J2C_WKBEXTMEM()

static void C2J_WKBEXTMEM(JNIEnv *ev, WKBEXTMEM c_extMem, jobject wkbextmem)
{
  jbyteArray    jbSignature;
  jbyteArray    jbData;

  InitExtMem(ev);
  (*ev)->SetShortField(ev, wkbextmem, extMem_ID.fsCtrl, c_extMem.fsCtrl);
  (*ev)->SetShortField(ev, wkbextmem, extMem_ID.iPage, c_extMem.iPage);

  jbSignature = (*ev)->GetObjectField(ev, wkbextmem, extMem_ID.abSignature);
  jbData      = (*ev)->GetObjectField(ev, wkbextmem, extMem_ID.abData);

  (*ev)->SetByteArrayRegion(ev, jbSignature, 0, sizeof(c_extMem.abSignature), c_extMem.abSignature);
  (*ev)->SetByteArrayRegion(ev, jbData, 0, sizeof(c_extMem.abData), c_extMem.abData);

} // C2J_WKBEXTMEM()

// *******************************************************************************************
// WIBU Functions JAVA - ADAPTION
// *******************************************************************************************


// *******************************************************************************************
// Subsystem (Network) Management
// *******************************************************************************************

/***** WkbAccess2 *****/
DLL_EXPORT jlong JNICALL 
Java_com_wibu_WkJavaApi_WkbAccess2(JNIEnv *ev, jclass cls1, jlong v1, jobject accessEntry)
{
  char errMessage[30] = "WkbAccess2";
  int retValue = 0;
  WKBACCESS c_vCtrl;
  InitAccess(ev);
  
  // init WKBACCESS data
  memset(&c_vCtrl, 0, sizeof(c_vCtrl));

  c_vCtrl.pszAppName = (char *) malloc(256);
  c_vCtrl.pszServerName = (char *) malloc(256);
  
 
  //WKBACCESS: java -> c
  c_vCtrl.flCtrl    = (ULONG) (*ev)->GetLongField(ev, accessEntry, access_ID.flCtrl);
  c_vCtrl.lFirmCode = (LONG) (*ev)->GetLongField(ev, accessEntry, access_ID.lFirmCode);
  c_vCtrl.lUserCode = (LONG) (*ev)->GetLongField(ev, accessEntry, access_ID.lUserCode);
  c_vCtrl.flStdCtrl = (ULONG) (*ev)->GetLongField(ev, accessEntry, access_ID.flStdCtrl);
  c_vCtrl.bVersionLow = (unsigned char) (*ev)->GetByteField(ev, accessEntry, access_ID.bVersionLow);
  c_vCtrl.bVersionHigh = (unsigned char) (*ev)->GetByteField(ev, accessEntry, access_ID.bVersionHigh);
  c_vCtrl.idNetUser = (unsigned short)(*ev)->GetIntField(ev, accessEntry, access_ID.idNetUser);
   
  if (Create_C_Str_From_Java(ev,(*ev)->GetObjectField(ev, accessEntry, access_ID.pszAppName), 
        (char *) c_vCtrl.pszAppName, errMessage) == 0)
  {
    c_vCtrl.pszAppName = NULL;
  }

  if (Create_C_Str_From_Java(ev,(*ev)->GetObjectField(ev, accessEntry, access_ID.pszServerName), 
        (char *) c_vCtrl.pszServerName, errMessage) == 0)
  {
    c_vCtrl.pszServerName = NULL;
  }
  
  // check 'c_vCtrl.flCtrl' parameter
  if ( c_vCtrl.flCtrl & WKB_USEOFFSETS ) {
    // not valid in any case:
    c_vCtrl.flCtrl &= ~WKB_USEOFFSETS;
  } // if

  DUMP_WKBACCESS(c_vCtrl);
  // call WIBU function
  retValue = (int)WkbAccess2((ULONG)v1, &c_vCtrl);
  DUMP_WKBACCESS(c_vCtrl);

  free((char *) c_vCtrl.pszAppName);
  free((char *) c_vCtrl.pszServerName);

  return (jlong)retValue;
}


/***** WkbRelease2 *****/
DLL_EXPORT jboolean JNICALL Java_com_wibu_WkJavaApi_WkbRelease2
   (JNIEnv *ev, jclass cls, jlong v1)
{
  return (jboolean)WkbRelease2((HWKBSYSTEM)v1);
}


/***** WkbQuerySystem2 *****/
// commands WKB_QSY_SYSTEM
DLL_EXPORT jint JNICALL Java_com_wibu_WkJavaApi_WkbQuerySystem2__JJLcom_wibu_WkJavaApi_00024WKBSYSTEM_2I
  (JNIEnv *ev, jclass cls, jlong hwkbsys, jlong flCtrl, jobject pvDest, jint cbDest)
{
  //!
  // *******************************************************************************************************
  // Die Struktur von WKBSYSTEM ist zu klein !!
  // *******************************************************************************************************
  typedef struct 
  {
    WKBSYSTEM wkbsys;
    char pszServerName[30];
  }WKBTHISOE;

  jint retValue = 0;
  char errMessage[30] = "WkbQuerySystem2";

  WKBTHISOE c_wkbsystem;

  InitSystem(ev);

  // call WIBU function
  retValue = (jint)WkbQuerySystem2((HWKBSYSTEM)hwkbsys, (ULONG)flCtrl, &c_wkbsystem, sizeof(c_wkbsystem));		

  // return immediately on error
  if ( retValue == 0 ) {
    return retValue;
  }

  // c -> java
  (*ev)->SetLongField(ev, pvDest, system_ID.flCtrl, c_wkbsystem.wkbsys.flCtrl);

  C2J_AppendToStringBuffer(ev, 
    (*ev)->GetObjectField(ev,pvDest, system_ID.pszServerName), c_wkbsystem.wkbsys.pszServerName, errMessage);

  (*ev)->SetShortField(ev, pvDest, system_ID.idNetUser, c_wkbsystem.wkbsys.idNetUser);
  (*ev)->SetShortField(ev, pvDest, system_ID.cMaxUser, c_wkbsystem.wkbsys.cMaxUser);
  (*ev)->SetShortField(ev, pvDest, system_ID.cFreeUser, c_wkbsystem.wkbsys.cFreeUser);
  (*ev)->SetShortField(ev, pvDest, system_ID.cbMaxData, c_wkbsystem.wkbsys.cbMaxData);
  (*ev)->SetShortField(ev, pvDest, system_ID.cRefreshError, c_wkbsystem.wkbsys.cRefreshError);
  (*ev)->SetShortField(ev, pvDest, system_ID.usReserved1, c_wkbsystem.wkbsys.usReserved1);

  return retValue;
} // WkbQuerySystem2(WKB_QSY_SYSTEM)


/***** WkbQuerySystem2 *****/
// commands WKB_QSY_OPENENTRY
DLL_EXPORT jint JNICALL Java_com_wibu_WkJavaApi_WkbQuerySystem2__JJLcom_wibu_WkJavaApi_00024WKBOPENENTRY_2I
  (JNIEnv *ev, jclass cls, jlong hwkbsys, jlong flCtrl, jobject pvDest, jint cbDest)
{
  // ******************************************************************************************************
  // STRUCTURE WKBOPENENTRY zu klein!
  // ******************************************************************************************************
  typedef struct 
  {
    WKBOPENENTRY wkboe;
    char pszPort[30];
  } WKBTHISOE;

  // declaration
  jclass openEntryCls; 
  jobject serial; 
  jclass serialCls;
  int retValue = 0;

  WKBTHISOE c_pvDest;
  char errMessage[30] = "WkbQuerySystem2";

  // init destination
  memset(&c_pvDest, 0, sizeof(c_pvDest));

  // call WIBU-function
  retValue = WkbQuerySystem2((HWKBSYSTEM)hwkbsys, (ULONG)flCtrl, &c_pvDest, sizeof(c_pvDest));
  // return immediately on error
  if ( retValue == 0 ) {
    return retValue;
  }

  // c -> java
  // ******************************************************************************************************
  // MAPPING WKBOPENENTRY STRUCTURE
  // ******************************************************************************************************

  //WKBOPENENTRY: c -> java
  openEntryCls = (*ev)->GetObjectClass(ev, pvDest);

  serial = (*ev)->GetObjectField(ev, pvDest, (*ev)->GetFieldID(ev, openEntryCls,"wkbsr", "Lcom/wibu/WkJavaApi$WKBSERIAL;")); 
  serialCls = (*ev)->FindClass(ev, "com/wibu/WkJavaApi$WKBSERIAL");


  (*ev)->SetLongField(ev, pvDest, (*ev)->GetFieldID(ev, openEntryCls, "flCtrl", "J"), c_pvDest.wkboe.flCtrl);
  (*ev)->SetLongField(ev, pvDest, (*ev)->GetFieldID(ev, openEntryCls, "hwkbsys", "J"), (jlong)c_pvDest.wkboe.hwkbsys);
  (*ev)->SetLongField(ev, pvDest, (*ev)->GetFieldID(ev, openEntryCls, "lPort", "J"), c_pvDest.wkboe.lPort);

  C2J_AppendToStringBuffer(ev, (*ev)->GetObjectField(ev,pvDest, 
    (*ev)->GetFieldID(ev, openEntryCls, "pszPort", "Ljava/lang/StringBuffer;")),
    c_pvDest.wkboe.pszPort, errMessage);
  
  // ********************************************************************************************************
  // MAPPING WKBSERIAL STRUCTURE
  // ********************************************************************************************************
  (*ev)->SetLongField(ev, serial, (*ev)->GetFieldID(ev, serialCls, "ulSerial", "J"), c_pvDest.wkboe.wkbsr.ulSerial);
  (*ev)->SetShortField(ev, serial, (*ev)->GetFieldID(ev, serialCls, "usMask", "S"), c_pvDest.wkboe.wkbsr.usMask);
  (*ev)->SetShortField(ev, serial, (*ev)->GetFieldID(ev, serialCls, "usReserved", "S"), c_pvDest.wkboe.wkbsr.usReserved);
  // *********************************************************************************************************
  (*ev)->SetShortField(ev, pvDest, (*ev)->GetFieldID(ev, openEntryCls, "usVersion", "S"), c_pvDest.wkboe.usVersion);
  (*ev)->SetShortField(ev, pvDest, (*ev)->GetFieldID(ev, openEntryCls, "usEntry", "S"), c_pvDest.wkboe.usEntry);
  (*ev)->SetLongField(ev, pvDest, (*ev)->GetFieldID(ev, openEntryCls, "ulFirmCode", "J"), c_pvDest.wkboe.ulFirmCode);
  (*ev)->SetLongField(ev, pvDest, (*ev)->GetFieldID(ev, openEntryCls, "ulUserCode", "J"), c_pvDest.wkboe.ulUserCode);

  return retValue;
}


/***** WkbQuerySystem2 *****/
// commands WKB_QSY_IPADDR
DLL_EXPORT jint JNICALL Java_com_wibu_WkJavaApi_WkbQuerySystem2__JJLjava_lang_Object_2I
  (JNIEnv *ev, jclass cls, jlong hwkbsys, jlong flCtrl, jobject pvDest, jint cbDest)
{	
  char c_charBuf[256];
  char errMessage[30] = "WkbQuerySystem2";
  int retValue = 0;

  Create_C_Str_From_Java(ev, pvDest, c_charBuf, errMessage);
  Delete_Str_In_Buffer(ev, pvDest, errMessage);

  // call WIBU function
  retValue = WkbQuerySystem2((HWKBSYSTEM)hwkbsys, (ULONG)flCtrl, c_charBuf, sizeof(c_charBuf));
  // return immediately on error
  if ( retValue == 0 ) {
    return retValue;
  } // if

  // c -> java
  C2J_AppendToStringBuffer(ev, pvDest, c_charBuf, errMessage);

  return retValue;
} // WkbQuerySystem2(WKB_QSY_IPADDR)


/***** WkbEnumServers2 *****/
DLL_EXPORT jint JNICALL Java_com_wibu_WkJavaApi_WkbEnumServers2
  (JNIEnv *ev, jclass cls, jlong flCtrl, jobject pszDest, jint cchDest)
{
  //!
  char c_charBuf[256];
  char errMessage[30] = "WkbEnumServers2";
  int retValue = 0;

  Create_C_Str_From_Java(ev, pszDest, c_charBuf, errMessage);
  Delete_Str_In_Buffer(ev,pszDest, errMessage);

  // call WIBU function
  retValue = WkbEnumServers2((ULONG)flCtrl, c_charBuf, 256);
  // return immediately on error
  if ( retValue == 0 ) {
    return retValue;
  }
  // c -> java
  C2J_AppendToStringBuffer(ev, pszDest, c_charBuf, errMessage);

  return retValue;
}


/***** WkbApplicationInfo2 *****/
DLL_EXPORT jint JNICALL Java_com_wibu_WkJavaApi_WkbApplicationInfo2
  (JNIEnv *ev, jclass cls, jlong hwkbsys, jlong flCtrl, jobject pvInfo, jint cbInfo)
{
  WkbSetLastError(WKB_ERROR_INVALID_FUNCTION);
  return 0;
} // Java_com_wibu_WkJavaApi_WkbApplicationInfo2()


/***** WkbOpen2 *****/
DLL_EXPORT jlong JNICALL Java_com_wibu_WkJavaApi_WkbOpen2
  (JNIEnv *ev, jclass cls, jlong v1, jlong v2, jstring pszPort, jlong v4, jlong v5, jobject v6)
{
  void      *pvCtrl  = 0;
  char      *c_pszPort;
  ULONG     flCtrl    = (ULONG) v2,
            lFirmCode = (ULONG) v4,
            lUserCode = (ULONG) v5;
  HWKBENTRY ret;
  WKBOPENENTRY  wkboeEntry;

  // check if v6 is a WKBOPENENTRY structure
  if ( v2 & WKB_OPEN_ENTRY ) {
    if ( v6 == 0 ) {
      return WKB_ERROR_INVALID_PARAMETER;
    } // if
    J2C_WKBOPENENTRY(ev, v6, &wkboeEntry);
    pvCtrl = &wkboeEntry;
  } // if

  if ( pszPort == NULL ) {
    c_pszPort = NULL;
  } else {
    c_pszPort = (char *)(*ev)->GetStringUTFChars(ev, pszPort, NULL);
  } // if

  // call WIBU function
  ret = WkbOpen2((HWKBSYSTEM)v1, flCtrl, c_pszPort, lFirmCode, lUserCode, pvCtrl);

  if ( pszPort != NULL ) {
    (*ev)->ReleaseStringUTFChars(ev, pszPort, c_pszPort);
  }
  return (jlong)ret;
} // Java_com_wibu_WkJavaApi_WkbOpen2()

/***** WkbSelect2 *****/
DLL_EXPORT jint JNICALL Java_com_wibu_WkJavaApi_WkbSelect2 
    (JNIEnv *ev, jclass cls, jlong hwkbe, jlong flCtrl, jlong ulSelectCode, jobject pvCtrl)
{
  // var declaration
  WKBSELECT c_wkbsel;
  jbyteArray abInit;

  int retValue = 0;

  InitSelect(ev);

  if ( (((ULONG)flCtrl & WKB_SEL_KNUTH20) != 0) ||
    (((ULONG)flCtrl & WKB_SEL_DEC_FEAL) != 0) ||
    (((ULONG)flCtrl & WKB_SEL_ENC_FEAL) != 0) ||
    (((ULONG)flCtrl & WKB_SEL_DEC_PERM) != 0) ||
    (((ULONG)flCtrl & WKB_SEL_ENC_PERM) != 0) ||
    (((ULONG)flCtrl & WKB_SEL_SYMFEAL) != 0)  )
  {	 
    //WKBSELECT java -> c
    c_wkbsel.flCtrl = (ULONG)(*ev)->GetLongField(ev, pvCtrl, select_ID.flCtrl);
    c_wkbsel.ulCacheId = (ULONG)(*ev)->GetLongField(ev, pvCtrl, select_ID.ulCacheId);
    c_wkbsel.ulCrc = (ULONG)(*ev)->GetLongField(ev, pvCtrl, select_ID.ulCrc);
    c_wkbsel.cbInit = (USHORT)(*ev)->GetShortField(ev, pvCtrl, select_ID.cbInit);
    abInit = (*ev)->GetObjectField(ev, pvCtrl, select_ID.abInit);
    (*ev)->GetByteArrayRegion(ev, abInit, 0, (*ev)->GetArrayLength(ev, abInit), c_wkbsel.abInit);

    retValue = WkbSelect2((HWKBENTRY)hwkbe, (ULONG)flCtrl, (ULONG)ulSelectCode, &c_wkbsel);

    //WKBSELECT c -> java
    (*ev)->SetLongField(ev, pvCtrl, select_ID.flCtrl, c_wkbsel.flCtrl);
    (*ev)->SetLongField(ev, pvCtrl, select_ID.ulCacheId, c_wkbsel.ulCacheId);
    (*ev)->SetLongField(ev, pvCtrl, select_ID.ulCrc, c_wkbsel.ulCrc);
    (*ev)->SetIntField(ev, pvCtrl, select_ID.cbInit, c_wkbsel.cbInit);

    (*ev)->SetByteArrayRegion(ev, (*ev)->GetObjectField(ev, pvCtrl, select_ID.abInit), 0, 
      c_wkbsel.cbInit, c_wkbsel.abInit);
  } else {
    retValue = WkbSelect2((HWKBENTRY)hwkbe, (ULONG)flCtrl, (ULONG)ulSelectCode, NULL);
  } // if

  return retValue;
}


/***** WkbUnSelect2 *****/
DLL_EXPORT jint JNICALL Java_com_wibu_WkJavaApi_WkbUnSelect2(JNIEnv *ev, jclass cls, jlong hwkbe)
{
  return WkbUnSelect2((HWKBENTRY) hwkbe);
}


/***** WkbQueryEntry2 (WKB_QE_OPEN mode) *****/
DLL_EXPORT jlong JNICALL Java_com_wibu_WkJavaApi_WkbQueryEntry2__JJLcom_wibu_WkJavaApi_00024WKBOPENENTRY_2
  (JNIEnv *ev, jclass cls, jlong v1, jlong v2, jobject openEntry)
{
  // *******************************************************************************************************
  // Die Struktur von WKBOPENENTRY ist zu klein !!
  // *******************************************************************************************************
  typedef struct 
  {
    WKBOPENENTRY wkboe;
    char szPort[30];
  } WKBTHISOE;

  // vars
  char      errMessage[30] = "WkbQueryEntry2";
  int       ret;
  jclass    openEntryCls; 
  jobject   serial,
            jpszPort;
  jclass    serialCls;
  WKBTHISOE c_openEntry;
    
  ret = WkbQueryEntry2((HWKBENTRY)v1, (ULONG)v2|WKB_QE_OPEN, &c_openEntry, sizeof(c_openEntry));
  // return immediately on error

  DUMP_WKBOPENENTRY(c_openEntry.wkboe, c_openEntry.szPort);
  if ( ret == 0 ) {
    return ret;
  }
  
  // ******************************************************************************************************
  // MAPPING WKBOPENENTRY STRUCTURE
  // ******************************************************************************************************
  
  //WKBOPENENTRY: c -> java
  openEntryCls = (*ev)->GetObjectClass(ev, openEntry);
  
  serial = (*ev)->GetObjectField(ev, openEntry, (*ev)->GetFieldID(ev, openEntryCls,"wkbsr", "Lcom/wibu/WkJavaApi$WKBSERIAL;")); 
  serialCls = (*ev)->FindClass(ev, "com/wibu/WkJavaApi$WKBSERIAL");
  
  
  (*ev)->SetLongField(ev, openEntry, (*ev)->GetFieldID(ev, openEntryCls, "flCtrl", "J"), c_openEntry.wkboe.flCtrl);
  (*ev)->SetLongField(ev, openEntry, (*ev)->GetFieldID(ev, openEntryCls, "hwkbsys", "J"), (jlong)c_openEntry.wkboe.hwkbsys);
  (*ev)->SetLongField(ev, openEntry, (*ev)->GetFieldID(ev, openEntryCls, "lPort", "J"), c_openEntry.wkboe.lPort);
  
  // set pszPort: delete old value and 'append' new one
  jpszPort = (*ev)->GetObjectField(ev,openEntry, 
    (*ev)->GetFieldID(ev, openEntryCls, "pszPort", "Ljava/lang/StringBuffer;"));
  Delete_Str_In_Buffer(ev, jpszPort, errMessage);
  C2J_AppendToStringBuffer(ev, jpszPort, c_openEntry.wkboe.pszPort, errMessage);


  // ********************************************************************************************************
  // MAPPING WKBSERIAL STRUCTURE
  // ********************************************************************************************************
  
  (*ev)->SetLongField(ev, serial, (*ev)->GetFieldID(ev, serialCls, "ulSerial", "J"), c_openEntry.wkboe.wkbsr.ulSerial);
  (*ev)->SetShortField(ev, serial, (*ev)->GetFieldID(ev, serialCls, "usMask", "S"), c_openEntry.wkboe.wkbsr.usMask);
  (*ev)->SetShortField(ev, serial, (*ev)->GetFieldID(ev, serialCls, "usReserved", "S"), c_openEntry.wkboe.wkbsr.usReserved);
  
  // *********************************************************************************************************
  
  (*ev)->SetShortField(ev, openEntry, (*ev)->GetFieldID(ev, openEntryCls, "usVersion", "S"), c_openEntry.wkboe.usVersion);
  (*ev)->SetShortField(ev, openEntry, (*ev)->GetFieldID(ev, openEntryCls, "usEntry", "S"), c_openEntry.wkboe.usEntry);
  (*ev)->SetLongField(ev, openEntry, (*ev)->GetFieldID(ev, openEntryCls, "ulFirmCode", "J"), c_openEntry.wkboe.ulFirmCode);
  (*ev)->SetLongField(ev, openEntry, (*ev)->GetFieldID(ev, openEntryCls, "ulUserCode", "J"), c_openEntry.wkboe.ulUserCode);
 
  return (jlong)ret;
} // WkbQueryEntry2 (WKB_QE_OPEN mode)


/***** WkbQueryEntry2 (WKB_QE_BOX mode) *****/
DLL_EXPORT jlong JNICALL Java_com_wibu_WkJavaApi_WkbQueryEntry2__JJLcom_wibu_WkJavaApi_00024WKBBOXENTRY_2
  (JNIEnv *ev, jclass cls, jlong v1, jlong v2, jobject boxEntry)
{
  WKBBOXENTRY c_boxEntry;
  int ret;

  InitBoxEntry(ev);

  // call WIBU function
  ret = WkbQueryEntry2((HWKBENTRY)v1, (ULONG)v2|WKB_QE_BOX, &c_boxEntry, sizeof(c_boxEntry));
  // return immediately on error
  if ( ret == 0 ) {
    return ret;
  }

  //WKBBOXENTRY c -> java
  (*ev)->SetLongField(ev, boxEntry, boxEntry_ID.flCtrl, c_boxEntry.flCtrl);
  (*ev)->SetLongField(ev, boxEntry, boxEntry_ID.ulFirmCode, c_boxEntry.ulFirmCode);
  (*ev)->SetLongField(ev, boxEntry, boxEntry_ID.ulUserCode, c_boxEntry.ulUserCode);
  (*ev)->SetLongField(ev, boxEntry, boxEntry_ID.ulData1, c_boxEntry.ulData1);
  (*ev)->SetLongField(ev, boxEntry, boxEntry_ID.ulData2, c_boxEntry.ulData2);

  return (jlong)ret;
} // WkbQueryEntry2 (WKB_QE_BOX mode)

/***** WkbQueryEntry2 (WKB_QE_MULTIENTRY  mode) *****/
DLL_EXPORT jlong JNICALL Java_com_wibu_WkJavaApi_WkbQueryEntry2__JJLcom_wibu_WkJavaApi_00024WKBBOXDATA_2
  (JNIEnv *ev, jclass cls, jlong v1, jlong v2, jobject boxData)
{
  WKBBOXDATA 		c_boxData;
  int 					ret,
  							awkbbxe_len,
  							i;
  jobjectArray 	awkbbxe;
  jobject 			j_boxEntry;

  InitBoxData(ev);

  // call WIBU function
  ret = WkbQueryEntry2((HWKBENTRY)v1, (ULONG)v2|WKB_QE_MULTIENTRY, &c_boxData, sizeof(c_boxData));
  // return immediately on error
  if ( ret == 0 ) {
    return ret;
  }
  
  if ( ret > sizeof(c_boxData) ) {
  	WkbSetLastError(WKB_ERROR_MEMORY_TOO_SMALL);
  	return 0;
 	} // 

   //WKBBOXDATA c -> java
  (*ev)->SetLongField(ev, boxData, boxData_ID.flCtrl, c_boxData.flCtrl);
  (*ev)->SetShortField(ev, boxData, boxData_ID.usVersion, c_boxData.usVersion);
  (*ev)->SetShortField(ev, boxData, boxData_ID.cEntry, c_boxData.cEntry);
  // BOXENTRY-array awkbbxe: make new if necessary
  awkbbxe = (*ev)->GetObjectField(ev, boxData, boxData_ID.awkbbxe);
  if(awkbbxe == NULL ||  (*ev)->GetArrayLength(ev, awkbbxe) < 10)
  {
    awkbbxe = (*ev)->NewObjectArray(ev, 10, boxEntryCls, NULL);
  }
  awkbbxe_len = (*ev)->GetArrayLength(ev, awkbbxe);
  for (i = 0; i < c_boxData.cEntry; i++)  {
    j_boxEntry = (*ev)->GetObjectArrayElement(ev, awkbbxe, i);
    if(j_boxEntry == NULL) {
      j_boxEntry = (*ev)->NewObject(ev, boxEntryCls, newBoxEntry_ID);
      (*ev)->SetObjectArrayElement(ev, awkbbxe, i, j_boxEntry);
    }
    (*ev)->SetLongField(ev, j_boxEntry, boxEntry_ID.flCtrl, c_boxData.awkbbxe[i].flCtrl);
    (*ev)->SetLongField(ev, j_boxEntry, boxEntry_ID.ulFirmCode, c_boxData.awkbbxe[i].ulFirmCode);
    (*ev)->SetLongField(ev, j_boxEntry, boxEntry_ID.ulUserCode, c_boxData.awkbbxe[i].ulUserCode);
    (*ev)->SetLongField(ev, j_boxEntry, boxEntry_ID.ulData1, c_boxData.awkbbxe[i].ulData1);
    (*ev)->SetLongField(ev, j_boxEntry, boxEntry_ID.ulData2, c_boxData.awkbbxe[i].ulData2);      
  } // for

  return (jlong)ret;
} // WkbQueryEntry2 (WKB_QE_MULTIENTRY  mode)

//*******************************************************************************************
// List Port and WIBU-BOX Contents
//*******************************************************************************************

/***** WkbEnumPorts2 *****/
DLL_EXPORT jint JNICALL Java_com_wibu_WkJavaApi_WkbEnumPorts2
  (JNIEnv *ev, jclass cls, jlong hwkbsys, jlong flCtrl, jobject pszDest, jint cchDest)
{
  char c_charBuf[256];
  char errMessage[30] = "WkbEnumPorts2";
  int retValue = 0;

  Create_C_Str_From_Java(ev, pszDest, c_charBuf, errMessage);
  Delete_Str_In_Buffer(ev,pszDest, errMessage);

  // call WIBU function
  retValue = WkbEnumPorts2((HWKBSYSTEM)hwkbsys, (ULONG)flCtrl, c_charBuf, 256);
  // return immediately on error
  if ( retValue == 0 ) {
    return retValue;
  }
  // c -> java
  C2J_AppendToStringBuffer(ev, pszDest, c_charBuf, errMessage);

  return retValue;	
}


/***** WkbListPort2 *****/
DLL_EXPORT jint JNICALL Java_com_wibu_WkJavaApi_WkbListPort2
  (JNIEnv *ev, jclass cls, jlong hwkbsys, jlong flCtrl, jstring pszPort, jobject pvDest, jint cbDest)
{
  typedef struct {
    // room for 10 serial number entries
    WKBBOXLIST wkbbxl;    // first serial number
    WKBSERIAL  awkbxe[9]; // nine other serial numbers
  } PORTLIST;
  PORTLIST prtlist;
  jobjectArray awkbsr;
  jobject j_boxEntry;
  int awkbser_len;
  char* port;
  int i;
  int retValue = 0;
  
  // init port list
  memset(&prtlist, 0, sizeof(prtlist));

  // limitation
  flCtrl = flCtrl & (1^WKB_PORTNUMBER); 
  
  // init boxlist structure
  InitBoxList(ev);
  
  // mapping pszPort
  if ( pszPort == NULL ) {
    port = NULL;
  } else {
    port = (char *)(*ev)->GetStringUTFChars(ev,pszPort, NULL);
  } // if/else
  
  
  // call WIBU function
  retValue = WkbListPort2((HWKBSYSTEM)hwkbsys, (ULONG)flCtrl, port, &prtlist, sizeof(prtlist));
  // return immediately on error
  if ( retValue == 0 ) {
    if ( pszPort != NULL ) {
      (*ev)->ReleaseStringUTFChars(ev, pszPort, port);
    } // if
    return retValue;
  } // if
  
  // c -> java
  (*ev)->SetLongField(ev, pvDest, boxlist_ID.flCtrl, prtlist.wkbbxl.flCtrl);
  (*ev)->SetShortField(ev, pvDest, boxlist_ID.cBox, prtlist.wkbbxl.cBox);
  (*ev)->SetShortField(ev, pvDest, boxlist_ID.usReserved, prtlist.wkbbxl.usReserved);
  
  
  // WKBSERIAL structure
  awkbsr = (*ev)->GetObjectField(ev, pvDest, boxlist_ID.awkbsr);
  
  awkbser_len = (*ev)->GetArrayLength(ev, awkbsr);
  
  for (i = 0; i < awkbser_len; i++) 
  {
    j_boxEntry = (*ev)->GetObjectArrayElement(ev, awkbsr, i);
    
    if(j_boxEntry == NULL)
    {
      j_boxEntry = (*ev)->NewObject(ev, serialCls, newSerial_ID);
      (*ev)->SetObjectArrayElement(ev, awkbsr, i, j_boxEntry);	
    }
    
    (*ev)->SetLongField(ev, j_boxEntry, serial_ID.ulSerial, prtlist.wkbbxl.awkbsr[i].ulSerial);
    (*ev)->SetShortField(ev, j_boxEntry, serial_ID.usMask, prtlist.wkbbxl.awkbsr[i].usMask);
    (*ev)->SetShortField(ev, j_boxEntry, serial_ID.usReserved, prtlist.wkbbxl.awkbsr[i].usReserved);     
  }
  
  if ( pszPort != NULL ) {
    (*ev)->ReleaseStringUTFChars(ev, pszPort, port);
  } // if

  return retValue;
} // Java_com_wibu_WkJavaApi_WkbListPort2()


/***** WkbListBox2 (WKB_LIST_DATA mode) *****/
DLL_EXPORT jlong JNICALL Java_com_wibu_WkJavaApi_WkbListBox2__JJJLjava_lang_String_2Lcom_wibu_WkJavaApi_00024WKBSERIAL_2Lcom_wibu_WkJavaApi_00024WKBBOXDATA_2
  (JNIEnv *ev,jclass cls, jlong v1, jlong flCtrl, jlong lPort, jstring szPort, jobject serial, jobject boxData)
{
  union {
    WKBBOXDATA data;
    //WKBBOXCONFIG config;
  } c_boxData;

  //WKBBOXDATA c_boxData;
  WKBSERIAL c_serial;
  jobjectArray awkbbxe;
  jobject j_boxEntry;
  int i;
  LONG ret;
  int awkbbxe_len;
  const char  *port;

  InitSerial(ev);
  InitBoxData(ev);
  InitBoxEntry(ev);

  //WKBSERIAL java -> c
  c_serial.ulSerial = (ULONG) (*ev)->GetLongField(ev, serial, serial_ID.ulSerial);
  c_serial.usMask = (*ev)->GetShortField(ev, serial, serial_ID.usMask);
  c_serial.usReserved = (*ev)->GetShortField(ev, serial, serial_ID.usReserved);

  //Determine if port is specified as String or Number
  if((flCtrl & WKB_PORTNUMBER) == WKB_PORTNUMBER) {
    port = MAKEINTRESOURCE((ULONG)lPort);
  } else {
    port = (szPort == NULL ? NULL : (char *)(*ev)->GetStringUTFChars(ev, szPort, NULL));
  }
  
  // call WIBU function
  ret = WkbListBox2((HWKBSYSTEM)v1, (ULONG)flCtrl | WKB_LIST_DATA, port, &c_serial, &c_boxData, sizeof(c_boxData));
  // return immediately on error
  if ( ret == 0 ) {
    // free memory
    if ( ((flCtrl & WKB_PORTNUMBER) != WKB_PORTNUMBER)
      && (szPort != NULL) ){
      (*ev)->ReleaseStringUTFChars(ev, szPort, port);
    } // if
    return ret;
  } // if

  //WKBBOXDATA c -> java
  (*ev)->SetLongField(ev, boxData, boxData_ID.flCtrl, c_boxData.data.flCtrl);
  (*ev)->SetShortField(ev, boxData, boxData_ID.usVersion, c_boxData.data.usVersion);
  (*ev)->SetShortField(ev, boxData, boxData_ID.cEntry, c_boxData.data.cEntry);
  // BOXENTRY-array awkbbxe: make new if necessary
  awkbbxe = (*ev)->GetObjectField(ev, boxData, boxData_ID.awkbbxe);
  if(awkbbxe == NULL ||  (*ev)->GetArrayLength(ev, awkbbxe) < 10)
  {
    awkbbxe = (*ev)->NewObjectArray(ev, 10, boxEntryCls, NULL);
  }
  awkbbxe_len = (*ev)->GetArrayLength(ev, awkbbxe);
  for (i = 0; i < c_boxData.data.cEntry; i++)  {
    j_boxEntry = (*ev)->GetObjectArrayElement(ev, awkbbxe, i);
    if(j_boxEntry == NULL) {
      j_boxEntry = (*ev)->NewObject(ev, boxEntryCls, newBoxEntry_ID);
      (*ev)->SetObjectArrayElement(ev, awkbbxe, i, j_boxEntry);
    }
    (*ev)->SetLongField(ev, j_boxEntry, boxEntry_ID.flCtrl, c_boxData.data.awkbbxe[i].flCtrl);
    (*ev)->SetLongField(ev, j_boxEntry, boxEntry_ID.ulFirmCode, c_boxData.data.awkbbxe[i].ulFirmCode);
    (*ev)->SetLongField(ev, j_boxEntry, boxEntry_ID.ulUserCode, c_boxData.data.awkbbxe[i].ulUserCode);
    (*ev)->SetLongField(ev, j_boxEntry, boxEntry_ID.ulData1, c_boxData.data.awkbbxe[i].ulData1);
    (*ev)->SetLongField(ev, j_boxEntry, boxEntry_ID.ulData2, c_boxData.data.awkbbxe[i].ulData2);      
  } // for

  if ( ((flCtrl & WKB_PORTNUMBER) != WKB_PORTNUMBER)
    && (szPort != NULL) ){
    (*ev)->ReleaseStringUTFChars(ev, szPort, port);
  } // if

  return ret;
}

/***** WkbListBox2 (WKB_LIST_CONFIG mode) *****/
DLL_EXPORT jlong JNICALL Java_com_wibu_WkJavaApi_WkbListBox2__JJJLjava_lang_String_2Lcom_wibu_WkJavaApi_00024WKBSERIAL_2Lcom_wibu_WkJavaApi_00024WKBBOXCONFIG_2
  (JNIEnv *ev,jclass cls, jlong v1, jlong flCtrl, jlong lPort, jstring szPort, jobject serial, jobject boxData)
{
  union{
    //WKBBOXDATA data;
    WKBBOXCONFIG config;
  } c_boxData;

  //WKBBOXDATA c_boxData;
  WKBSERIAL c_serial;
  LONG ret;
  const char  *port;

  // initialize output
  memset(&c_boxData, 0, sizeof(WKBBOXCONFIG)); 


  InitSerial(ev);
  InitBoxConfig(ev);

  //WKBSERIAL java -> c
  c_serial.ulSerial = (ULONG) (*ev)->GetLongField(ev, serial, serial_ID.ulSerial);
  c_serial.usMask = (*ev)->GetShortField(ev, serial, serial_ID.usMask);
  c_serial.usReserved = (*ev)->GetShortField(ev, serial, serial_ID.usReserved);

  //Determine if port is specified as String or Number
  if((flCtrl & WKB_PORTNUMBER) == WKB_PORTNUMBER) {
    port = MAKEINTRESOURCE((ULONG)lPort);
  } else {
    port = (szPort == NULL ? NULL : (char *)(*ev)->GetStringUTFChars(ev,szPort, NULL));
  }

  // call WIBU function
  ret = WkbListBox2((HWKBSYSTEM)v1, (ULONG)flCtrl | WKB_LIST_CONFIG, port, &c_serial, &c_boxData, sizeof(c_boxData));
  // return immediately on error
  if ( ret == 0 ) {
    // free memory
    if ( ((flCtrl & WKB_PORTNUMBER) != WKB_PORTNUMBER)
      && (szPort != NULL) ){
      (*ev)->ReleaseStringUTFChars(ev, szPort, port);
    } // if
    return ret;
  } // if

  (*ev)->SetLongField(ev, boxData, boxConfig_ID.flCtrl, c_boxData.config.flCtrl);
  (*ev)->SetShortField(ev, boxData, boxConfig_ID.usVersion, c_boxData.config.usVersion);
  (*ev)->SetShortField(ev, boxData, boxConfig_ID.usReserved, c_boxData.config.usReserved);
  (*ev)->SetLongField(ev, boxData, boxConfig_ID.ulProgramCount, c_boxData.config.ulProgramCount);
  (*ev)->SetLongField(ev, boxData, boxConfig_ID.ulConfig, c_boxData.config.ulConfig);
  (*ev)->SetLongField(ev, boxData, boxConfig_ID.flCtrl, c_boxData.config.flCtrl);
  (*ev)->SetShortField(ev, boxData, boxConfig_ID.usExtMemSize, c_boxData.config.usExtMemSize);
  (*ev)->SetShortField(ev, boxData, boxConfig_ID.usAsicBuild, c_boxData.config.usAsicBuild);

  // dereference port
  if ( ((flCtrl & WKB_PORTNUMBER) != WKB_PORTNUMBER) 
      && (szPort != NULL) ) {
      (*ev)->ReleaseStringUTFChars(ev, szPort, port);
  }
  
  return ret;
} // WkbListBox2 (WKB_LIST_CONFIG mode) 


/***** WkbListBox2 (WKB_LIST_EXTMEM mode) *****/
DLL_EXPORT jlong JNICALL Java_com_wibu_WkJavaApi_WkbListBox2__JJLjava_lang_String_2Lcom_wibu_WkJavaApi_00024WKBSERIAL_2Lcom_wibu_WkJavaApi_00024WKBEXTMEM_2
  (JNIEnv *ev, jclass cls, jlong v1, jlong flCtrl, jstring szPort, jobject serial, jobject wkbextmem)
{
  LONG            ret = 0;
  const char  *port = 0;
  char            szPortBuffer[64];
  WKBSERIAL       c_serial;
  WKBEXTMEM       c_extMem;

  // map parameters
  if (szPort != NULL ) {
    J2C_String(ev, szPort, szPortBuffer, (int)sizeof(szPortBuffer));
    port = szPortBuffer;
  } else {
    port = NULL;
  } // if

  // WKB_EXT_MEM is set by default
  flCtrl  |= WKB_LIST_EXTMEM;


  J2C_WKBSERIAL(ev, serial, &c_serial);
  DUMB_WKBSERIAL(c_serial);
  J2C_WKBEXTMEM(ev, wkbextmem, &c_extMem);

  // call WIBU function
  ret = WkbListBox2((HWKBSYSTEM) v1, (ULONG) flCtrl, port, &c_serial, &c_extMem, sizeof(c_extMem));

  // return immediately on error
  if ( ret == 0 ) {
    return ret;
  } // if

  // set destination
  C2J_WKBEXTMEM(ev, c_extMem, wkbextmem);

  return ret;
} // Java_com_wibu_WkJavaApi_WkbListBox2__JJLjava_lang_String_2Lcom_wibu_WkJavaApi_00024WKBSERIAL_2Lcom_wibu_WkJavaApi_00024WKBEXTMEM_2()

/***** WkbCrypt2 *****/
DLL_EXPORT jint JNICALL Java_com_wibu_WkJavaApi_WkbCrypt2
(JNIEnv *ev, jclass cls, jlong v1, jlong v2, jbyteArray pvDest, jobject area, jobject pcbDest)
{
  WKBAREA c_area;
  WKBSELECT c_select;
  jbyteArray pbGapMap;
  jobject select;
  jbyteArray abInit;
  int cbDest;
  //BOOL retVal;
  int retVal;
  jbyte* pData;

  InitArea(ev);
  InitSelect(ev);
  InitPInt(ev);

  //WKBAREA java -> c
  c_area.flCtrl = (ULONG) (*ev)->GetLongField(ev, area, area_ID.flCtrl);
  c_area.flSelCtrl = (ULONG) (*ev)->GetLongField(ev, area, area_ID.flSelCtrl);
  c_area.ulSelectCode = (ULONG) (*ev)->GetLongField(ev, area, area_ID.ulSelectCode);
  c_area.ulCrc = (ULONG) (*ev)->GetLongField(ev, area, area_ID.ulCrc);
  c_area.ulReserved = (ULONG) (*ev)->GetLongField(ev, area, area_ID.ulReserved);
  pbGapMap = (*ev)->GetObjectField(ev, area, area_ID.pbGapMap);
  c_area.pbGapMap = (pbGapMap == NULL ? NULL : (*ev)->GetByteArrayElements(ev, pbGapMap, NULL));
  select = (*ev)->GetObjectField(ev, area, area_ID.pvCtrl);


  /*  If the WKB_AREA_SELECT flag is not set or flSelCtrl contains
   WKB_SEL_DIRECT or WKB_SEL_USE_CACHE, this value must be set to NULL. */
  if ( ((c_area.flCtrl & WKB_AREA_SELECT) == 0)
  || ( (c_area.flSelCtrl & WKB_SEL_MASK) == WKB_SEL_DIRECT) ) {
    c_area.pvCtrl = 0;
  } else {
    c_area.pvCtrl = &c_select;
    //WKBSELECT java -> c
    c_select.flCtrl = (ULONG)(*ev)->GetLongField(ev, select, select_ID.flCtrl);
    c_select.ulCacheId = (ULONG)(*ev)->GetLongField(ev, select, select_ID.ulCacheId);
    c_select.ulCrc = (ULONG)(*ev)->GetLongField(ev, select, select_ID.ulCrc);
    c_select.cbInit = (USHORT)(*ev)->GetIntField(ev, select, select_ID.cbInit);
    abInit = (*ev)->GetObjectField(ev, select, select_ID.abInit);
    (*ev)->GetByteArrayRegion(ev, abInit, 0, (*ev)->GetArrayLength(ev, abInit), c_select.abInit);
  } // if/else

  //pvDest java-array -> c-array
  pData = (*ev)->GetByteArrayElements(ev, pvDest, NULL);

  DUMP_WKBAREA(c_area);

  //actual function-call
  retVal = WkbCrypt2((HWKBENTRY)v1, (ULONG)v2, pData, &c_area, (UINT)(*ev)->GetArrayLength(ev, pvDest), &cbDest);

  (*ev)->ReleaseByteArrayElements(ev, pvDest, pData, 0);
  if(pbGapMap != NULL) {
    (*ev)->ReleaseByteArrayElements(ev, pbGapMap, (jbyte *)c_area.pbGapMap, 0);
  }
   
  // return  on error (after releasing memory)
  if ( retVal == 0 ) {
    return retVal;
  }

  //c-int* -> java-pInt
  (*ev)->SetIntField(ev, pcbDest, pInt_ID, cbDest);
  
  // check for WKB_AREA_CALCCRC flag
  if ( c_area.flCtrl & WKB_AREA_CALCCRC ) {    
    (*ev)->SetLongField(ev, area, area_ID.ulCrc, c_area.ulCrc);    
  } // if

  return retVal;
} // Java_com_wibu_WkJavaApi_WkbCrypt2()


/***** WkbClose2 *****/
DLL_EXPORT jboolean JNICALL Java_com_wibu_WkJavaApi_WkbClose2(JNIEnv *ev, 
                                                             jclass cls, jlong v1)
{
  // call WIBU function
  return (jboolean) WkbClose2((HWKBENTRY)v1);
} // Java_com_wibu_WkJavaApi_WkbClose2()

// *******************************************************************************************
// General Error and Execution Management
// *******************************************************************************************


// ***** WkbGetLastError *****
DLL_EXPORT jint JNICALL Java_com_wibu_WkJavaApi_WkbGetLastError
  (JNIEnv *ev, jobject ob1)
{
  // call WIBU function
  return (jint) WkbGetLastError();
}


/***** WkbSetLastError *****/
DLL_EXPORT void JNICALL Java_com_wibu_WkJavaApi_WkbSetLastError
  (JNIEnv *ev, jclass cls, jint idError)
{
	// call WIBU function
	WkbSetLastError(idError);
}


/***** WkbIsComplete *****/
DLL_EXPORT jboolean JNICALL Java_com_wibu_WkJavaApi_WkbIsComplete
  (JNIEnv *ev, jclass cls, jint nActionID, jint pidReturn)
{
  int idReturn  = 0;
  // call WIBU function
  if ( WkbIsComplete(nActionID, &idReturn) ) {
    pidReturn = idReturn;
    return TRUE;
  } else {
    return FALSE;
  } // if
} // Java_com_wibu_WkJavaApi_WkbIsComplete()


//*******************************************************************************************
// Driver Status and Control
//*******************************************************************************************

/***** WkbGetVersion2 *****/
DLL_EXPORT jint JNICALL Java_com_wibu_WkJavaApi_WkbGetVersion2
  (JNIEnv *ev, jclass cls, jlong hwkbsys)
{
	// call WIBU function
	return (jint) WkbGetVersion2((HWKBSYSTEM)hwkbsys);	
}


/***** WkbGetVersion3 *****/
DLL_EXPORT jint JNICALL Java_com_wibu_WkJavaApi_WkbGetVersion3
  (JNIEnv *ev, jclass cls, jlong hwkbsys)
{
	// call WIBU function
	return (jint)WkbGetVersion3((HWKBSYSTEM)hwkbsys);	
}


/***** WkbControlSystem2 *****/
DLL_EXPORT jint JNICALL Java_com_wibu_WkJavaApi_WkbControlSystem2
  (JNIEnv *ev, jclass cls, jlong hwkbsys, jlong flCtrl, jobject pvCtrl, jlong cbCtrl)
{
  // call WIBU function
  return (jint)WkbControlSystem2((HWKBSYSTEM)hwkbsys, (ULONG)flCtrl, &pvCtrl, (ULONG)cbCtrl);	
}


/***** WkbQueryInfo2 *****/
DLL_EXPORT jint JNICALL 
Java_com_wibu_WkJavaApi_WkbQueryInfo2(JNIEnv *ev, jclass cls, jlong hwkbsys,
                                      jlong flCtrl, jobject pvDest, 
                                      jint cbDest)
{
	WKBVERSION  c_wkbversion;
	char        c_charBuf[256];
	char        errMessage[30] = "WkbQueryInfo2";
	int         retValue = 0;
	
  // init wkbversion java structure
  InitVersion(ev);

	if (((ULONG)flCtrl & WKB_QI_JNI) != 0) {
    // special JAVA version flag
    // c -> java
    if (((ULONG)flCtrl & WKB_QI_VERVALUE) != 0) {
      (*ev)->SetShortField(ev, pvDest, version_ID.usVersion, DRV_VERSION);
      (*ev)->SetShortField(ev, pvDest, version_ID.usSubVersion, DRV_SUBVERSION);
      (*ev)->SetShortField(ev, pvDest, version_ID.usBuild, DRV_VER_BUILD);
      (*ev)->SetShortField(ev, pvDest, version_ID.usCount, DRV_VER_COUNT);
      (*ev)->SetShortField(ev, pvDest, version_ID.usYear, 2000+DRV_VER_YEAR);
      (*ev)->SetByteField(ev, pvDest, version_ID.bMonth, DRV_VER_MONTH);
      (*ev)->SetByteField(ev, pvDest, version_ID.bDay, DRV_VER_DAY);
      (*ev)->SetLongField(ev, pvDest, version_ID.ulReserved1, 0);
      retValue = sizeof(c_wkbversion);
    } else {
      WkbSetLastError(WKB_ERROR_OPTION_NOT_SUPPORTED);
    } // if/else
  } else if (((ULONG)flCtrl & WKB_QI_VERVALUE) != 0) {
    // paranoia check - only one version info can be retrieved in one function call
    ULONG flag  = (ULONG) flCtrl;
    if ( (flag & WKB_QI_DRIVER)
      && (flag & WKB_QI_SERVER) ) {
      WkbSetLastError(WKB_ERROR_INVALID_PARAMETER);
      return 0;
    } // if

    // call WIBU function
    retValue = WkbQueryInfo2((HWKBSYSTEM)hwkbsys, (ULONG)flCtrl, &c_wkbversion, sizeof(c_wkbversion));

    // c -> java
    (*ev)->SetShortField(ev, pvDest, version_ID.usVersion, c_wkbversion.usVersion);
    (*ev)->SetShortField(ev, pvDest, version_ID.usSubVersion, c_wkbversion.usSubVersion);
    (*ev)->SetShortField(ev, pvDest, version_ID.usBuild, c_wkbversion.usBuild);
    (*ev)->SetShortField(ev, pvDest, version_ID.usCount, c_wkbversion.usCount);
    (*ev)->SetShortField(ev, pvDest, version_ID.usYear, c_wkbversion.usYear);
    (*ev)->SetByteField(ev, pvDest, version_ID.bMonth, c_wkbversion.bMonth);
    (*ev)->SetByteField(ev, pvDest, version_ID.bDay, c_wkbversion.bDay);
    (*ev)->SetLongField(ev, pvDest, version_ID.ulReserved1, c_wkbversion.ulReserved1);
  } else	{
    retValue = Create_C_Str_From_Java(ev, pvDest, c_charBuf, errMessage);
    retValue = Delete_Str_In_Buffer(ev, pvDest, errMessage);

    if (retValue != 0) {
      // call WIBU function
      retValue = WkbQueryInfo2((HWKBSYSTEM)hwkbsys, (ULONG)flCtrl, c_charBuf, sizeof(c_charBuf));
      // return immediately on error
      if ( retValue == 0 ) {
        return retValue;
      } // if
      // c -> java
      C2J_AppendToStringBuffer(ev, pvDest, c_charBuf, errMessage);
    } // if
  } // if/else

  return (jint) retValue;
} // Java_com_wibu_WkJavaApi_WkbQueryInfo2()


/***** WkbQueryStatus2 *****/
DLL_EXPORT jint JNICALL Java_com_wibu_WkJavaApi_WkbQueryStatus2
  (JNIEnv *ev, jclass cls, jlong hwkbsys, jlong flCtrl, jobject pwkbst, jint cbDest)
{
	//!
	WKBSTATUS c_wkbstatus;
	int retValue = 0;
	
	// init wkbstatus java structure
	InitQueryStatus(ev);
	
	// call WIBU function
	retValue = WkbQueryStatus2((HWKBSYSTEM)hwkbsys, (ULONG)flCtrl, &c_wkbstatus, sizeof(c_wkbstatus));
  // return immediately on error
  if ( retValue == 0 ) {
    return retValue;
  }
	
	// c -> java
	(*ev)->SetShortField(ev, pwkbst, queryStatus_ID.usFlags, c_wkbstatus.usFlags);
  	(*ev)->SetShortField(ev, pwkbst, queryStatus_ID.usDrvInt, c_wkbstatus.usDrvInt);
  
  	return (jint)retValue;
}


// *******************************************************************************************
// Port Status and Control
// *******************************************************************************************

// ***** WkbQueryPort2 *****
DLL_EXPORT jint JNICALL Java_com_wibu_WkJavaApi_WkbQueryPort2
  (JNIEnv *ev, jclass cls, jlong hwkbsys, jlong flCtrl, jstring pszPort, jobject pvDest, jint cbDest)
{
  typedef struct {
    WKBPORTNAME wkbprtn;
    char szPort[30];
  } WKBTHISOE;
  WKBPORT         c_wkbport;
  WKBTHISOE       c_wkbportname;
  const char  *port;
  char            c_charBuf[256];
  char            errMessage[30] = "WkbQueryPort2";
  int             retValue = 0;

  // mapping pszPort
  port = (pszPort == NULL ? NULL : (char *)(*ev)->GetStringUTFChars(ev,pszPort, NULL));

  if (((ULONG)flCtrl & WKB_QP_NAME) != 0)
  {
    InitPortName(ev);

    // call WIBU function
    retValue = (int)WkbQueryPort2((HWKBSYSTEM)hwkbsys, (ULONG)flCtrl, port, &c_wkbportname, sizeof(c_wkbportname));
    // return immediately on error
    if ( retValue == 0 ) {
      return retValue;
    }

    // c -> java
    (*ev)->SetLongField(ev, pvDest, portname_ID.flCtrl, c_wkbportname.wkbprtn.flCtrl);
    (*ev)->SetLongField(ev, pvDest, portname_ID.lPort, c_wkbportname.wkbprtn.lPort);
    (*ev)->SetObjectField(ev, pvDest,portname_ID.pszPort, (*ev)->NewStringUTF(ev, c_wkbportname.wkbprtn.pszPort)); 

  } else if (((ULONG)flCtrl & WKB_QP_CONFIG_NAMES) != 0) {
    Create_C_Str_From_Java(ev, pvDest, c_charBuf, errMessage);
    Delete_Str_In_Buffer(ev, pvDest, errMessage);

    if (cbDest > 256)
      cbDest = 256;

    if (retValue != 0)
    {
      // call WIBU function
      retValue = WkbQueryPort2((HWKBSYSTEM)hwkbsys, (ULONG)flCtrl, port, c_charBuf, cbDest);
      // return immediately on error
      if ( retValue == 0 ) {
        return retValue;
      }
      // c -> java
      C2J_AppendToStringBuffer(ev, pvDest, c_charBuf, errMessage);
    }
  } else {
    // init wkbport java structure
    InitPort(ev);

    // call WIBU function
    retValue = WkbQueryPort2((HWKBSYSTEM)hwkbsys, (ULONG)flCtrl, port, &c_wkbport, sizeof(c_wkbport));
    // return immediately on error
    if ( retValue == 0 ) {
      return retValue;
    }

    // c -> java
    (*ev)->SetLongField(ev, pvDest, port_ID.ulAddress, (jlong)c_wkbport.ulAddress);
  } // if/elseif/else

  if ( pszPort != NULL ) {
    (*ev)->ReleaseStringUTFChars(ev, pszPort, port);
  } // if

  return (jint)retValue;		
}



/***** WkbSetPort2 *****/
DLL_EXPORT jint JNICALL Java_com_wibu_WkJavaApi_WkbSetPort2
  (JNIEnv *ev, jclass cls, jlong hwkbsys, jlong flCtrl, jstring pszPort, jbyteArray pbData, jint cbData)
{
  int retValue;
  const char *port;
  jbyte *c_pbData = (jbyte *) malloc(cbData);

  // mapping pszPort
  // only strings
  flCtrl = flCtrl & (1^WKB_PORTNUMBER);

  port = (pszPort == NULL ? NULL : (const char *)(*ev)->GetStringUTFChars(ev,pszPort, NULL));

  // java -> c
  (*ev)->GetByteArrayRegion(ev, pbData, 0, (jint)cbData, c_pbData);

  // call WIBU function
  retValue = WkbSetPort2((HWKBSYSTEM)hwkbsys, (ULONG)flCtrl, port, c_pbData, (UINT)cbData);
  //!I:jo do NOT return immediately on error

  // house keeping
  free(c_pbData);
  return (jint)retValue; 
}


// *******************************************************************************************
// Programming WIBU-BOXes
// *******************************************************************************************

// ***** WkbProgramm2 *****
// commands WKB_PGM_NEW | WKB_PGM_ADD | WKB_PGM_MODIFY | WKB_PGM_MODIFY_ADDED 
DLL_EXPORT jboolean JNICALL Java_com_wibu_WkJavaApi_WkbProgram2__JJLjava_lang_String_2Lcom_wibu_WkJavaApi_00024WKBSERIAL_2Lcom_wibu_WkJavaApi_00024WKBPROGRAM_2
  (JNIEnv *ev, jclass cls, jlong hwkbsys, jlong flCtrl, jstring pszPort, jobject pwkbsr, jobject pvPgmData)
{
  WKBSERIAL c_pwkbsr;
  WKBPROGRAM c_pvPgmData;
  const char *port;
  jbyteArray pbSignature;

  InitSerial(ev);
  InitProgram(ev);

  // WKBSERIAL java -> c
  c_pwkbsr.ulSerial = (ULONG) (*ev)->GetLongField(ev, pwkbsr, serial_ID.ulSerial);
  c_pwkbsr.usMask = (*ev)->GetShortField(ev, pwkbsr, serial_ID.usMask);
  c_pwkbsr.usReserved = (*ev)->GetShortField(ev, pwkbsr, serial_ID.usReserved);

  if ( flCtrl & WKB_PGM_CONFIG ) {
    // not yet supported - requires new overloaded function with WKBBOXCONFIG (Structure)
    WkbSetLastError(WKB_ERROR_INVALID_FUNCTION);
    return 0;
  } else {
    c_pvPgmData.flCtrl = (ULONG) (*ev)->GetLongField(ev, pvPgmData, program_ID.flCtrl);
    c_pvPgmData.wkbbxe.flCtrl = (ULONG) (*ev)->GetLongField(ev, (*ev)->GetObjectField(ev, pvPgmData, program_ID.wkbbxe), boxEntry_ID.flCtrl);
    c_pvPgmData.wkbbxe.ulData1 = (ULONG) (*ev)->GetLongField(ev, (*ev)->GetObjectField(ev, pvPgmData, program_ID.wkbbxe), boxEntry_ID.ulData1);
    c_pvPgmData.wkbbxe.ulData2 = (ULONG) (*ev)->GetLongField(ev, (*ev)->GetObjectField(ev, pvPgmData, program_ID.wkbbxe), boxEntry_ID.ulData2);
    c_pvPgmData.wkbbxe.ulFirmCode = (ULONG) (*ev)->GetLongField(ev, (*ev)->GetObjectField(ev, pvPgmData, program_ID.wkbbxe), boxEntry_ID.ulFirmCode);
    c_pvPgmData.wkbbxe.ulUserCode = (ULONG) (*ev)->GetLongField(ev, (*ev)->GetObjectField(ev, pvPgmData, program_ID.wkbbxe), boxEntry_ID.ulUserCode);

    //pvDest java-array -> c-array
    pbSignature = (*ev)->GetObjectField(ev, pvPgmData, program_ID.abSignature);
    if ( pbSignature == 0 ) {
      WkbSetLastError(WKB_ERROR_BAD_ADDRESS);
      return 0;
    } // if
    memcpy(c_pvPgmData.abSignature, (*ev)->GetByteArrayElements(ev, pbSignature, NULL), sizeof(c_pvPgmData.abSignature));
  } // if/else

  //mapping pszPort
  port = (pszPort == NULL ? NULL : (char *)(*ev)->GetStringUTFChars(ev,pszPort, NULL));

  return WkbProgram2((HWKBSYSTEM)hwkbsys, (ULONG) flCtrl, port, &c_pwkbsr, &c_pvPgmData);
} // Java_com_wibu_WkJavaApi_WkbApplicationInfo2()


/***** WkbProgramm2 *****/
// commands WKB_PGM_CONFIG
DLL_EXPORT jboolean JNICALL Java_com_wibu_WkJavaApi_WkbProgram2__JJLjava_lang_String_2Lcom_wibu_WkJavaApi_00024WKBSERIAL_2Lcom_wibu_WkJavaApi_00024WKBBOXCONFIG_2
   (JNIEnv *ev, jclass cls, jlong hwkbsys, jlong flCtrl, jstring pszPort, jobject pwkbsr, jobject pvPgmData)
{
  WkbSetLastError(WKB_ERROR_INVALID_FUNCTION);
  
  return 0;	
} // Java_com_wibu_WkJavaApi_WkbProgram2__JJLjava_lang_String_2Lcom_wibu_WkJavaApi_00024WKBSERIAL_2Lcom_wibu_WkJavaApi_00024WKBBOXCONFIG_2()


/*
 * Class:     com_wibu_WkJavaApi
 * Method:    WkbProgram2
 * Signature: (JJLjava/lang/String;Lcom/wibu/WkJavaApi$WKBSERIAL;Lcom/wibu/WkJavaApi$WKBEXTMEM;)Z
 */
/***** WkbProgramm2(WKBEXTMEM) *****/
DLL_EXPORT jboolean JNICALL Java_com_wibu_WkJavaApi_WkbProgram2__JJLjava_lang_String_2Lcom_wibu_WkJavaApi_00024WKBSERIAL_2Lcom_wibu_WkJavaApi_00024WKBEXTMEM_2
  (JNIEnv *ev, jclass cls, jlong hwkbsys, jlong flCtrl, jstring pszPort, jobject pwkbsr, jobject pvExtMem)
{
  WKBSERIAL   c_wkbsr;
  WKBEXTMEM   c_wkbem;
  //jobject     serial;
  const char *port;

  // 'repair' flCtrl
  flCtrl  |= WKB_PGM_EXTMEM;

  // Map parameters
  J2C_WKBSERIAL(ev, pwkbsr, &c_wkbsr);
  J2C_WKBEXTMEM(ev, pvExtMem, &c_wkbem);
  port = (pszPort == NULL ? NULL : (char *)(*ev)->GetStringUTFChars(ev, pszPort, NULL));

  return WkbProgram2((HWKBSYSTEM)hwkbsys, (ULONG) flCtrl, port, &c_wkbsr, &c_wkbem);
} // Java_com_wibu_WkJavaApi_WkbProgram2__JJLjava_lang_String_2Lcom_wibu_WkJavaApi_00024WKBSERIAL_2Lcom_wibu_WkJavaApi_00024WKBEXTMEM_2()

// *******************************************************************************************
//  Driver Local Memory Management
// *******************************************************************************************

// ***** WkbAllocMem *****
DLL_EXPORT jlong JNICALL Java_com_wibu_WkJavaApi_WkbAllocMem
  (JNIEnv *ev, jclass cls, jlong hwkbsys, jlong cbAlloc)
{
	return (jlong)WkbAllocMem((HWKBSYSTEM)hwkbsys, (UINT)cbAlloc);
}


/***** WkbFreeMem *****/
DLL_EXPORT jboolean JNICALL Java_com_wibu_WkJavaApi_WkbFreeMem
  (JNIEnv *ev, jclass cls, jlong hwkbmem)
{
	return WkbFreeMem((HWKBMEM)hwkbmem);
}


/***** WkbWriteMem *****/
DLL_EXPORT jlong JNICALL Java_com_wibu_WkJavaApi_WkbWriteMem
  (JNIEnv *ev, jclass cls, jlong hwkbmem, jbyteArray pvSrc, jlong cbData)
{
	int retValue;
	jbyte *c_buf = (jbyte *) malloc((jint)cbData);
		
	// java -> c
	(*ev)->GetByteArrayRegion(ev, pvSrc, 0, (jint)cbData, c_buf);
	
	retValue = WkbWriteMem((HWKBMEM)hwkbmem, c_buf, (UINT)cbData);
	
	free(c_buf);
	
	return (jlong)retValue; 
}


/***** WkbReadMem *****/
DLL_EXPORT jlong JNICALL Java_com_wibu_WkJavaApi_WkbReadMem
  (JNIEnv *ev, jclass cls, jlong hwkbmem, jbyteArray pvDest, jlong cbDest)
{
 	jbyte *c_buf = (jbyte *) malloc((jint)cbDest);
 	int retValue = 0;
 	
 	retValue = WkbReadMem((HWKBMEM)hwkbmem, c_buf, (UINT)cbDest);
 	
 	// c -> java
 	(*ev)->SetByteArrayRegion(ev, pvDest, 0, (jint)cbDest, c_buf);
 	
 	free(c_buf);
 	
 	return (jlong)retValue;
}

/*****************************************************************************

------------------- V E R S I O N   I N F O R M A T I O N --------------------

*****************************************************************************/


#define CONCAT(arg1, arg2)      arg1 ## arg2
#define VERSION_FUNC(type, num) CONCAT( wibuKJni_ ## type, num)



DLL_EXPORT const char *wibuKJni_version_string()
{
  static char	szVersion[256];
  char		szVerText[64];
  char		szVerSub[64];
  const char *szMonth[] = {"January", "February", "March", "April", 
    "May", "June", "July", "August", "September", "October", "November", "December"};
  
  // main version
  sprintf(szVerText, "Version %d.%02d", DRV_VERSION, DRV_SUBVERSION);
  
  // sub version
  sprintf(szVerSub, "%s", "");

  // date
  sprintf(szVersion, "%s%s of %s %d, 20%02d", szVerText, szVerSub, 
  szMonth[DRV_VER_MONTH-1], DRV_VER_DAY, DRV_VER_YEAR);
  
  return szVersion;
} // wibuKJni_version_string()
 

/*
 * Class:     com_wibu_WkJavaApi
 * Method:    WibuKJniGetVersion
 * Signature: (J)I
 */

DLL_EXPORT jint JNICALL Java_com_wibu_WkJavaApi_WibuKJniGetVersionString(JNIEnv *ev, jclass cls, jobject pszDest)
{
  char errMessage[30] = "wibuKJni_version_string";
  const char *pszVersion = wibuKJni_version_string();
  C2J_AppendToStringBuffer(ev, pszDest, pszVersion, errMessage);
  return (jint) strlen(pszVersion);
} // wibuKJni_get_version_string()




struct
{
  jfieldID  ulCtrl;
  jfieldID  idUpdate;
  jfieldID  cEntry;
  jfieldID  cSubsequent;
  jfieldID  awkmusr;
} userList_ID;

static BOOL userListIsInit = FALSE;


#if JNI_DEBUG
DLL_EXPORT const char *wibuKJni_version_debug()
{
  return "This Library version contains debug output to the console.";
} // wibuKJni_version_debug()
#endif	// JNI_DEBUG



/* Defined by native libraries. */
DLL_EXPORT jint JNICALL JNI_OnLoad(JavaVM *vm, void *reserved)
{
  assert( sizeof(USHORT) == 2 );
  assert( sizeof(UINT) == 4 );

  return JNI_VERSION_1_2;
} // JNI_OnLoad()

DLL_EXPORT void JNICALL JNI_OnUnload(JavaVM *vm, void *reserved)
{
  return;
} // JNI_OnUnload()


// ===========================================
// wk/dev/lib/java/wibuKJni: end of wibuKJni.c
// ===========================================
