/*****************************************************************************
S r c D r v                                Portable Source Code WibuKey Driver
==============================================================================
proj.h                                             Central project header file
==============================================================================

This header file contains all declarations which are used by several
driver source modules.

==============================================================================

The public WibuKey source driver (SRCDRV) bases on the WIBU-SYSTEMS
internally used portable driver which is used for WKWIN32.DLL, WKWIN.DLL,
WIBUKEY.SYS, WIBUKEY.VXD and WKDOS.EXE.

The SRCDRV driver variant supports the full WibuKey API functionality except
indirect encryption, Expiration Dates, WkLAN, diagnosis event recording and
some minor features.

The SRCDRV can be used for several DOS extender programs or in special
hardware, system or processor technology environments.

All sources are written in ANSI C, except the using of C++ style single line
comments.

==============================================================================

Copyright  1992 - 2010 by WIBU-SYSTEMS AG
           Rueppurrer Strasse 52-54, D-76137 Karlsruhe, Germany/Europe
           Website: http://www.wibu.com, Email: info@wibu.com
           Phone: +49-721-93172-0, Fax: +49-721-93172-22
           All rights reserved.

Version .. 4.31 of 2010-Jun-24
Project .. wibu/wk/dev/drv/src
Author ... Marcellus Buchheit (mabu), Stephan Sptitz (StS)
           Markus Neumann (mn), Heiko Abraham (ha)
System ... All POSIX OS
Tools .... GNU gcc

Change Report
92Sep13 1.33 mabu: start of development
93Jun06 1.35 mabu: conversation to large memory model
93Jun21 1.35 mabu: adaptation to Win32s
93Jul22 1.35 mabu: adaptation to DOS (real mode)
93Aug01 1.35 mabu: adaptation to PcCard slot
93Aug21 2.00 mabu: greater changes in API of new functions
93Oct29 2.00 mabu: separate memory allocation implemented
94Mar08 2.00 mabu: low level driver variants added
94Apr26 2.00 mabu: adaptation to Windows NT kernel driver
94Sep16 2.00 mabu: adaptation to buffered communication like WibuBox/A
94Oct21 2.00 mabu: structure TIMEMMT introduced
94Nov11 2.00 mabu: version 2.00 released
95Jan02 2.01 mabu: version 2.01 released
95Feb06 2.02 mabu: memory option DS!=SS activated for DOS driver variant
95Feb09 2.02 mabu: KRNL.C split into multiple BSxxx.C and CTRL.C modules
95Feb11 2.05 mabu: introduction of the CALLSTATUS structure
95Jun04 2.06 mabu: adding of WKINT calling under DOS (CALL_WKINT option)
96Jul06 2.06 mabu: renamed from WKDRV.H
95Aug23 2.07 mabu: adaptation to Visual C++/Macintosh
95Oct08 2.08 mabu: precision timing implemented for Windows NT kernel driver
95Oct31 2.08 mabu: absolute timing mechanism implemented
95Dec02 2.09 mabu: no interrupt switching in WKWIN.DLL
95Dec21 2.09 mabu: version 2.09 released
95Dec22 2.10 mabu: new low-level timing/status management implemented
96Jan29 2.10 mabu: adapted to new NT kernel port addressing scheme
96Feb20 2.10 mabu: replacing of WkInt calling by WkDos calling scheme
96May13 2.11 mabu: beginning of WkLAN implementation, based on sockets
96May29 2.11 mabu: version 2.11 released
96Jun23 2.13 mabu: new ordering of declarations in this file
96Jul23 2.13 sts:  adapted to Symantec C++ (Macintosh) compiler
96Sep27 2.14 mabu: new datagram format introduced for WkLAN
96Oct19 2.15 mabu: WkDOS calling mechanism partially expanded to WKWIN.DLL
96Dec06 2.21 mabu: saving of subsystem Firm/User Code in base system structure
97Feb22 2.25 mabu: separate status saving for Unicode and asynchronous calls
97Feb25 2.30 mabu: adding of resources constants for dialogs, texts & icons
97Feb26 2.30 mabu: multi-open mechanism integrated into open-entry structure
97Mar09 2.31 mabu: multi-platform mechanism integrated (KEMACH.C module)
97Jul07 2.31 mabu: support of WkNet by WKDOS.EXE (resident driver)
97Aug08 2.31 mabu: version 2.31 released
97Aug12 2.31 mabu: version 2.31a released
98Jan12 2.41 mabu: support for PCCARD by WIBUKEY.SYS/NT (Award CardWare)
98Mar16 2.41 mabu: adding of midlevel driver to support USB interface
98Sep29 2.51 StS:  adaptation to Borland DPMI
99Aug19 2.52 as:   adaptation and extract of the LINUX/UNIX source driver
00Aug04 3.00 mn:   improved adaptation to LINUX/UNUIX
08Oct02 4.30 ha:   add usb support for arm/linux v4l
10Jun24 4.31 ha:   build update
*****************************************************************************/

#ifndef PROJ_H
#define PROJ_H

#include "config.h"

#define VERSION     3
#define SUBVERSION  0

#define VER_DAY     25
#define VER_MONTH   10
#define VER_YEAR    8

/************************/

#define VER_FIRSTYEAR 93

// minimum version for calling driver, required by kernel driver
#define MINDRV_VERSION 2
#define MINDRV_SUBVERSION 31

#define DRV_VERSION 4
#define DRV_SUBVERSION 31
#define DRV_VER_BUILD 85
#define DRV_VER_COUNT 301
#define DRV_VER_YEAR4 2010
#define DRV_VER_MONTH 6
#define DRV_VER_DAY   24


// minimum version for kernel driver, required by calling driver
#define MINKRNL_VERSION 2
#define MINKRNL_SUBVERSION 31


/*****************************************************************************
Compiler Dependencies
==============================================================================

Name of Compiler          preproc. var.  must be specified externally
------------------------  -------------  ----------------------------
Microsoft C/C++ ......... _MSC_VER ..... no

*****************************************************************************/

#if defined(_DOSASM)
  // avoid including of library search BSRxx, WKSxxx and WKDxxx
  #define BSR_NODEFLIB
  #if !defined(_DOS)
    #define _DOS
  #endif
#endif

#if defined(_UNIX)
  #define UNIX   1
  #define _memicmp(p2, p1, cb) memcmp(p2, p1, cb)
#else
  #define UNIX   0
#endif

// =========================
// Get external declarations
// =========================

#if defined(_DEBUG)
  #define BSRDEBUG 1
#else
  #define BSRDEBUG 0
#endif

#if !defined(BSRAPPNAME)
  // set default name
  #define BSRAPPNAME "WKDRV"
#endif

#if defined(_DOSASM)
  #define DOSASM 1
#else
  #define DOSASM 0
#endif

// define type of DOS interupt calling
#define USE_DOSINT 1

// define support of asynchronous mode or not
#if defined(ASYNCMODE)
  #undef ASYNCMODE
#endif
#define ASYNCMODE 0


// define absolute WibuBox addressing timing management
#define ABSTIME 0

// define using of BIOS addresses
#define BIOSADDR 1

// define support multi machine mode (for example IBM PC / NEC PC-98xx switch)
#define MULTIMACH 0

#if MULTIMACH
  // multi machine support: support of IBM-PC and NEC PC-98xx
  #define MACH_IBMPC 1
  #define MACH_NEC98 1
#else
  // no multi machine support: support of IBM-PC only
  #define MACH_IBMPC 1
  #define MACH_NEC98 0
#endif

// ===================================
// declarations to support portability
// ===================================

// other public declarations
#include <stdlib.h>
#include <stdio.h>
#include <stddef.h>
#include <ctype.h>
#include <string.h>
#include <assert.h>
#include <limits.h>
#if !USE_DOSINT
  #include <dos.h>
  #include <fcntl.h>
  #include <share.h>
  #include <io.h>
#endif
#if defined(_PHARLAP)
  #include <pharlap.h>
#endif
#if defined(__APPLE__) || defined(__linux__)
  #include <sys/stat.h>
  #include <fcntl.h>
  #include <semaphore.h>
  #include <errno.h>
#endif


// get public WibuKey declarations
//#include <wk.h>
#include "wk.h"


// ============================================
// Driver specific calling variant declarations
// ============================================

// guarantee that APIENTRY is not used
#if !defined(_MAC) //!P:mabu: Proper setting must be done!
  #undef APIENTRY
#endif

// read all additional WibuKey standard declarations
#include "wkstd.h"

// maximum length of an API transferred string
#define CCH_MAXSTRING 512


#if defined(__APPLE__) || defined(__linux__)
#define GLOBAL_PERMISSIONS   0666

  // definition and declarations for accessing the shared memory (MacOS)
  BOOL WkdEnterCS(sem_t** pidSemaphore, const char* pszSemahore);
  void WkdLeaveCS(sem_t** pidSemaphore);

  extern pthread_mutex_t api_mutex;
  extern sem_t* gThreadSemaphoreID;
  #define THREAD_API_LOCK     pthread_mutex_lock(&api_mutex);
  #define THREAD_API_UNLOCK   pthread_mutex_unlock(&api_mutex);
  #define THREAD_API_INIT     pthread_mutex_init(&api_mutex, NULL);
  #define THREAD_API_EXIT     pthread_mutex_destroy(&api_mutex);
  
#else
  #define THREAD_API_LOCK
  #define THREAD_API_UNLOCK
#endif

/*****************************************************************************

---- Operating System Specific Adaptations -----------------------------------

*****************************************************************************/

#define WkdToUpper(c) toupper(c)

// =========================
// General support functions
// =========================

// special routine to get profile value
UINT WkdGetProfileValue(enum EWkProfileTopic idTopic, const char *pszName,
    char *pszValue, UINT cchValue);


/*****************************************************************************

---- General Driver Management Functions -------------------------------------

*****************************************************************************/

// ==============================
// Driver local memory management
// ==============================

void * WkdAllocMem(size_t cb);

#define USE_KEEPMEM 1

// ===================================
// Management of memory object handles
// ===================================

// type constants of handle addressed memory
enum {
  HTYP_NULL = 0, // 0: no valid handle/empty entry in table
  HTYP_SYS,      // 1: local system handle, corresponds with HWKBSYSTEM
  HTYP_ENTRY,    // 2: local entry handle, corresponds with HWKBENTRY
  HTYP_RMTENTRY, // 3: remote entry handle, corresponds with HWKBENTRY
  HTYP_MEMBLOCK  // 4: memory block, corresponds with HWKBMEM
};


void * WkdCreateHandle(UINT uType, UINT cbSize, HANDLE *phDest);
void * WkdGetHandlePtr(HANDLE h);
UINT WkdGetHandleType(HANDLE h);
ULONG WkdGetHandleData(HANDLE h);
void WkdSetHandleData(HANDLE h, ULONG ul);

void WkdDestroyHandle(HANDLE h);
void WKAPIENTRY WkdCleanUp(void);


// ===================================
// Parsing of driver setting variables
// ===================================

char * WkdCompString(char *pszParse, char *pszPattern);
void WkdSkipSpace(char **ppszParse);
void WkdSkipSep(char **ppszParse);
ULONG WkdReadHexNumber(char **ppszParse);
ULONG WkdReadDecimalNumber(char **ppszParse);
INT WkdGetProfileInt(enum EWkProfileTopic idTopic, const char *pszName);
INT WkdCheckEnvVar(char *pszVarName, char *pszTypeName);

const char *WkdFindEnumName(const char *pszName,
  const char *pszNameList);


// =====================================
// Selected WibuBoxes conflict handling
// =====================================

BOOL WkdIsBoxSelected(const WKBSERIAL *pwkbsr);
void WkdFreeLockedBoxes(void);


void WkdCloseAllOfSubsystem(HANDLE hwkbsys);


// =============================
// User memory buffer management
// =============================

BOOL WkdConvertMemHandle(const void **ppv);


/*****************************************************************************

---- Management of Opened WibuBox entries and WibuBox Encryption -----------

*****************************************************************************/




/*****************************************************************************
O P E N E N T R Y                               Entry of opened WibuBox entry
==============================================================================

This structure defines an opened WibuBox entry. Such a structure is created
by the WkbOpen2() function, destroyed by the WkbClose2() function and used by
functions like WkbQueryEntry2(), WkbSelect2() and WkbCrypt2().

Public Members (copy of WKBOPENENTRY structure)
-----------------------------------------------
flCtrl
    contains information about the type of entry. Following options
    are supported:
      WKB_VERSION1 .....  (0x000000) opened for WibuBox algorithm 1
      WKB_VERSION2 .....  (0x000001) opened for WibuBox algorithm 2
      WKB_VERSION3 .....  (0x000002) opened for WibuBox algorithm 3
      WKB_VERSION4 .....  (0x000003) opened for WibuBox algorithm 4
      WKB_VERSION5 .....  (0x000004) opened for WibuBox algorithm 5
      WKB_OE_MASTER ....  (0x000008) set if opened via master entry
      WKB_OE_SELECTED ..  (0x000010) opened entry currently selected
      WKB_OE_NONETLIMIT   (0x100000) opened with WKB_OPEN_NONETLIMIT
      WKB_OE_GLOBAL ....  (0x200000) opened with WKB_OPEN_GLOBAL
      WKB_OE_EXCLUSIVE    (0x400000) opened with WKB_OPEN_EXCLUSIVE
      WKB_OE_STATIONSHARE (0x800000) opened with WKB_OPEN_STATIONSHARE
hwkbsys
    contains to the handle of the used subsystem. If the value is HWKB_LOCAL,
    then the local system is used.
lPort
    contains the logical port of the opened WibuBox entry. If this value
    is -1, the port is not opened.
pszPort
    points to the name of the logical port of the opened WIBU-BOX entry.
    This entry is compatible to the WKBOPENENTRY structure and not used by
    the OPENENTRY structure.
wkbsr
    contains the serial number of the WibuBox which entry is opened.
usBoxVersion
    contains the version of the WibuBox in the range 1..3
iEntry
    contains the used entry index in the WibuBox in the range 0..9.
    If the entry is not specified (MULTIFIND only), -1 is stored.
ulFirmCode
    contains the Firm Code of the opened WibuBox.
ulUserCode
    contains the User Code of the opened WibuBox.

Internal Members (no part of WKBOPENENTRY structure)
----------------------------------------------------
ice
    (only if INDCRYPT) contains the indirect encryption entry description.
    This member is only defined if the INDCRYPT preprocessor variable is TRUE.
ulSelectCode
    contains the Selection Code for the WkNet access. This member is only
    defined if the NETACC preprocessor variable is TRUE.
fMultiFind
    contains TRUE if the WibuBox entry is opened with WKB_OPEN_MULTIFIND
    and FALSE if not.
fuEntryOpen
    contains the bitmask of all entries which are opened with the WkbOpen2().
    If the WKB_OPEN_FIND or WKB_OPEN_ENTRY command is used, only one bit
    is set, if the WKB_OPEN_MULTIENTRY option, several bits may be set.

!OPENENTRY!
*****************************************************************************/

typedef struct {
  // start of public area (compatible with WKBOPENENTRY structure)
  ULONG flCtrl;
  HWKBSYSTEM hwkbsys;
#if !defined(WK32)
  USHORT usFill; // fill gap to 32 bit compliance
#endif
  LONG lPort;
  char *pszPort;
  WKBSERIAL wkbsr;
  USHORT usBoxVersion;
  SHORT iEntry;
  ULONG ulFirmCode;
  ULONG ulUserCode;
  // end of public area
  BOOLEAN fMultiFind;
  UINT fuEntryOpen;
  UINT uExpDate;
} OPENENTRY; // prefix "ope"

BOOL WkdIsEntryConflict(const OPENENTRY *popeConflict);

/*****************************************************************************

---- Management of Subsystems ------------------------------------------------

*****************************************************************************/

// flag "subsystem specific profile read", set to FALSE by WKB_CTS_RESET
extern BOOL fWkdSubSysProfileRead;


#include <netdb.h>
#include <netinet/in.h>
#include <pthread.h>
#include <sys/socket.h>

// definitions for WkLAN (sockets)
#define SOCKET unsigned int
#define WCHAR USHORT
#define INVALID_SOCKET (SOCKET)(~0)
#define SOCKET_ERROR (-1)
typedef struct sockaddr_in SOCKADDR_IN;
typedef struct sockaddr SOCKADDR;
typedef struct in_addr IN_ADDR;
typedef struct hostent HOSTENT;
// general definitions
#define _tcslen strlen

#if USE_USB

  // Vendor ID of the WIBU-SYSTEMS AG, Germany
  #define WK_VENDOR_ID       0x064f
  // Product ID of the WibuBox/U
  #define WK_PROD_ID_BOXU    0x0bd7
  // Product ID of the WibuBox/RU
  #define WK_PROD_ID_BOXRU   0x0bd8
  // Product ID of the WibuBox/RU (SecuriKey???)
  #define WK_PROD_ID_BOXS    0x0bd9


#define METHOD_BUFFERED    0
#define CTL_CODE(DeviceType, Function, Methode, Access) \
    (((DeviceType) << 16) | ((Access) << 14) | ((Function) << 2) | (Methode))

#define FILE_DEVICE_WIBUKEY 0x00008200
#define WK_IOCTL_INDEX 0xA00

enum {
  WKCMD_GET_VERSION = 0,
  WKCMD_EXECUTE = 1,
  WKCMD_GET_CALLVERSION = 2,
  WKCMD_GET_DEVICE_LIST = 3,
  WKCMD_INVALIDATE_DEVICE = 4,
  WKCMD_GET_API_CALLING_ENTRY = 5
};

#define FILE_READ_DATA     0x0001
#define FILE_WRITE_DATA    0x0002
#define FILE_SHARE_READ    0x00000001
#define FILE_SHARE_WRITE   0x00000002
#define GENERIC_READ       0x80000000
#define GENERIC_WRITE      0x40000000

#define IOCTL_WK_GET_VERSION ((UINT32)CTL_CODE(FILE_DEVICE_WIBUKEY,\
   WK_IOCTL_INDEX + WKCMD_GET_VERSION,\
   METHOD_BUFFERED, FILE_READ_DATA))

#define IOCTL_WK_GET_CALLVERSION ((UINT32)CTL_CODE(FILE_DEVICE_WIBUKEY,\
   WK_IOCTL_INDEX + WKCMD_GET_CALLVERSION,\
   METHOD_BUFFERED, FILE_READ_DATA|FILE_WRITE_DATA))

#define IOCTL_WK_GET_DEVICE_LIST ((UINT32)CTL_CODE(FILE_DEVICE_WIBUKEY,\
   WK_IOCTL_INDEX + WKCMD_GET_DEVICE_LIST,\
   METHOD_BUFFERED, FILE_READ_DATA|FILE_WRITE_DATA))

#define IOCTL_WK_EXECUTE ((UINT32)CTL_CODE(FILE_DEVICE_WIBUKEY,\
   WK_IOCTL_INDEX + WKCMD_EXECUTE,\
   METHOD_BUFFERED, FILE_READ_DATA|FILE_WRITE_DATA))

#define IOCTL_WK_INVALIDATE_DEVICE ((UINT32)CTL_CODE(FILE_DEVICE_WIBUKEY,\
   WK_IOCTL_INDEX + WKCMD_INVALIDATE_DEVICE,\
   METHOD_BUFFERED, FILE_READ_DATA|FILE_WRITE_DATA))

typedef struct {
  UINT16 usVersion;
  UINT16 usSubVersion;
  UINT16 usBuild;
  UINT16 usCount;
} BSRMODVERSION; // prefix "bsrmver"

#endif // USE_USB


// type for dummy remote system
typedef void RMTSYSTEM;


/*****************************************************************************

---- Communication between Caller Driver and Kernel Driver -------------------

*****************************************************************************/

// convert HANDLE32 to type strict 16/32 bit handle
#define GETHANDLE32(h, t) ((t)(UINT)(ULONG)h)


/*****************************************************************************

---- WibuKey Subsystem Management -------------------------------------------

*****************************************************************************/

enum {
  STYPE_LOCAL = WKB_ACC_LOCAL,
  STYPE_WKLAN = WKB_ACC_WKLAN,
  STYPE_WKNET = WKB_ACC_WKNET,
  STYPE_KERNEL = 0x80
};

#define STYPE_REDIR (STYPE_WKLAN|STYPE_KERNEL)

#define SFLAG_NONETLIMIT    0x01
#define SFLAG_EXCLUSIVE     0x02
#define SFLAG_STATIONSHARE  0x04

/*****************************************************************************
S Y S E N T R Y                             Entry of opened WibuKey subsystem
==============================================================================

This structure is returned by the WkbQuerySystem2 function and specifies
partially the internal status of a WibuKey subsystem. The first entry
specifies with one the WKB_ACC flag the type of the subsystem, the further
members of the structure depends on this type.

Members
-------
fuType
    contains system specific status flags and is a combination of the
    following values:
        STYPE_LOCAL   (0x01) the specified system addresses the WibuBox
                      directly and locally.
        STYPE_WKLAN   (0x02) the specified system addresses the WibuBox
                      via the WkLAN network.
        STYPE_WKNET   (0x04) the specified system addresses the WibuBox
                      via the WkNet network.
        STYPE_KERNEL  (0x80) the specified system addresses the WibuBox
                      locally via the remote redirector and a separate
                      kernel driver.
fuFlags
    contains flags which control the using of the opened subsystem. Currently
    following flags are possible:
        SFLAG_NONETLIMIT . (0x01) is set when the WKB_ACC_NONETLIMIT option is
                      specified in the WkbAccess2() function.
flStdCtrl
    contains the standard control flags which are specified in the WKBACCESS
    structure of the WkbAccess2() call and which are used for all functions
    which WKB_EXCLMODE flag is not set.
lFirmCode
    contains the Firm Code which was used in the WKBACCESS structure.
lUserCode
    contains the User Code which was specified in the WKBACCESS structure.
hwkbsysRmt
    contains the handle of the remote side subsystem. This can be a WkLAN
    network server or a separate kernel driver.

The following four members are used only in a kernel driver.

hwkbeGlobal;
    contains the handle of the globally opened WIBU-BOX entry which is a
    handle, returned by WkbOpen2().
usPort
    contains the port at which the WibuBox is connected in which the
    subsystem entry is stored.
usEntry
    contains the WibuBox entry index (0..9) which is used for the subsystem.
wkbsr
    contains the serial number of the WibuBox with the used subsystem entry.
fMaster
    contains TRUE if the subsystem contains a Master Code and FALSE if it
    is contain a standard User Code.

The following two pointers are stored in a union because they are never used
at the same time:

plansys
    points to a subsystem which addresses a WkLAN network system.
pnetsys
    points to a subsystem which addresses a WkNet network system.

!SYSENTRY!
*****************************************************************************/

typedef struct {
  // prefix "sys"
  UINT fuType;
  UINT fuFlags;
  ULONG flStdCtrl;
  LONG lFirmCode;
  LONG lUserCode;
  // the following members are no part of WKBSYSTEM
  HWKBENTRY hwkbeGlobal;
  USHORT usPort;
  USHORT usEntry;
  WKBSERIAL wkbsr;
  BOOL fMaster;
} SYSENTRY;

#define STDFNFLAGS 0xFFF00  // standard function flags in <flStdCtrl>
#define STDSTFLAGS 0xF0000  // standard status flags


/*****************************************************************************

---- Kernel Driver Specific Declarations -------------------------------------

*****************************************************************************/

// general bits for addressing hardware within a byte
#define BIT7  0x80
#define BIT6  0x40
#define BIT5  0x20
#define BIT4  0x10
#define BIT3  0x08
#define BIT2  0x04
#define BIT1  0x02
#define BIT0  0x01

// WIBU-BOX commands
#define BOXCMD_RESET    0x00
#define BOXCMD_FIND     0x01
#define BOXCMD_SELECT1  0x02
#define BOXCMD_CRYPT    0x03
#define BOXCMD_FSEARCH  0x04
#define BOXCMD_NSEARCH  0x05
#define BOXCMD_LIST     0x06
#define BOXCMD_PROGRAM  0x07
#define BOXCMD_SELECT2  0x08


/*****************************************************************************
P O R T A D D R                                          Physical Port Address
==============================================================================

This structure specifies a true physical port address and reflects the
different schemes of the supported operating systems:

* DOS, 16 bit Windows and 16 bit DOS extenders specify a 16 bit port address
  which is directly used for the port addressing.
* Windows 95 VxD and 32 bit DOS extenders specify a 32 bit port address value
  which is directly used for the port addressing.
* Windows NT specifies a pointer to a byte which is determined by the function
  MmMapIoSpace().

*****************************************************************************/

typedef UINT PORTADDR; // prefix "pta"


/*****************************************************************************
P O R T S P E C                             Logical Port Address Specification
==============================================================================

This structure specifies a logical port address and reflects the different
addressing schemes of the supported operating systems:

* DOS, 16 bit Windows and 16 bit DOS extenders specify a 16 bit port address
  which is directly used for the port addressing.
* Windows 95 VxD and 32 bit DOS extenders specify a 32 bit port address value
  which is directly used for the port addressing.
* Windows NT specifies a 64 port value, a bus type and a bus index which
  are converted to an addressing pointer.
* Windows 95 supports the proper port acquiring. The driver stores the
  contention function and the resource identification which are required for
  this acquiring. Also the acquiring identification itself is stored 

Members
-------
pta
    contains the physical address which is used for the addressing. Under
    Windows NT this is a mapped memory pointer to the port hardware. Other-
    wise, the port address itself is used. The Pc Card driver stores only
    the logical address, the physical address is not stored here.
ulSpecAddress
    contains under Windows NT the lower 32 bits of the specified logical
    address from the system registry hardware description or from the user
    set environment.
adrSystem
    contains under Windows NT the system logical address, determined by the
    function HalTranslateBusAddress() from the specified logical address and
    the bus description.
bBusType
    contains under Windows NT as part of the bus description the bus type.
bBusIndex
    contains under Windows NT as part of the bus description the bus index,
    relative to 0.
idResource
    contains under Windows 95 the resource identification for the proper
    port acquiring. If this value is 0, the port is not supported for
    proper port acquiring.
pfnContention
    contains under Windows 95 the address of the a function to gain the
    port acquiring. This value is only valid if <idResource> does not contain
    a 0 value.
idAcquire
    contains under Windows 95 the acquiring constant which is used in the
    port release operation. This value is only valid if <idResource> does not
    contain a 0 value. If <idAcquire> itself is 0, the port was not acquired.

*****************************************************************************/

typedef struct {
  PORTADDR pta;
} PORTSPEC; // prefix "pts"


/*****************************************************************************
R E A D / W R I T E _ P O R T _ B Y T E          Physical addressing functions
==============================================================================

These macro functions implements the low level functions to address the port
hardware. Under Windows NT, these functions are mapped to corresponding kernel
DDK macros. Otherwise, intrinsic port addressing functions of the extended,
Microsoft specific, C library are used.

*****************************************************************************/

#ifndef UNIX
  #include <conio.h>  // _inp(), _outp()

    #define READ_PORT_BYTE(Port)        ((BYTE)_inp((USHORT)(Port)))
    #define WRITE_PORT_BYTE(Port, Byte) _outp((USHORT)(Port), (BYTE)(Byte))


#endif // #ifndef UNIX


/*****************************************************************************
E N A B L E / D I S A B L E _ I N T E R R U P T  Interrupt Switching Functions
==============================================================================

These macro functions implements enabling and disabling of the processor
specific interrupt flag. They are implemented as inline assembler functions
or, if interrupt switching is disabled, as empty functions.

*****************************************************************************/

// macros for enabling and disabling interrrupts
// no interrupt switching: no code generation
#define ENABLE_INTERRUPT()
#define DISABLE_INTERRUPT()



/*****************************************************************************
B I O S _ P T R                               Macro for BIOS memory addressing
==============================================================================

This macro converts a specified linear address to a far pointer which can be
used to address the data located at this address.

Parameters
  usAddr  contains the offset in the BIOS area (start addrex 0x400) in the
          range from 0x000..0x3FF.

*****************************************************************************/

#if defined(WK32)
  // 32 bit DOS extender: flat pointer to BIOS table
  #define BIOS_PTR(usAddr) ((USHORT*)(0x400 + (usAddr)))
#elif defined(_WINDOWS)
  // 16 bit Windows: set only offset, add later special segment index __0040H
  #define BIOS_PTR(usAddr) (MAKELP(0, (usAddr)))
#else
  // DOS: far pointer to BIOS table
  #define BIOS_PTR(usAddr) ((USHORT _far *)(ULONG)(0x400 + (usAddr)))
#endif


/*****************************************************************************
D E V O B J E C T                                     Device Object List Entry
==============================================================================

This structure contains a device object of the list of found WibuBox devices.

Members
-------
pNext
    points to the next device entry in the list. If this pointer is NULL if
    the end of the list is reached.
wkbsr
    contains the serial number of the WibuBox.
wkdevobj
    contains the device object member.

*****************************************************************************/

typedef struct _DEVOBJECT {
  // prefix "devobj"
  struct _DEVOBJECT *pNext;
  WKBSERIAL wkbsr;
  WKDEVOBJ wkdevobj;
} DEVOBJECT;


/*****************************************************************************
R E Q O P T _ C R Y P T            Class Driver Crypt Request Option Structure
==============================================================================

This structure specifies the request option structure for the BOXCMD_CRYPT
operation.

Members
-------
cbEncrypted
     receives the number of bytes which are encrypted correctly during the
     class driver function execution. This value is only filled if the
     function failed. As default, 0 is stored.

*****************************************************************************/

typedef struct {
  UINT cbEncrypted;
} REQOPT_CRYPT;


/*****************************************************************************
R E Q O P T _ L I S T               Class Driver List Request Option Structure
==============================================================================

This structure specifies the request option structure for the BOXCMD_LIST
operation.

Members
-------
iEntry
    contains the index of an WibuBox entry (range 0..9) if only the contents
    of this entry and possibly its added entry is read. If the complete
    WibuBox is read, the complete request option structure is not set.

*****************************************************************************/

typedef struct {
  INT iEntry;
} REQOPT_LIST;


/*****************************************************************************
R E Q O P T _ P R O G R A M      Class Driver Program Request Option Structure
==============================================================================

This structure specifies the request option structure for the BOXCMD_PROGRAM
operation.

Members
-------
fNoStatusHoldChk
    contains the "ignore hold-error in reading status byte" flag.
    This flag is set whenever a Limit Counter is added or modified. It fixes
    a bug of the WibuBox-ASIC which occurres when a Limit Counter is added
    or modified: The WibuBox does not wait in this situation that the driver
    sets to the sleep mode. In constrast, it goes immediately into this mode
    after sending the status byte.
fSetSerialNumber
    contains the "serial number change" flag. The USB class driver invalidates
    the device in the device list of the plug & play device to enforce that
    all calling driver reread the serial number.

*****************************************************************************/

typedef struct {
  BOOL fNoStatusHoldChk;
  BOOL fSetSerialNumber;
} REQOPT_PROGRAM;


/*****************************************************************************
R E Q O P T I O N S          Additional Class Driver Request Options Structure
==============================================================================

This structure contains additional request options which are transferred to
<pfClDrvRequest> function.

Members
-------
Crypt
    contains additional information for the BOXCMD_CRYPT operation.
List
    contains additional information for the BOXCMD_LIST operation.
Program
    contains additional information for the BOXCMD_PROGRAM operation.

*****************************************************************************/

typedef union {
  REQOPT_CRYPT Crypt;
  REQOPT_LIST List;
  REQOPT_PROGRAM Program;
} REQOPTIONS; // prefix "reqopt"


/*****************************************************************************
C L A S S D R I V E R                      Middle Level Class Driver Structure
==============================================================================

This structure is used to describe the interface to the middle level drivers.
Each middle level driver type has its own description via this structure.

Members
-------
pfClDrvReset
    points to the function which is called to reset the class driver. This
    reset forces the rereading of profile information etc.
pfClDrvSelect
    points to the function which is called to select a specified port.
pfClDrvRelease
    points to the function which is called to release the last selected
    port.
pfClDrvGetDeviceList
    points to the function which gets a list of found WibuBox devices at
    the last selected port.
pfclDrvGetDeviceFromSerial
    returns a device object which match with the specified serial number.
pfClDrvFindDevice
    points to the function which finds a device which contains the specified
    Firm and User Code.
pfClDrvRequest
    points to the function which executes a WibuBox communication request
    program.
pfClDrvPortData
    points to the function which reads or updates the low level port data
    of the last selected port.
pfClDrvCleanUp
    cleans up the class driver before the driver is terminated. Especially
    all dynamically allocated memory is freed.
cbMaxCrypt
    contains the maximum length of a direct encryption.

*****************************************************************************/

typedef struct {
  void (*pfClDrvReset)(INT nPort, UINT fuCtrl);
  BOOL (*pfClDrvSelect)(INT nPort);
  UINT (*pfClDrvRelease)(void);
  UINT (*pfClDrvGetDeviceList)(DEVOBJECT **ppdevobj);
  DEVOBJECT *(*pfClDrvGetDeviceFromSerial)(const WKBSERIAL *pwkbsr);
  BOOL (*pfClDrvFindDevice)(OPENENTRY *pope);
  BOOL (*pfClDrvRequest)(WKDEV_REQUEST *pwkdevreq,
    BYTE *pabDest, REQOPTIONS *preqopt);
  BOOL (*pfClDrvPortData)(WKBPORT *pwkbprt, BOOL fSet);
  void (*pfClDrvCleanUp)(INT nPort);
  UINT cbMaxCrypt;
} CLASSDRIVER; // prefix "clsdrv"


/*****************************************************************************
L O W D R I V E R                               Interface for low-level driver
==============================================================================

This structure is used to describe the interface to the low level drivers.
Each low level driver type has its own description via this structure.

Members
-------
cVariant
    contains the number of variants of the driver. This value is at least
    1 and controls the counting of the second parameter of the LowSelectPort()
    function.
cbCluster
    contains the number of bytes which can be sent and read as one cluster
    within the cluster low level communication. Normally this value is 1
    (no clustering available). For a buffered communication like with the
    WibuBox/A, the value may be higher, at most 256.
pfLowReset
    points to the low level routine which resets the specified port or the
    complete low level driver.
pfLowGetPortData
    points to the low level routine which returns driver and port specific
    information.
pfLowAccessPort
    points to the low level routine which accesses the specified port.
pfLowReleasePort
    points to the low level routine which releases the specified port.
pfLowWakeUp
    points to the low level routine which wakes up the WibuBoxes at the
    specified port.
pfLowSendByte
    sends a byte to the WibuBoxes at the specified port.
pfLowReadByte
    reads a byte from a WibuBox at the specified port.
pfLowLookAhead
    specifies read and release operations in near future. This function is
    used only for the buffer-orientated communication, for example for the
    WibuBox/A.
pfLowWaitReady
    waits until the WibuBox is ready at the specified port.
pfLowSetup
    setups the port with the specified initialization data of type WKBPORT.

All functions returns a WibuKey API error code or 0 if no error occured.

*****************************************************************************/

typedef struct {
  INT  cVariant;
  INT  cbCluster;
  UINT (*pfLowReset)(INT nPort, ULONG flFlags);
  UINT (*pfLowGetPortData)(INT nPort, void *pvDest, INT idType);
  UINT (*pfLowAccessPort)(INT nPort, INT nVariant);
  UINT (*pfLowReleasePort)(INT nPort);
  UINT (*pfLowWakeUp)(INT nPort);
  UINT (*pfLowSendByte)(INT nPort, UINT uData);
  UINT (*pfLowReadByte)(INT nPort, UINT *puDest);
  UINT (*pfLowLookAhead)(INT nPort, UINT cbRead, UINT fuCtrl);
  UINT (*pfLowWaitReady)(INT nPort, UINT uTimeOut);
  UINT (*pfLowSetup)(INT nPort, const WKBPORT *pwkbpt);
} LOWDRIVER; // prefix "lowdrv"

// constants for the <fuCtrl> control argument of LowLookAhead()
#define LOW_RELEASE  1 // release call will follow without further send
#define LOW_CLUSTER  2 // specifies that the send/write is clustered

// special command codes of <cbDest> of LowGetPortData()
#define GLPD_CHECK     0
#define GLPD_WKBPORT   1
#define GLPD_STATUS    2
#define GLPD_OPTIONS   3
#define GLPD_ADDRESS   4
#define GLPD_CTRLFLAGS 5

// constants which are returned by LowGetPortData()/GLPD_OPTIONS
#define OPT_AUTOECHO  1
#define OPT_ABSTIMING 2


/*****************************************************************************
P O R T G R O U P                    WibuKey Port Group Description Structure
==============================================================================

This structure describes a specific WibuKey port or a port group which
manages one or more WibuKey devices (WibuBoxes).

The structure contains the name, the port number/range, the configuration name
string and the pointers to the low-level and the mid-level drvier description.

Members
-------
nFirstPort
    contains a number which specifies the first number of a group of
    successive ports in range 0..127.
nLastPort
    contains a number which specifies the last number of a group of
    successive ports in range 0..127. If this value is identical with
    <nFirstPort>, the port is no group. Such a port is managed by one
    fix port number and the port name is not extended by an index value.
pszName
    points to the name which leads the name of all ports of the specified
    drivers.
pszConfigNames
    points to the list of names for the configuration bits.
pclsdrv
    points to the description of the class driver which is used to handle
    this port.
plowdrv
    points to the low level driver description if the class driver requires
    a low level driver description. Otherwise this pointer may be NULL.

*****************************************************************************/

typedef struct {
  INT  nFirstPort;
  INT  nLastPort;
  char *pszName;
  char *pszConfigNames;
  CLASSDRIVER *pclsdrv;
  LOWDRIVER *plowdrv;
} PORTGROUP; // prefix "prtgrp"


// ===================================
// Driver Description Data Declaration
// ===================================
// The definition of the structures is stored in module KECOMMON.C

#define C_PARALLEL_PORT 4  // number of supported parallel ports
#define C_SERIAL_PORT   4  // number of supported serial ports
#define C_PCCARD_SOCKET 6  // number of supported PC Card sockets


// declaration of class devices
// unused CLASSDRIVER clsdrvWkdSeq;  // XX: support for WibuBox/P[36],/SP,/SX,/M
extern CLASSDRIVER clsdrvWkdSrl;  // 16-19: support of WibuBox/ST (SRL)
// unused CLASSDRIVER clsdrvWkdAdb;  // 40: support of WibuBox/A (ADB)
extern CLASSDRIVER clsdrvWkdUsb;  // 48: support of WibuBox/U,/RU (USB)

// declaration of low level drivers, used by <clsdrvWkdClassic>
#if MACH_NEC98
extern LOWDRIVER lowdrvWkdLpt;    // 0: bit communication at LPT ports
extern LOWDRIVER lowdrvWkdLptNec; // 0: bit communication at NEC LPT port
extern LOWDRIVER lowdrvWkdCom;    // 8: bit communication at COM ports
extern LOWDRIVER lowdrvWkdComNec; // 8: bit communication at NEC COM port
#endif //  MACH_NEC98
//unused extern LOWDRIVER lowdrvWkdSrl;    // 8: serial communication at COM ports
//unused extern LOWDRIVER lowdrvWkdPcc;    // 32: bit communication in PcCard slots

// the following driver will be removed alter
//unused extern LOWDRIVER lowdrvWkdAdb;    // 40: buffer communication at ADB port

// declaration of all supported port group descriptions
//unused PORTGROUP prtgrpWkdLpt;  // "LPTx" port (WibuBox/P[36],/RP)
//unused PORTGROUP prtgrpWkdCom;  // "COMx" port (WibuBox/SP,/SX,/ST)
extern PORTGROUP prtgrpWkdSrl;    // "SRL" port (WibuBox/ST)
//unused PORTGROUP prtgrpWkdPcc;  // "PCCARDx" port (WibuBox/M)
//unused PORTGROUP prtgrpWkdAdb;  // "ADB" port (WibuBox/A)
extern PORTGROUP prtgrpWkdUsb;    // "USB" port (WibuBox/U,/RU)

// declaration of port group description array
extern PORTGROUP * apprtgrpWkd[];
extern INT gcWkdPortGroup;


// ====================================================
// Converting of System Addressing Functions and Macros
// ====================================================

// dummy functions: returns parameter
#define WkdGetBiosPtr(p) ((void*)(p))

#if ABSTIME

// =================================
// absolute time accessing mechanism
// =================================
// The time to address the WibuBox is delayed by timing loops
// which bases on absolute microseconds value, independent of the T value
// of a connected WibuBox. This technique is used in NT technology and Windows
// 9x kernel drivers. It is assumed that the T value is not longer than 50
// microseconds.

// Windows NT: base is 1 microseconds
#define STALL_T  50

// stall values, relative to T
#define STALL_0P125_T  ((STALL_T + 7) / 8)
#define STALL_0P25_T   ((STALL_T + 3) / 4)
#define STALL_0P33_T   ((STALL_T + 2) / 3)
#define STALL_0P50_T   ((STALL_T + 1) / 2)
#define STALL_1P0_T    (STALL_T)
#define STALL_2P0_T    (2 * STALL_T)
#define STALL_3P0_T    (3 * STALL_T)
#define STALL_3P5_T    (3 * STALL_T + (STALL_T + 1) / 2)

// absolute value: 100 microseconds
#define STALL_100  (2*50)

// own delay function for IBM PC compatible system
void WkdStallExecution(UINT idDelay);

#ifndef STALL
  #define STALL(Delay) WkdStallExecution(Delay)
#endif

#ifndef DELAY
  #define DELAY(Port, TDelay, AbsDelay) STALL(AbsDelay)
#endif

// extern UINT guWkdAcquireWait; // contains max. delay for port acquiring

#else

// global variable for setting determined system time
extern UINT guWkdSysT;

// variable for setting minimum T value
#if !ABSTIME
extern INT gnWkdMinT;
#endif

// functions to determine system timing & T value relative timing delay
UINT WkdGetSysTime(PORTADDR ptaSync);
void WkdDelayTime(PORTADDR ptaSync, UINT uCycles);

#ifndef DELAY
  #define DELAY(Port, TDelay, AbsDelay) WkdDelayTime(Port, TDelay)
#endif

#endif // #else #if ABSTIME


/*****************************************************************************
L O W S T A T U S                           Low Level Driver Status Management
==============================================================================

This structure contains the public part of the low level driver port.
It is used to exchange data between the low level part and the high level
part of the driver. Some members are only specified if the T value timing
management variant is used. If the WkbControlSystem2 function with
WKB_CTS_RESET and WKB_RESET_TIMING is called, the complete status structure
of every low level driver port is set to 0.

Members
-------
uT
    contains the current used T value. This value is normally determined from
    the T cycle of the WibuBox but, for the first access, this value can
    determined from the system timer.
uRefT
    contains the T value which was determined on the previous WakeUp() cycle.
    This value is never modified by the low level driver itself but only by
    the WakeBox() function in CTRL.C. This value is 0, if no reference value
    is determined.
fuStatus
    contains flags which specify the status of the port. The lower 8 bits
    specify up to 8 invalid variants of the low level driver. Bit 8 up to 11
    are reserved and must be set to 0.
    The other bits has following meaning:
      15  (LWST_SELECTED)
          is 1 if the port is selected and 0 if not.
      14  (LWST_FIRSTWAKEUP)
          is 1 if the WakeUp is called at first after the selection.
      13  (LWST_GOODWAKEUP)
          is 1 if the port was previously successfully waken up. This state
          suppresses the creation of static diagnosis event records.
      11  (LWST_MINT)
          is 1 if the minimum T value is stored into the diagnosis and FALSE
          if not.
pdevobjeList
    points to the list of read WibuBox device objects (serial numbers).

*****************************************************************************/

typedef struct {
#if !ABSTIME
  UINT uT;
  UINT uRefT;
#endif
  UINT fuStatus;
  DEVOBJECT *pdevobjList;
} LOWSTATUS; // prefix "lwst"

#define LWST_SELECTED      0x8000
#define LWST_FIRSTWAKEUP   0x4000
#define LWST_GOODWAKEUP    0x2000
#define LWST_MINT          0x1000
#define LWST_INVALID_MASK  0x00FF


// ===========================================
// Standard kernel driver management functions
// ===========================================

// proper read port function to avoid polls
BYTE WkdReadPortProperly(PORTADDR pta, BYTE bMask);

#if MULTIMACH
void WkdDefineLowDriver(INT nPort, LOWDRIVER *plowdrv);

// flag "machine variant is checked"
extern BOOL gfWkdMachineChecked;

#endif // #if MULTIMACH

BOOL WkdSelectPortGroup(INT nPort);
BOOL WkdSelectPort(INT nPort, BOOL fForAccess);
BOOL WkdReleasePort(void);
BOOL WkdGetPortName(char *pszPort, UINT cbLen, UINT *pcch);
void WkdConvertStatusIntoError(UINT uData);
void WkdResetDriver(UINT fuReset);
const char * WkdParsePortSpec(const char *pszPort, INT *pnPortDest);
void WkdResetPortEnumeration(void);
BOOL WkdSelectNextPort(BOOL fForAccess);
BOOL WkdAccessPortSpec(const char *pszPort);
BOOL WkdSetBoxEntry(const BYTE *pbBox, BYTE *pbBoxAdd,
    INT nVersion, INT nIndex, WKBBOXENTRY *pwkbboxeDest, BOOL8 fIsFC0Allowed);
BOOL WkdReadPortAddr(char **ppszParse, PORTSPEC *pptsDest);
void WkdQueryEnvPorts(char *pszVarName, char *pszOptionName,
  PORTSPEC *paptsDest, UINT *pauOptions, INT cDestBuf);



/*****************************************************************************
---- LINUX/UNIX specific definitions  -------------------------------------
*****************************************************************************/

#if defined(_UNIX)
  BOOL8 AccessCOM (INT nPort, INT nDivisor);  //declaration for drvtest.c RF
  BOOL8 ReleaseCOM (INT nPort); //declaration for drvtest.c RF
  INT WkdQueryEnvWkCom(char *pszVarName);

enum {
  /* ids for Array of ENVS. prefix 'ENV_'  */
  ENV_PORTNAME = 0, /* WKVAR_PORTNAME   */
  ENV_COM,          /* WKVAR_COM        */
  ENV_SYSTEM,       /* WKVAR_SYSTEM     */
  ENV_HOSTNAME,     /* WKVAR_HOSTNAME   */
  ENV_SERVERNAME,   /* WKVAR_SERVERNAME */
  ENV_IPPORT,       /* WKVAR_IPPORT     */
  ENV_BAUDRATE,     /* WKVAR_BAUDRATE   */
  ENV_PORTENUM,     /* WKVAR_PORTENUM   */
  ENV_PORT,         /* WKVAR_PORT       */
  ENV_LPT,          /* WKVAR_LPT        */
  ENV_LANG,         /* WKVAR_LANG       */
  ENV_TIMEOUT       /* WKVAR_TIMEOUT    */
};

// This is a trial to store Environment Variables globally (as a kind of Hash-Set)
typedef struct {
  INT   id;           /* This might be obsolete, but :-) */
  char *pszName;     /* Environment Variables' Names */
  char *pszValue;    /* The apropriate Values */ //!C:mn Has to be declared dynamically!
  const char *pszDefValue; /* The Default Value */
} ENVS;//prefix "ev"

/* global Array of EnvironmentVariables, defined in kesrl.c:244 */
/* it stores the Values from Constants below */
extern ENVS gaevWK[];

/*
 * This should be the solely place to name
 *   WK-Environment-Variables for Linux
 */
// prefix  WKVAR_
#define WKVAR_PORTNAME   "WKPORTNAME"
#define WKVAR_COM        "WKCOM"
#define WKVAR_SYSTEM     "SUBSYSTEM"
#define WKVAR_HOSTNAME   "HOSTNAME"
#define WKVAR_SERVERNAME "WKSERVERNAME"
#define WKVAR_IPPORT     "IPPORT"
#define WKVAR_BAUDRATE   "BAUDRATE"
#define WKVAR_PORTENUM   "WKPORTENUM"
#define WKVAR_PORT       "WKPORT"
#define WKVAR_LPT        "WKLPT"
#define WKVAR_LANG       "WKLANG"
#define WKVAR_TIMEOUT    "CONNECTTIMEOUT"

/*  ******* Standard portname, if no enviromentvariable is defined.
 *  "ttyS" ist used for LINUX (formerly "cua")
 *  Remind: The portnumber (e.g. in 'ttyS0' the '0') is set later!
 */
// prefix  DEFAULT_

#if LINUX
  #define DEFAULT_PORTNAME   "/dev/ttyS"
  #define DEFAULT_BAUDRATE   "19200"
#elif AIX
  #define DEFAULT_PORTNAME  "/dev/tty"
  #define DEFAULT_BAUDRATE   "19200"
#else
  #define DEFAULT_PORTNAME  "/dev/cua"
  #define DEFAULT_BAUDRATE   "19200"
#endif

#define DEFAULT_COM        "*,*,/D,/D"
#define DEFAULT_SYSTEM     "Kernel,WkLan"
#define DEFAULT_HOSTNAME   "localhost"
#define DEFAULT_SERVERNAME "localhost"
#define DEFAULT_IPPORT     "22347"

#define DEFAULT_PORTENUM   "DIGIT"
#define DEFAULT_PORT       "Com,USB"
#define DEFAULT_LPT        "/D,/D,/D,/D"
#define DEFAULT_LANG       "FALSE"
#define DEFAULT_TIMEOUT    "10"


  extern BOOL gfPortsChecked;
  extern BOOL gafPorts[C_SERIAL_PORT];
  extern char gszDevName[30];          // empty preset global Un!x/Linux-DeviceName

  BOOL GetComputerName2(char *pszDest, UINT *pcchDest);
#endif




// ======================================================
// System Specific Kernel Addressing Functions and Macros
// ======================================================

void WkdQuerySystemPorts(PORTSPEC *aptsDest, INT cDestPort,
  USHORT *pusSysAddr, INT cSysPort);
// port mapping outside of Windows NT: no conversation: valid if addr. not 0
#define WkdMapPort(ppts, cb, fIoSpace) ((INT)(ppts)->pta != 0)

#define QUERY_SYSTEM_PORTS(aptsDest, cDestPort, idDev, pusSysAddr, cSysPort)\
  WkdQuerySystemPorts(aptsDest, cDestPort, pusSysAddr, cSysPort)

#if ASYNCMODE

/*****************************************************************************

---- Management of Asynchronous States (Action Handling) ---------------------

*****************************************************************************/

/*****************************************************************************
A S Y N C A C T I O N                        Local system management structure
==============================================================================

This structure contains an asynchronous state which can be read via
WkbIsComplete or similiar functions.

Members
-------
pNext
    points to the next action in the list. The end of the list is specified
    by NULL.
ppPrev
    points to the <pNext> member of the predecessor action in the list.
    This pointer is required to remove a single action from the action list.
psys
    points to the subsystem which created the action.
idCode
    contains the action code of the current action.
idState
    contains the current state of the action. This is one of the following
    actions:
      ACT_NONE ... the action is invalid.
      ACT_RUN .... the action is running.
      ACT_COMPLETE the action has completed.
idError
    contains the error code of a completed action.
idReturn
    contains the return value of a completed action.
hwkbsysDestroy
    contains the handle of the subsystem which is released (= destroyed)
    in the WkbIsComplete() function as a deferred WkbRelease2() operation.

WkNet WkbSelect2()/WkbCrypt2() specific members [only if NETACC]
----------------------------------------------------------------
Crypt.pbDest
    points to the destination which receives the encrypted data finally,
    including skipped gap-map data.
Crypt.pcbDest
    points to an UINT variable which receives the number of encrypted bytes
    or is NULL.
Crypt.fuDestCtrl
    contains the flags which control the storing of the encrypted
    data. These are one of the WKB_AREA_MASK flags or WKB_AREA_NONE.
Crypt.pbTempDest
    points to the destination which is written with the encrypted WkNet data.
    This pointer is not valid if <fuDestCtrl> has set the WKB_AREA_NONE flag.
Crypt.pabBuffer
    points to an optional buffer which receives the WkNet data. If this
    pointer is NULL (the default), no such buffer is used and the data
    are stored directly at <pcbDest>. This buffer is automatically freed
    whnenver an ASYNCACTION object is freed.
Crypt.cbEncrypted
    contains the number of bytes which must be encrypted.
Crypt.cbDest
    contains the number of bytes which is stored in the destination,
    optionally with skipped gap-map-controlled data.
Crypt.uNewCrc
    contains the calculated CRC value of the source data which stored back
    into the WKBAREA structure.
Crypt.pwkbar
    points to a WKBAREA structure which contains additional flags and
    encryption information or is NULL if no area structure is specified.
IndSelect.uType
    contains the type of the indirect selection (one of the WKB_SEL_...
    algorithm constants). If this value is 0, no indirect selection is
    active [only if INDCRYPT].
IndSelect.pwkbsel
    points to the WKBSELECT structure which controls the indirect encryption
    [only if INDCRYPT].
IndSelect.pabBuffer
    points to a buffer which is dynamically allocated and contains the
    data bytes for the initialization of the indirect selection which are
    encrypted [only if INDCRYPT].
IndSelect.pwkbar
    points to a dynamically allocated WKBAREA structure which contains
    additional information for the encryption of the indirect selection
    initialization. If no information is required, this pointer is set
    to NULL [only if INDCRYPT].
IndCrypt.pwkbar
    points to a WKBAREA structure which contains additional flags and indirect
    encryption information or is NULL if no area structure is specified [only
    if INDCRYPT].
IndCrypt.pbDest
    points to the destination which receives the indirect encrypted data
    finally, including skipped gap-map data [only if INDCRYPT].
IndCrypt.pcbDest
    points to an UINT variable which receives the number of encrypted bytes
    or is NULL [only if INDCRYPT].
IndCrypt.fuDestCtrl
    contains the flags which control the storing of the indirectly encrypted
    data. These are one of the WKB_AREA_MASK flags or WKB_AREA_NONE [only if
    INDCRYPT].
IndCrypt.pbSrc
    points to the source which contains the data to be indirectly encrypted
    or skipped (if a gap map if specified) [only if INDCRYPT].
IndCrypt.cbDest
    contains the number of bytes which is stored in the destination,
    optionally with skipped gap-map-controlled data [only if INDCRYPT].

Comments
* In a direct WkbCrypt2() is executed via WkNet, <IndSelect.uType> is set
  to 0.
* If an indirect WkbSelect2() is executed via WkNet, <IndSelect.uType>
  contains the indirect selection algorithm variant and <IndCrypt.pwkbar>
  is set to NULL.
* If an indirect WkbCrypt2() with WKB_AREA_SELECT is executed via WkNet,
  <IndSelect.uType> contains the indirect selection algorithm variant and
  <IndCrypt.pwkbar> is not NULL.

*****************************************************************************/

typedef struct _ASYNCACTION {
 // prefix "asac"
  struct _ASYNCACTION *pNext;
  struct _ASYNCACTION **ppPrev;
  SYSENTRY *psys;
  INT idAction;
  INT idState;
  INT idError;
  INT idReturn;
  HWKBSYSTEM hwkbsysDestroy;
} ASYNCACTION;

// contants for state of an action
enum {
  ACT_NONE = 0,
  ACT_RUN,
  ACT_COMPLETE
};

ASYNCACTION * WkdCreateAction(void);
void WkdRemoveAction(ASYNCACTION *pasac);
void WkdRemoveAllActionsOfSubSystem(const SYSENTRY *psys);
ASYNCACTION * WkdFindAction(INT idAction);
BOOL WkdFindCallerAction(INT idAction);

// special code for <cstWkd.idError> to wait for end of asynchronous operation
#define ASYNCWAIT (-1)

#endif // #if ASYNCMODE

/*****************************************************************************

---- API Management support functions ----------------------------------------

*****************************************************************************/


// special values for Expiration Dates
#define NO_EXPDATE  UINT_MAX
#define UNDET_EXPDATE (UINT_MAX - 1)

void WkdClose(HWKBENTRY hwkbe);
void WkdUnSelectEntry(void);
UINT WkdStoreCryptData(BYTE *pbDest, const BYTE *pbSrc, UINT cbSrc,
  UINT cbDest, UINT fuAreaCtrl, const BYTE *pbGapTable, INT idError);

WKBBOXDATA *WkdGetBoxContents(const WKBSERIAL *pwkbsr,
  UINT uEntryMask, UINT *pcbData);
void WkdSetMultiFindOptions(const WKBBOXDATA *pwkbbxd,
  OPENENTRY *pope);
BOOL WkdGetOpenBoxEntry(const WKBSERIAL *pwkbsr, INT iEntry,
  WKBBOXENTRY *pwkbbxeDest, INT *pnVersion, BOOL8 fIsFC0Allowed);


/*****************************************************************************

---- Driver Information Flow Management --------------------------------------

*****************************************************************************/

/*****************************************************************************
CALLSTATUS                                     Status of current function call
==============================================================================

This structure contains the status of the current executed function state.
This state is currently stored in a global variable (cstWkd), but for a later
multithreaded version of the driver, the status could be stored also in an
own dynamic created data object.

Members
-------
fuSysType
    contains the type of the system access. This is one of the
    STYPE_... constants (STYPE_LOCAL, STYPE_WKLAN, STYPE_WKNET, STYPE_KERNEL).
idError
    contains the error code of the current call. This value is initially set
    to WKB_NO_ERROR.
flCtrl
    contains the function control flags in which following flags are valid:
      * WKB_LEVELMASK ... contains the used function level
      * WKB_UNICODE ..... use/create Unicode instead of ASCII strings
                          always set to 0 (caller's copy stored in
                          <fUniCodeCall>).
      * WKB_ASYNCHRONOUS  execute the function asynchronously
      * WKB_USEOFFSETS .. create offsets in structures instead addresses
flCtrlDefMask
    contains the mask value of <flCtrl> flags which are set by <flCtrlDefSet>.
    The default value is 0.
flCtrlDefSet
    contains the set <flCtrl> flags within the <flCtrlDefMask>.
nSelPort
    contains the last selected port number which was set by WkdAccessPort().
    This state is also saved when this port is released and no longer
    accessed. If this value is -1, no port is selected.
iSelPortGroup
    contains the index of the last selected port group which was set by the
    WkdAccessPortGroup(). If this value is -1, no port group is selected.
pprtgrpSel
    points to the selected port group which corresponds with the index in
    <iSelPortGroup> and the port index in <nSelPort>. This pointer is only
    valid if <iSelPortGroup> is not -1.
pclsdrvSel
    points to the class driver of the current selected port group. This
    pointer is only valid if <iSelPortGroup> is not -1.
fAccessed
    is TRUE if the port is accessed and FALSE if not.
pasacThis
    is only valid if WKB_ASYNCHRONOUS is set in <flCtrl> and points to the
    current asynchronous execution object. This member is only defined if the
    ASYNCMODE preprocessor variable is TRUE.
hwkbsys
    contains the used handle for addressing the subsystem. This parameter
    is normally identical with the specified system handle except if a
    local subsystem is addressed. In this case, the HWKB_LOCAL constant
    is stored.
psys
    points to the current accessed system object.
bLevel
    contains the API function calling level (0 or 1).
fUnicodeCall
    contains TRUE if WKB_UNICODE is set in <flCtrl> and FALSE if not. Outside
    of the WKWIN[32].DLL, this flag contains always FALSE.

    contains TRUE if WKB_ASYNCHRONOUS is set in <flCtrl> and FALSE if not.
    This member is only defined if the ASYNCMODE preprocessor variable
    is TRUE.
fNoMsg
    is TRUE if output messages are suppressed and FALSE if not.
prmtsys
    points to the remote system specification if it exists.
prmte
    points to the remote open WibuBox entry object if it exists.
pnetsys
    points to the current accessed WkNet system. Only valid if
    STYPE_WKNET is valid.
pope
    points to the current accessed open WibuBox entry object if it exists.
pice
    points to the entry state of an indirect encryption. This pointer points
    always to the INDCRYPTENTRY structure of the used open entry. This
    structure may stored in a OPENENTRY or in a RMTENTRY structure. This
    member is only defined if the INDCRYPT preprocessor variable is TRUE.
idProcess
    contains in WIBUKEY.SYS and WIBUKEY.VXD the process/file identification
    which accesses currently the kernel driver.

*****************************************************************************/

typedef struct {
  // prefix "cst"
  INT fuSysType;
  INT idError;
  ULONG flCtrl;
  ULONG flCtrlDefMask;
  ULONG flCtrlDefSet;
  ULONG flGlobalInternal;
  INT nSelPort;
  INT iSelPortGroup;
  PORTGROUP *pprtgrpSel;
  CLASSDRIVER *pclsdrvSel;
  BOOLEAN fAccessed;
#if ASYNCMODE
  ASYNCACTION *pasacThis;
#endif
  HWKBSYSTEM hwkbsys;
  SYSENTRY *psys;
  BYTE bLevel;
  BOOLEAN fUnicodeCall;
#if ASYNCMODE
  BOOLEAN fAsyncCall;
#endif
  OPENENTRY *pope;
} CALLSTATUS;

// constances for flGlobalInternal flag of the CALLSTATUS structure
#define DRVC_FC0_ALLOWED    0x01
#define DRVC_EVERYSERIAL    0x02

// current function call status
extern CALLSTATUS cstWkd;


// ====================================
// WibuKey API entry object management
// ====================================



// standard subsystem for HWKB_LOCAL
extern SYSENTRY gsysWkdLocal;

// last returned error code
extern INT gidWkdLastError;

void WkdResetStatus (void);
BOOL WkdGetCtrlFlags (ULONG flCtrl);
BOOL8 WkdCheckMachine (void);
BOOL WkdGetSystem (HWKBSYSTEM hwkbsys, ULONG flCtrl);
BOOL WkdGetEntrySystem (HWKBENTRY hwkbe, ULONG flCtrl);
void WkdSetFirstError(INT idError);

// ===================================
// WibuKey API exit object management
// ===================================

INT WkdReturn(INT idReturn);
INT WkdReturnBool(void);
INT WkdReturnChecked(INT idReturn);
INT WkdReturnLength(INT cbReturn);

// ==============================
// WibuKey API support functions
// ==============================

INT WkdGetApiStringLength(const char *psz,
  UINT cchMax, BOOL fUnicode);
void WkdGetApiString(char *pszDest, const char *pszSrc,
  BOOL fUnicode);
void WkdSetApiString(char *pszDest, const char *pszSrc, INT cchSrc,
  BOOL fUnicode);
const void * WkdGetStructAddr(const void *pvStruct,
  const void *pvEntry);
BOOL WkdSetStructString(void *pvStruct, UINT cbStruct,
  void *pvEntry, UINT *puOffset, const char *psz);
INT WkdStoreEnumResult(const char *pszItemList,
  char *pszDest, UINT cchDest);

#if 0 // BSRDEBUG // set to BSRDEBUG only for tests!
  #undef BSR_BIG_ENDIAN
  #define BSR_BIG_ENDIAN 1
#endif


#define DRVC_IDS_VERINFO_TEXT1 508
#define DRVC_IDS_VERINFO_TEXT2 509
#define DRVC_IDS_VERINFO_TEXT3 510
#define DRVC_IDS_VERINFO_TEXT4 511
#define DRVC_IDS_VERINFO_TEXT5 512

/*****************************************************************************

---- WibuKey Subsystem Management -------------------------------------------

*****************************************************************************/


// handle to access keep-memory objects, "hkm"
DECLARE_HANDLE(HKEEPMEM);


// define the functions for all other operating systems
#define SetCommonMem(x)      (x)
#define GetCommonMem(x)      (x)
#define HndlMgmtAllocMem(x)  BsrAllocMem(x)
#define HndlMgmtFreeMem(x)   BsrFreeMem(x)


void WkdDestroyConnection (RMTSYSTEM *prmtsys);
BOOL8 WkdCheckBoxVersion (ULONG *pflCtrl, USHORT usBoxVersion, WKBSERIAL *pwkbser);
UINT BSRAPI BsrGetCurrentDateCount(void);
UINT32 WkdGetBoxVersion (const WKBSERIAL *pwkbsr);
HANDLE WkdGetLocalSysHandle (HANDLE hwkbsys);
BOOL WkdRestoreConnection (RMTSYSTEM *prmtsys);
int BsrConvertDateCount(UINT uDate, SYSTEMTIME *pst);

#if USE_KEEPMEM
BOOL WkdDiscardKeepMem(HKEEPMEM hkm);
void *WkdGetKeepMem (HKEEPMEM hkm);
#endif 


// status variable "invalid LAN profile"
extern BOOL gfWkdLanProfileRead;

void WkdSetWKBSERIAL(WKBSERIAL *pwkbsr);
void WkdGetWKBBOXLIST(WKBBOXLIST *pwkbbxl, UINT cbDest);
void WkdGetWKBBOXDATA(WKBBOXDATA *pwkbbxd);

// boolean macros
#define IS_TRUE(d) ((d) != 0)


void WkdSetWKBPROGRAM(WKBPROGRAM *pwkbpgm);
void WkdSetWKBBOXCONFIG(WKBBOXCONFIG *pwkbbxc);
void WkdSetWKBEXTMEM(WKBEXTMEM *pwkbxmw);
void WkdSetWKBAREA(WKBAREA *pwkbar);
void WkdSetWKBOPENENTRY(WKBOPENENTRY *pwkboe);
BOOL8 AccessWkUSBDriver();
BOOL8 ReleaseWkUSBDriver();
void *WkdAllocKeepMem (UINT cbData, ULONG ulTimeOut, HKEEPMEM *phDest);
BOOL8 ControlWkUSBDriver (UINT idCtrlCode, void *pvInput,
    UINT cbInput, void *pvOutput, UINT cbOutput);
HWKBSYSTEM WkdAccessLanSystem (const WKBACCESS *pwkbacc, ULONG flCtrl);
const char* WkdQueryServers (BOOL fLanValid, BOOL fAddr);
void WkdGetChipInfo (const WKBSERIAL *pwkbsr, 
    USHORT *pusAsic, USHORT *pusExtendedMem);
UINT WkdGetLANSystemInfo(BOOL fUpdate, WKBSYSTEM *pwkbsysDest,
  UINT cbDest);
const char * WkdGetServerAddress(const RMTSYSTEM *prmtsys);
INT SizeofEnvVarArray(void);
INT SetEnvVarArrayEntry(INT id, const char *pszString);


#endif //#ifndef PROJ_H

// ==================================
// end of proj.h
// ==================================
