/*****************************************************************************
D r v C                                 Portable WibuKey Driver, Written in C
==============================================================================
usblinux.h                                                         header file
==============================================================================

This file contains all declarations which are used by the C files of the
project. It does not contain any resource specific declarations.

==============================================================================

The portable WibuKey driver source is used under the following systems:
 - WKDOS2(d).LIB       - for internal use to link to DOS programs.
 - WKWIN.DLL           - 16 Bit calling driver on 32 Bit Windows systems.
 - WKWIN32.DLL         - 32 Bit Windows calling driver.
 - WIBUKEY.SYS         - NT based kernel driver.
 - WIBUKEY.VXD         - W9x based kernel driver.
 - WkMAC.LIB           - MacOS 9 driver (Shared Lib).
 - WkMacLibX.Framework - MacOS X driver.
 - LibWkLin.so         - Linux and other Un!xes (Shared Object).
 
All sources are written in ANSI C, except the using of C++ style
single line comments.

==============================================================================

Copyright  2000 - 2010 by WIBU-SYSTEMS AG
           Rueppurrer Strasse 52, D-76137 Karlsruhe, Germany/Europe
           Website: http://www.wibu.com, Email: info@wibu.com
           Phone: +49-721-93172-0, Fax: +49-721-93172-22
           All rights reserved.

Version .. 4.31 of 2010-Jun-24
Project .. libwklin.so
Author ... Markus Neumann (mn), Heiko Abraham (ha)
System ... Linux or Un!x
Tools .... GNU gcc

Change Report
01Aug07 3.10 mn:   initial version
06Jan24 5.10 ha:   CodeMerge
08Jul23 5.11 ha:   add ch9 support
10Jun24 4.31 ha:   build update

*****************************************************************************/

#if defined(__linux__)

#ifndef USBLINUX_H
#define USBLINUX_H


#if defined(__GNUC__)
  // We now use GCC 3.3.1 / 3.3.3 / 3.3.4 / 3.3.5 / 4.x
  // #define HAVE_GCC_3X 1

  // We now use GCC 3.0 (debian 3.0)
  // #define HAVE_GCC_3X 0
  #if __GNUC__ > 2
    #if ( __GNUC__ > 3 ) || (__GNUC__ == 3 && (__GNUC_MINOR__ > 2 ) )
      // use gcc-3.3.x or gcc-4.x
      #define HAVE_GCC_3X 1
      // for SuSE 9.3 kernel 
      #ifndef __le16
      #define __le16 __u16
      #endif
    #else
      // use gcc 2.9 or gcc-3.0
      #define HAVE_GCC_3X 0
    #endif
  #else
    #warning "__GNUC__ not valid defined"
    #define HAVE_GCC_3X 0
  #endif // __GNUC__
#endif

/* USB constants: */
#include <linux/version.h>
#include <linux/types.h>



#if LINUX_VERSION_CODE > KERNEL_VERSION(2,6,20)
  // add for kernel > 2.6.20
  #include <linux/usb/ch9.h>
#elif (LINUX_VERSION_CODE < KERNEL_VERSION(2,4,0)) || defined(_CRIS)
  // add for kernel 2.4.x
  #include <linux/usb.h>
#else
  // add for kernel =< 2.6.20
  #include <linux/usb_ch9.h>
#endif

/* Prototype for struct usbdevfs_ctrltransfer: */
#include <linux/usbdevice_fs.h>


#include <linux/version.h>
#if LINUX_VERSION_CODE < KERNEL_VERSION(2,2,16)
  #error "This stuff needs a Linux Kernelversion 2.2.16 or newer"
#else
  #define KERNELVERSION  LINUX_VERSION_CODE

  #ifdef USB_DT_DEVICE_SIZE
    #undef USB_DT_DEVICE_SIZE
  #endif
  #define USB_DT_DEVICE_SIZE sizeof(struct usb_device_descriptor)

  #include <syslog.h>

  #if !defined(_GNU_SOURCE)
    #define _GNU_SOURCE
  #endif
  
  #include <unistd.h>
  #include <errno.h>
  #include <fcntl.h>
  #include <signal.h>
  #include <stdlib.h>
  #include <stdio.h>
  #include <string.h>
  #include <sys/types.h>
  #include <sys/stat.h>
  #include <sys/ioctl.h>
  #include <sys/poll.h>
  #include <time.h>

  #define WIBU_WRITE 0x40
  #define WIBU_READ  0xc0

  /* WIBU3504 Command Codes */
  /* for bRequest (WibuCommand->request) values */
  #define WIBU_RESET    0
  #define WIBU_FIND     1
  #define WIBU_SELECT_1 2
  #define WIBU_CRYPT    3
  #define WIBU_FSEARCH  4
  #define WIBU_NSEARCH  5
  #define WIBU_LIST     6
  #define WIBU_PROGRAM  7
  #define WIBU_SELECT_2 8




/* see /usr/include/linux/udf_167.h */
#ifdef __KERNEL__
  #include <linux/types.h>
  #define UInt8     __u8
  #define UInt16    __u16
  #define UInt32    __u32
#else
  #define UInt8     unsigned char
  #define UInt16    unsigned short
  #define UInt32    unsigned int
#endif // __KERNEL__

#define IOReturn	int

#if !HAVE_GCC_3X
#if !defined (MN_LINUX_USB_NOT_USED)
struct usb_device_descriptor {
    UInt8  bLength;
    UInt8  bDescriptorType;
    UInt8  bcdUSB[2];
    UInt8  bDeviceClass;
    UInt8  bDeviceSubClass;
    UInt8  bDeviceProtocol;
    UInt8  bMaxPacketSize0;
    UInt8  idVendor[2];
    UInt8  idProduct[2];
    UInt8  bcdDevice[2];
    UInt8  iManufacturer;
    UInt8  iProduct;
    UInt8  iSerialNumber;
    UInt8  bNumConfigurations;
};
#endif // !defined (MN_LINUX_USB_NOT_USED)
#endif //HAVE_GCC_3X

struct usbdevice {
  int fd;
  BOOL fIsStillOpen;
  struct usb_device_descriptor desc;
};

struct _BOXLIST;
//from /usr/include/linux/usbdevice_fs.h:
typedef struct usbdevfs_ctrltransfer IOUSBDevRequest;
typedef struct _BOXLIST BOXLIST;



void print_DevClassString(FILE *pFile, struct usb_device_descriptor *pUSBDevDescr);


void wibu_usb_close(struct usbdevice *dev);


struct usbdevice *wibu_usb_open(BOXLIST *pbox, BOOL fLeaveOpen);

struct usbdevice *wibu_usb_open_bynumber(BOXLIST *pbox, unsigned int busnum, unsigned int devnum, int productid, BOOL fLeaveOpen);//N:mn

struct usbdevice *wibu_proc_usb_devices_open(UINT timeout);

int wibu_control_msg(struct usbdevice*, IOUSBDevRequest*);

int  wibu_reset2(struct usbdevice *dev, UINT timeout);



typedef IOReturn (*PFIOR_pV)             (void*);
typedef IOReturn (*PFIOR_pV_pI16)        (void*,UInt16*);
typedef IOReturn (*PFIOR_pV_pUDR)        (void*,IOUSBDevRequest*);
typedef IOReturn (*PFIOR_pUD_UI)         (struct usbdevice*,UINT);
typedef IOReturn (*PFIOR_pUD_US_pV_UI)   (struct usbdevice*,USHORT,void*,UINT);
typedef IOReturn (*PFIOR_pUD_US_US_pV_UI)(struct usbdevice*,USHORT,USHORT,void*,UINT);
typedef IOReturn (*PFIOR_pUD_pUDR)       (struct usbdevice*,IOUSBDevRequest*);
typedef IOReturn (*PFIOR_pUD_pURB)       (struct usbdevice*,struct usbdevfs_urb*);
typedef void     (*PFV_pFD_pUDD)         (FILE*, struct usb_device_descriptor*);
typedef void     (*PFV_pUD)              (struct usbdevice*);


struct IOUSBDeviceStruct;
struct IOUSBDevRequest;
static IOReturn USBDeviceOpen(void*);
static IOReturn USBDeviceClose(void*);
static IOReturn GetDeviceVendor(void*,UInt16*);
static IOReturn GetDeviceProduct(void*,UInt16*);
static IOReturn DeviceRequest(void*,IOUSBDevRequest*);


typedef struct IOUSBDeviceStruct {
    struct IOUSBDeviceStruct *pvThis;
    UINT uBusNum;
    UINT uDevNum;
    struct usbdevice *pDev;
    PFIOR_pV Release;
    PFIOR_pV USBDeviceOpen;
    PFIOR_pV USBDeviceClose;
    PFIOR_pV_pI16 GetDeviceVendor;
    PFIOR_pV_pI16 GetDeviceProduct;
    PFIOR_pV_pUDR DeviceRequest;
} IOUSBDeviceInterface;


inline static IOReturn Release(void *self)
{
    IOUSBDeviceInterface **ppDev = self;

    if ((*ppDev)->pDev)
      if (((*ppDev)->pDev)->fIsStillOpen == TRUE) 
           (*ppDev)->USBDeviceClose(ppDev);
    free(ppDev);
    ppDev = NULL;

    return (1);
} 


inline static IOReturn USBDeviceOpen(void *self)
{
  IOUSBDeviceInterface **ppDev = self;
  struct usbdevice *dev;

  if ((*ppDev)->pDev == NULL)
      return (0);

  dev = (*ppDev)->pDev;
  if(dev->fIsStillOpen) {
      return (1);
  } else {
      return (0);
  }
} 


inline static IOReturn USBDeviceClose(void *self)
{
    IOUSBDeviceInterface **ppDev = (IOUSBDeviceInterface **)self;
    if ((*ppDev)->pDev != NULL)
        wibu_usb_close((*ppDev)->pDev); 
    return (1);
} //USBDeviceClose()


inline static IOReturn GetDeviceVendor(void *self, UInt16 *devVendor)
{
    IOUSBDeviceInterface **ppDev = self;
    if(((*ppDev)->pDev) == NULL){
        devVendor = 0;
        return (0);
    } else {
#if !HAVE_GCC_3X
        *devVendor = (UInt16)(*ppDev)->pDev->desc.idVendor[0] |
                            ((*ppDev)->pDev->desc.idVendor[1] << 8);
#else
        *devVendor = (UInt16)(*ppDev)->pDev->desc.idVendor;
#endif //!HAVE_GCC_3X
        return (1);
    }
}//GetDeviceVendor()


inline static IOReturn GetDeviceProduct(void *self, UInt16 *devProduct)
{
    IOUSBDeviceInterface **ppDev = self;
    if(((*ppDev)->pDev) == NULL){
        devProduct = 0;
        return (0);
    } else {
#if !HAVE_GCC_3X
        *devProduct = (UInt16)(*ppDev)->pDev->desc.idProduct[0] |
                             ((*ppDev)->pDev->desc.idProduct[1] << 8);
#else
        *devProduct = (UInt16)(*ppDev)->pDev->desc.idProduct;
#endif //!HAVE_GCC_3X
        return (1);
    }
}//GetDeviceProduct()




/*****************************************************************************
DeviceRequest
==============================================================================

Send a BoxRequest command to usb-wibubox. 

Parameters
  self     ??
  req      the box-request command as usbdevfs_ctrltransfer-struct

Return
  Return 0 (zero) or a non negativ value, if successfull. On some USB-Controllers
  we return the number of transfered bytes.
  On error, -1 is returned. (See errno for more details).

*****************************************************************************/
inline static int DeviceRequest(void *self, IOUSBDevRequest *req)
{
  IOUSBDeviceInterface **ppDev = (IOUSBDeviceInterface **)self;
  if ( (ppDev == NULL) || ( (*ppDev)->pDev == NULL) ) {
      return (0);
  } else {
      return wibu_control_msg((*ppDev)->pDev, req);
  }
}//DeviceRequest()


inline static void setUSBDevIf(IOUSBDeviceInterface *pIoUsbDevIf)
{
  if (pIoUsbDevIf == NULL)
      return;
  *pIoUsbDevIf = (IOUSBDeviceInterface) {
      NULL,
      0,
      0,
      NULL,
      &Release,
      &USBDeviceOpen,
      &USBDeviceClose,
      &GetDeviceVendor,
      &GetDeviceProduct,
      &DeviceRequest
  };
}//setUSBDevIf()


struct _BOXLIST {
  struct _BOXLIST *pNext;
  struct _BOXLIST *pPrev;
  IOUSBDeviceInterface **ppDev;
  WKBSERIAL wkbsr;
  BOOL8 fBoxAlreadyInList;
};

#endif /* #if LINUX_VERSION_CODE < KERNEL_VERSION(2,2,16) */


int WkhUsbUseSysFs();
char *WkhUsbGetDeviceMountPoint();

#endif //#ifndef USBLINUX_H

#endif //#if _UNIX
